/* eslint-disable @next/next/no-sync-scripts */
/* eslint-disable @next/next/no-page-custom-font */
/* eslint-disable no-console */
import "@/styles/global.css";

import { GoogleAnalytics } from "@next/third-parties/google";
import { cookies } from "next/headers";

import { DomainStateEnum } from "@/@types/enums/domain-state-enum";
import { Error } from "@/app/error";
import Footer from "@/components/shared/footer/Footer";
import FooterB2C from "@/components/shared/footer/Footer-b2c";
import FooterOtel from "@/components/shared/footer/Footer-otel";
import Header from "@/components/shared/header/Header";
import Loader from "@/components/shared/loader/Loader";
import NotifyDialogControl from "@/components/shared/notify-dialog/notify-dialog-control";
import Overlay from "@/components/shared/overlay/overlay";
import type { StrategyType } from "@/hooks/useApp";
import MainContent from "@/layouts/main-layout/main-content";
import { getGoogleAnalyticsAddon } from "@/libs/helpers/addon-helper";
import type { IClientConfiguration } from "@/models/client-configs.model";
import CoreProvider from "@/providers/core-provider/core-provider";
import { ErrorBoundary } from "@/providers/error-boundary/error-boundary";
import { InitializeProvider } from "@/providers/initialize-provider/initialize-provider";
import { LocalizationProvider } from "@/providers/localization-provider/localization-provider";
import QueryProvider from "@/providers/query-provider/QueryProvider";
import { ServerDataProvider } from "@/providers/server-data-provider/server-data-provider";
import StoreProvider from "@/providers/store-provider/StoreProvider";
import ThemeProvider from "@/providers/theme-provider/ThemeProvider";
import { Preload } from "@/server/preload/preload";
import type { StrategyKeysTypes } from "@/services/app-api/apis/strategy.services";
import { Main } from "@/templates/Main";

export { metadata } from "@/data/app-data/metadata";

interface AppMetaData {
  title: string;
}

const appMetaData: Record<StrategyType, AppMetaData> = {
  B2C: {
    title: "Tatil | Size Uygun tatil Seçenekleri",
  },
  Corporate: {
    title: "Jolly Kurumsal Seyahatler",
  },
  Otel: {
    title: "Otel",
  },
};

export default async function StaticRoutesLayout({
  children,
}: {
  children: React.ReactNode;
}) {
  const baseUrl = await Preload.getBaseUrl();
  let clientToken = await Preload.getClientToken(baseUrl.origin);

  if (!clientToken) {
    return <Error message="Token" />;
  }

  let clientConfig: IClientConfiguration | null =
    await Preload.getClientConfigs(baseUrl.origin, clientToken);

  if (!clientConfig) {
    await Preload.clearClientToken(baseUrl.origin);
    clientToken = await Preload.getClientToken(baseUrl.origin, true);
    clientConfig = await Preload.getClientConfigs(baseUrl.origin, clientToken);
    if (!clientConfig) {
      return <Error message="Config" />;
    }
  }

  if (clientConfig.domainState === DomainStateEnum.Closed) {
    return <Error message="Domain Closed" />;
  }

  const strategyKeys = await Preload.getStrategyKeys(
    baseUrl.origin,
    clientToken,
  );

  if (!strategyKeys) {
    return <Error message="strategyKeys" clientConfig={clientConfig} />;
  }

  const appAddons = await Preload.getDomainAddons(baseUrl.origin, clientToken);
  if (!appAddons) {
    return <Error message="Addons" clientConfig={clientConfig} />;
  }

  const GaAddonWithKey = getGoogleAnalyticsAddon(appAddons);

  const authCookie = cookies().get("auth");
  const authTokenInfo = authCookie ? JSON.parse(authCookie.value) : null;

  const languages = cookies().get("lng");
  const currency = cookies().get("currency");

  const appData = {
    strategy: clientConfig.domainType,
  };

  const documentTitle =
    appData.strategy === "Otel"
      ? clientConfig.clientScope
      : appMetaData[clientConfig.domainType as StrategyType]?.title;

  return (
    <html lang={languages?.value ?? "tr"}>
      <head>
        <title>{documentTitle}</title>
        <link rel="shortcut icon" href={clientConfig.favIconUrl} />
        <link
          href="https://fonts.googleapis.com/css2?family=Bai+Jamjuree:wght@500;600&family=Bitter:wght@500;600&family=Chakra+Petch:wght@500;600&family=Comfortaa:wght@500;600&family=Exo+2:wght@500;600&family=IBM+Plex+Sans:wght@500;600&family=Inter:wght@300;400;500;600;700&family=Maven+Pro:wght@500;600&family=Montserrat:wght@500;600&family=Nunito:wght@500;600&family=Quicksand:wght@500;600&display=swap"
          rel="stylesheet"
        />
        {baseUrl.host.includes("turkishairlines.jollytur.com") && (
          <script src="https://bundles.efilli.com/turkishairlines.jollytur.com.prod.js" />
        )}
        {baseUrl.host.includes("turkishairlines-strategy.gordios.ws") && (
          <script src="https://bundles.efilli.com/turkishairlines.gordios.ws.prod.js" />
        )}
        {baseUrl.host.includes("turkishairlines.gordios.ws") && (
          <script src="https://bundles.efilli.com/turkishairlines.gordios.ws.prod.js" />
        )}
        {appData.strategy !== "Corporate" && GaAddonWithKey && (
          <GoogleAnalytics gaId={GaAddonWithKey.key} />
        )}
      </head>
      <body>
        <ErrorBoundary>
          <ServerDataProvider
            clientToken={clientToken}
            strategyKeys={strategyKeys as unknown as StrategyKeysTypes}
            clientConfig={clientConfig}
            addonList={appAddons}
          >
            <StoreProvider
              authTokenInfo={authTokenInfo}
              localizationData={{
                language: languages?.value ?? "tr",
                currency: currency?.value ?? "TRY",
              }}
              appData={appData}
            >
              <QueryProvider>
                <LocalizationProvider lang={languages?.value ?? "tr"}>
                  <ThemeProvider>
                    <InitializeProvider>
                      <CoreProvider>
                        <Main>
                          <div>
                            <Overlay />
                            <Loader />
                          </div>
                          <Header />
                          <MainContent>{children}</MainContent>
                          <NotifyDialogControl />
                          {appData.strategy === "B2C" && <FooterB2C />}
                          {appData.strategy === "Corporate" && <Footer />}
                          {appData.strategy === "Otel" && <FooterOtel />}
                        </Main>
                      </CoreProvider>
                    </InitializeProvider>
                  </ThemeProvider>
                </LocalizationProvider>
              </QueryProvider>
            </StoreProvider>
          </ServerDataProvider>
        </ErrorBoundary>
      </body>
    </html>
  );
}
