/* eslint-disable react-hooks/rules-of-hooks */
import type { ForwardedRef, RefObject } from "react";
import { forwardRef, useEffect, useImperativeHandle } from "react";
import { useFormContext } from "react-hook-form";

import {
  Select,
  SelectContent,
  SelectItem,
  SelectTrigger,
  SelectValue,
} from "@/components/ui/select";
import { cn } from "@/libs/utils";
import { useCitiesMutation } from "@/services/white-label/location/cities.services";

import type { FormItemProps } from "../../form-item";
import FormItem from "../../form-item";

export interface FormCitiesHandles {
  getData: (countryId: number) => void;
  removeValue: () => void;
}

interface FormCitiesSelectProps extends Omit<FormItemProps, "children"> {
  placeholder?: string;
}

const FormCitiesSelect = (
  props: FormCitiesSelectProps,
  ref: ForwardedRef<FormCitiesHandles>,
) => {
  const methods = useFormContext();
  const citiesMutation = useCitiesMutation();

  useEffect(() => {
    if (!methods.getValues(`${props.name}`)) {
      methods.setValue(`${props.name}`, "");
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  useImperativeHandle(ref, () => ({
    getData: (countryId: number) => citiesMutation.mutate(countryId),
    removeValue: () => methods.setValue(props.name, null),
  }));

  return (
    <FormItem name={props.name} rules={props.rules}>
      {({ field, setValue, fieldState }) => (
        <div>
          <Select
            value={
              citiesMutation.data?.result.find(
                (x) => x.name === field.value?.name,
              )?.name
            }
            onValueChange={(itemName) => {
              const item = citiesMutation.data?.result.find(
                (x) => x.name === itemName,
              );
              if (item) {
                setValue(item.name);
              }
            }}
            defaultValue={field.value}
          >
            <SelectTrigger
              className={cn(
                "h-[52px] rounded-primary border border-gray-200 bg-white py-6 shadow-xs",
                {
                  "border-error-500": fieldState.error,
                },
              )}
            >
              <SelectValue
                defaultValue={field.value}
                placeholder={props.placeholder}
              />
            </SelectTrigger>
            <SelectContent className="scroll-bar max-h-[200px] text-sm text-gray-500">
              {citiesMutation.data?.result
                .filter((item, index) => {
                  const isExist = citiesMutation.data?.result.findIndex(
                    (x) => x.name === item.name,
                  );
                  return isExist === index;
                })
                .map((item) => (
                  <SelectItem
                    key={`${item.name}-${item.id}`}
                    className="cursor-pointer px-3.5"
                    value={item.name.toString()}
                  >
                    {item.name}
                  </SelectItem>
                ))}
            </SelectContent>
          </Select>
          {fieldState.error && (
            <div className="mt-1">
              <span className="text-sm text-error-500">
                {fieldState.error?.message}
              </span>
            </div>
          )}
        </div>
      )}
    </FormItem>
  );
};

export const useFormCitiesSelect = (
  ref: RefObject<FormCitiesHandles>,
  countryPrefix: string,
) => {
  const methods = useFormContext();
  const countryd = methods.watch(`${countryPrefix}-country`)?.id;

  const initFormCitiesSelect = () => {
    if (countryd) {
      ref?.current?.getData(countryd);
    }
  };

  const getCitiesData = (countryId: number) => {
    // Note: When the country changes, the city value should be reset.
    ref?.current?.removeValue();
    ref?.current?.getData(countryId);
  };

  return { initFormCitiesSelect, getCitiesData };
};

export default forwardRef(FormCitiesSelect);
