import React, { useEffect, useState } from "react";
import { useFormContext } from "react-hook-form";

import {
  Select,
  SelectContent,
  SelectItem,
  SelectTrigger,
  SelectValue,
} from "@/components/ui/select";
import { useTourSummary } from "@/features/modules/tour/summary/hooks/use-tour-summary";
import { cn } from "@/libs/utils";

import type { FormItemProps } from "../../form-item";
import FormItem from "../../form-item";

interface FormDeparturePointsSelectProps
  extends Omit<FormItemProps, "children"> {}

export interface DeparturePoint {
  key: string;
  city: string;
  destination: string;
  name: string;
  time: string;
  isNextDay: boolean;
}

const FormDeparturePointsSelect = (props: FormDeparturePointsSelectProps) => {
  const [selectedCity, setSelectedCity] = useState("");
  const [destinations, setDestinations] = useState<DeparturePoint[]>([]);
  const [cities, setCities] = useState<string[]>([]);
  const { data } = useTourSummary();
  const methods = useFormContext();

  const handleCityChange = (city: string) => {
    setSelectedCity(city);
  };

  useEffect(() => {
    if (data.payload?.departurePoints?.length) {
      setCities([
        ...new Set(data.payload?.departurePoints?.map((point) => point.city)),
      ]);

      setDestinations(
        data.payload?.departurePoints?.filter(
          (point) => point.city === selectedCity,
        ) || [],
      );
    }
  }, [data, selectedCity]);

  useEffect(() => {
    const defaultDestination = methods.getValues(props.name);
    if (defaultDestination) {
      const defaultSelectedCity = data.payload?.departurePoints?.find(
        (point) => point.key === defaultDestination,
      )?.city;
      if (defaultSelectedCity) setSelectedCity(defaultSelectedCity);
    }
  }, [methods, props.name, data.payload.departurePoints]);

  return (
    <FormItem name={props.name} rules={props.rules}>
      {({ field, setValue, fieldState }) => (
        <div className="mt-2 flex flex-col gap-2 md:flex-row">
          <div className="flex w-full flex-col gap-1">
            <Select
              defaultValue={selectedCity}
              value={selectedCity}
              onValueChange={handleCityChange}
            >
              <SelectTrigger
                className={cn(
                  "h-[52px] rounded-primary border border-gray-200 bg-white py-6 shadow-xs",
                  {
                    "border-error-500": fieldState.error,
                  },
                )}
              >
                <SelectValue placeholder="Şehir" />
              </SelectTrigger>
              <SelectContent>
                {[...cities]?.map((item) => (
                  <SelectItem key={item} value={item}>
                    {item}
                  </SelectItem>
                ))}
              </SelectContent>
            </Select>
            {fieldState.error && (
              <div className="mt-1">
                <span className="text-sm text-error-500">
                  {fieldState.error?.message}
                </span>
              </div>
            )}
          </div>
          <div className="flex w-full flex-col gap-1">
            <Select
              defaultValue={field.value}
              value={field.value}
              onValueChange={(destinationKey) => {
                setValue(destinationKey);
              }}
            >
              <SelectTrigger
                className={cn(
                  "h-[52px] rounded-primary border border-gray-200 bg-white py-6 shadow-xs",
                  {
                    "border-error-500": fieldState.error,
                  },
                )}
              >
                <SelectValue placeholder="Durak*" />
              </SelectTrigger>
              <SelectContent>
                {destinations.map((point) => (
                  <SelectItem key={point.key} value={point.key}>
                    {point.name}
                  </SelectItem>
                ))}
              </SelectContent>
            </Select>
            {fieldState.error && (
              <div className="mt-1">
                <span className="text-sm text-error-500">
                  {fieldState.error?.message}
                </span>
              </div>
            )}
          </div>
        </div>
      )}
    </FormItem>
  );
};

export default FormDeparturePointsSelect;
