import { useEffect } from "react";
import { useFormContext, type ValidateResult } from "react-hook-form";

import { Button } from "@/components/ui/button/button";
import { Checkbox } from "@/components/ui/checkbox/checkbox";
import type { InputProps } from "@/components/ui/input/input";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import MaskedInput from "@/components/ui/masked-input/masked-input";
import { useLocale } from "@/hooks/useLocale";
import {
  checkSameTcNoInput,
  checkTcNoInput,
} from "@/validations/form-rules/rules/tc-no-input-rule";

import type { FormItemProps } from "../../form-item";
import FormItem from "../../form-item";

interface FormIdentityOrPassportInputProps
  extends FormItemProps,
    Omit<InputProps, "name" | "onChange"> {
  validate?: (value: string) => ValidateResult | Promise<ValidateResult>;
  showWarning?: boolean;
}

const FormIdentityOrPassport = (props: FormIdentityOrPassportInputProps) => {
  const methods = useFormContext();
  const { t } = useLocale();

  useEffect(() => {
    if (!methods.getValues(`${props.name}Check`)) {
      methods.setValue(`${props.name}Check`, false);
    }

    if (!methods.getValues(`${props.name}`)) {
      methods.setValue(`${props.name}`, "");
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  return (
    <>
      <div>
        <FormItem
          name={props.name}
          rules={{
            ...props.rules,
            validate: (value) => {
              if (props.validate) {
                return props.validate(value);
              }
              const checkValue = methods.getValues(`${props.name}Check`);
              const checkSameTcNoInputResult = checkSameTcNoInput(
                {
                  fieldName: props.name,
                  fieldValue: value,
                  fieldCheckValue: checkValue,
                },
                methods.getValues(),
              );

              if (!checkSameTcNoInputResult) {
                return t("The same {{type}} may not be used more than once.", {
                  type: checkValue
                    ? t("Passport number")
                    : t("TC Identification Number"),
                });
              }

              if (checkValue) {
                return true;
              }

              const checkTcNoInputResult = checkTcNoInput(value);
              if (typeof checkTcNoInputResult === "string") {
                return checkTcNoInputResult;
              }

              return true;
            },
          }}
        >
          {({ field, fieldState, setValue }) => (
            <div>
              {methods.watch(`${props.name}Check`) ? (
                <MaskedInput
                  defaultValue={field.value}
                  onChange={setValue}
                  onBlur={field.onBlur}
                  mask="identity"
                  isInvalid={Boolean(fieldState.error)}
                  errorMessage={fieldState.error?.message}
                  {...props}
                  label={`${t("Passport number")}*`}
                />
              ) : (
                <MaskedInput
                  defaultValue={field.value}
                  onChange={setValue}
                  onBlur={field.onBlur}
                  mask="identity"
                  isInvalid={Boolean(fieldState.error)}
                  errorMessage={fieldState.error?.message}
                  maxLength={11}
                  {...props}
                />
              )}
            </div>
          )}
        </FormItem>
        <FormItem name={`${props.name}Check`}>
          {({ setValue }) => (
            <div className="mt-2 flex select-none items-center justify-start">
              <Checkbox
                size="sm"
                id="checked"
                label=""
                labelClassname="text-xs"
                className="!mr-0 !gap-0"
                checked={methods.watch(`${props.name}Check`)}
                onCheckedChange={(checked) => {
                  setValue(Boolean(checked.valueOf()));
                }}
              />
              <Button
                type="button"
                variant="ghost"
                className="!m-0 !p-0 !text-xs text-gray-800 hover:!bg-transparent"
                onClick={() => {
                  setValue(!methods.getValues(`${props.name}Check`));
                }}
              >
                {t("I am not a Turkish Citizen")}
              </Button>
            </div>
          )}
        </FormItem>
      </div>
      {props.showWarning && methods.watch(`${props.name}Check`) && (
        <div className="col-span-1 flex items-start justify-start gap-4 rounded-primary border border-orange-200 bg-orange-50 p-4 md:col-span-2">
          <div className="flex h-8 w-12 items-center justify-center rounded border border-orange-200 bg-white">
            <LIcon name="User" size={14} className="text-orange-300" />
          </div>
          <div className="text-xs">
            {t(
              "The determined prices are only for T.C. applicable to its citizens. T.R. For our non-citizens, a price difference may be requested at the time of check-in.",
            )}
          </div>
        </div>
      )}
    </>
  );
};

export default FormIdentityOrPassport;
