import { useState } from "react";

import { type InputProps } from "@/components/ui/input/input";
import MaskedInput from "@/components/ui/masked-input/masked-input";
import {
  Select,
  SelectContent,
  SelectItem,
  SelectTrigger,
  SelectValue,
} from "@/components/ui/select";
import { Skeleton } from "@/components/ui/skeleton/skeleton";
import { useCountryCodesQuery } from "@/services/white-label/location/country-codes.services";
import type { CountryCode } from "@/services/white-label/location/location.services.types";

import type { FormItemProps } from "../../form-item";
import FormItem from "../../form-item";

interface FormPhoneInputProps
  extends FormItemProps,
    Omit<InputProps, "name" | "onChange"> {}

const FormPhoneInput = ({ name, rules, ...props }: FormPhoneInputProps) => {
  const countryCodesQuery = useCountryCodesQuery();
  const [searchTerm, setSearchTerm] = useState("");
  if (
    !countryCodesQuery.isSuccess ||
    !countryCodesQuery.data ||
    !countryCodesQuery.data.success ||
    !countryCodesQuery?.data.result
  ) {
    return (
      <div>
        <Skeleton className="h-[54px]" />
      </div>
    );
  }

  const countryCodes = countryCodesQuery?.data.result;

  const findSelectedCountry = (
    countries: CountryCode[],
    countryCode: string,
  ): CountryCode | undefined => {
    return countries.find((item) => item.code === countryCode);
  };

  return (
    <FormItem name={name} rules={rules}>
      {({ field, setValue, fieldState }) => {
        const defaultValue = `${field.value?.areaCode || ""}${
          field.value?.phoneNumber || ""
        }` as unknown as any; // this is any because mask input has a unknown error

        if (!field.value) {
          setTimeout(() => {
            setValue({
              code: findSelectedCountry(countryCodes, "TR")?.code,
              dialCode: findSelectedCountry(countryCodes, "TR")?.dialCode,
              phoneNumber: "",
            });
          }, 500);
        }
        return (
          <div>
            <div className="relative">
              <div className="absolute left-0 top-0 z-50">
                <Select
                  value={`${field.value?.code}`}
                  onValueChange={(value) => {
                    const selectedCountry = countryCodes.find(
                      (item) => item.code === value,
                    );
                    setValue({
                      code: selectedCountry?.code || "",
                      dialCode: selectedCountry?.dialCode || "",
                      phoneNumber: field.value?.phoneNumber ?? "",
                    });
                    setSearchTerm("");
                  }}
                  defaultValue={findSelectedCountry(countryCodes, "TR")?.code}
                >
                  <SelectTrigger className="h-[54px] border-none bg-transparent">
                    <SelectValue placeholder="Ülke/ Uyruk">
                      +{field.value?.dialCode}
                    </SelectValue>
                  </SelectTrigger>
                  <SelectContent
                    className="w-64"
                    searchable
                    onSearch={(e) => {
                      setSearchTerm(e.target.value);
                    }}
                    searchTerm={searchTerm}
                  >
                    <div className="scroll-bar relative h-fit max-h-[280px] scroll-m-20">
                      <div className="text-sm text-gray-500">
                        {countryCodes
                          .reduce((prev, curr) => {
                            const isExist = prev.some(
                              (item) => item.dialCode === curr.dialCode,
                            );
                            if (
                              !isExist &&
                              // Search according to name and dial code
                              (curr.name
                                .toLowerCase()
                                .includes(searchTerm.toLowerCase()) ||
                                curr.dialCode.toString().includes(searchTerm) ||
                                curr.code.includes(field.value.code))
                            ) {
                              prev.push(curr);
                            }
                            return prev;
                          }, [] as CountryCode[])
                          .map((item) => (
                            <SelectItem
                              key={`${item.name}-${item.code}-${item.dialCode}`}
                              className="static cursor-pointer px-3.5"
                              value={item.code}
                            >
                              {`${item.name} +${item.dialCode}`}
                            </SelectItem>
                          ))}
                      </div>
                    </div>
                  </SelectContent>
                </Select>
              </div>
              <MaskedInput
                inputContentClassName="pl-[54px]"
                defaultValue={defaultValue}
                isInvalid={!!fieldState.error?.message}
                onChangeUnMasked={(value) => {
                  setValue({
                    code: field.value?.code,
                    dialCode: field.value?.dialCode,
                    phoneNumber: value,
                  });
                }}
                mask={field.value?.dialCode === "90" ? "phone" : "phoneAbroad"}
                {...props}
              />
            </div>
            {fieldState.error && fieldState.error?.message && (
              <div className="mt-1">
                <span className="line-clamp-[20px] text-sm text-error-500">
                  {fieldState.error?.message}
                </span>
              </div>
            )}
          </div>
        );
      }}
    </FormItem>
  );
};

export default FormPhoneInput;
