"use client";

import React, { useEffect, useRef, useState } from "react";

import { Badge } from "@/components/ui/badge/badge";
import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import { useLocale } from "@/hooks/useLocale";
import { cn } from "@/libs/utils";

export interface AppliedFilterData {
  filterKey: string;
  filterValue: string | number;
  filterLabel: string | number;
}

interface AppliedFilterProps {
  ref?: React.Ref<HTMLDivElement>;
  className?: string;
  data: AppliedFilterData[];
  onRemoveItem?: (item: AppliedFilterData) => void;
  onClearFilter?: () => void;
}

const generateSpecificKey = (item: AppliedFilterData) =>
  `${item.filterKey}-${item.filterValue}`;

function AppliedFilter({
  ref,
  className,
  data,
  onRemoveItem,
  onClearFilter,
}: AppliedFilterProps) {
  const cardRef = useRef<HTMLDivElement>(null);
  const outsideWrapperRef = useRef<HTMLDivElement>(null);
  const insideWrapperRef = useRef<HTMLDivElement>(null);
  const [buttonsVisible, setButtonsVisible] = useState(false);
  const [dataList, setDataList] = useState<AppliedFilterData[]>([]);
  const { t } = useLocale();

  const handleClose = (item: AppliedFilterData) => {
    setDataList(
      dataList.filter(
        (listItem) =>
          generateSpecificKey(listItem) !== generateSpecificKey(item),
      ),
    );
    if (onRemoveItem) onRemoveItem(item);
  };

  const handleClearFilter = () => {
    setDataList([]);
    if (onClearFilter) onClearFilter();
  };

  const checkWrappersWidth = () => {
    const insideWrapper = insideWrapperRef?.current?.getBoundingClientRect();

    const insideChildrens = insideWrapperRef.current?.children;
    let childrenWidths = 0;

    // eslint-disable-next-line no-plusplus
    for (let i = 0; i < (insideChildrens?.length || 0); i++) {
      if (insideChildrens) {
        const elem = insideChildrens[i]?.getBoundingClientRect();
        childrenWidths += elem?.width || 0;
      }
    }

    if (childrenWidths > 0) {
      childrenWidths += (insideChildrens?.length || 0) - 16; // '16 is added to total width count. Because it is items gap value'
    }

    if (childrenWidths >= (insideWrapper?.width || 1)) {
      setButtonsVisible(true);
    } else {
      setButtonsVisible(false);
    }
  };

  const rightScroll = () => {
    if (typeof insideWrapperRef.current?.scrollLeft !== "undefined")
      insideWrapperRef.current.scrollLeft += 50;
  };
  const leftScroll = () => {
    if (typeof insideWrapperRef.current?.scrollLeft !== "undefined")
      insideWrapperRef.current.scrollLeft -= 50;
  };

  useEffect(() => {
    setDataList(data);
  }, [data]);

  useEffect(() => {
    checkWrappersWidth();
    if (cardRef.current) {
      const parentWidth = cardRef.current.parentElement?.clientWidth || 0;
      if (parentWidth > 0) {
        cardRef.current.style.width = `${parentWidth}px`;
      }
    }
  }, [dataList]);

  return (
    dataList.length > 0 && (
      <Card
        ref={ref || cardRef}
        className={cn(
          "flex flex-row items-center justify-stretch gap-4",
          className,
        )}
      >
        {/* Title */}
        <p className="w-[60px] font-semibold leading-normal text-slate-800">
          {t("Filters")}
        </p>
        {/* Main Content Wrapper */}
        <div
          ref={outsideWrapperRef}
          className="flex w-full items-center justify-start gap-1 overflow-hidden"
        >
          {buttonsVisible && (
            <Button
              leftIcon={{
                name: "ArrowLeft",
                className: "h-6 w-6",
              }}
              type="button"
              variant="outline"
              className="max-h-[32px] max-w-[32px] justify-center !p-1 text-gray-500"
              onClick={leftScroll}
            />
          )}
          <div
            ref={insideWrapperRef}
            className="flex items-center gap-2 overflow-x-scroll scroll-smooth scrollbar-none"
          >
            {dataList.map((item) => (
              <Badge
                key={generateSpecificKey(item)}
                variant="primary"
                className="!h-8 whitespace-nowrap !rounded-primary"
                closable
                closeIconPosition="left"
                onClose={() => handleClose(item)}
              >
                {item.filterLabel}
              </Badge>
            ))}
          </div>
          {buttonsVisible && (
            <Button
              leftIcon={{
                name: "ArrowRight",
                className: "h-6 w-6",
              }}
              type="button"
              variant="outline"
              className="col-span-1 max-h-[32px] max-w-[32px] !p-1 text-gray-500"
              onClick={rightScroll}
            />
          )}
        </div>
        {/* Clean filter button */}
        <Button
          variant="primary-text"
          className="w-[52px] text-sm font-medium leading-tight text-blue-500"
          onClick={handleClearFilter}
        >
          {t("Clean")}
        </Button>
      </Card>
    )
  );
}

export const useAppliedFilter = () => {
  const [data, setData] = useState<AppliedFilterData[]>([]);

  const addFilter = (filter: AppliedFilterData) => {
    setData([...data, filter]);
  };

  const removeFilter = (filter: AppliedFilterData) => {
    setData(
      data.filter(
        (item) => generateSpecificKey(item) !== generateSpecificKey(filter),
      ),
    );
  };

  const updateFilterByKey = (
    filterKey: string,
    filterValues: AppliedFilterData[],
  ) => {
    const filteredData = data.filter((item) => item.filterKey !== filterKey);
    setData([...filteredData, ...filterValues]);
  };

  const triggerFilter = (filter: AppliedFilterData) => {
    if (
      data.some(
        (item) => generateSpecificKey(item) === generateSpecificKey(filter),
      )
    ) {
      removeFilter(filter);
    } else {
      addFilter(filter);
    }
  };

  const clearFilter = () => {
    setData([]);
  };

  const replaceFilter = (newFilter: AppliedFilterData) => {
    const filteredData = data.filter(
      (item) => item.filterKey !== newFilter.filterKey,
    );
    setData([...filteredData, newFilter]);
  };

  return {
    data,
    setData,
    addFilter,
    removeFilter,
    clearFilter,
    triggerFilter,
    replaceFilter,
    updateFilterByKey,
  };
};

export default AppliedFilter;
