import type { ChangeEvent, FormEvent, PropsWithChildren } from "react";
import React, { cloneElement, useCallback, useRef, useState } from "react";
import { useUpdateEffect } from "usehooks-ts";

import { Button } from "@/components/ui/button/button";
import { Checkbox } from "@/components/ui/checkbox/checkbox";
import {
  Collapsible,
  CollapsibleContent,
  CollapsibleTrigger,
} from "@/components/ui/collapsible/collapsible";
import { Input } from "@/components/ui/input/input";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import { Tooltip } from "@/components/ui/tooltip/tooltip";
import { useBreakpoints } from "@/hooks/useBreakpoints";
import { useLocale } from "@/hooks/useLocale";
import { cn } from "@/libs/utils";

export interface FilterCheckboxItemType {
  key: string;
  value?: string;
  /**
   * @description this'll should be a string
   */
  label: string;
  labelExtra?: string;
}

interface FilterCheckboxProps {
  isCollapsible?: boolean;
  collapseTitle: string;
  collapseDefaultOpen?: boolean;
  items: FilterCheckboxItemType[];
  searchable?: boolean;
  searchPlaceholder?: string;
  searchValue?: string;
  loading?: boolean;
  initialSelectedItems?: string[];
  onChange?: (selectedItems: string[]) => void;
}

const itemVisibleLimit = 5;
const filterItems = (val: string, items: FilterCheckboxItemType[]) =>
  items.filter((item) => item.label.toLowerCase().includes(val.toLowerCase()));

function CheckboxLabel({
  children,
}: PropsWithChildren<Record<string, unknown>>) {
  const { isDesktop } = useBreakpoints();
  const content = (
    <div className="line-clamp-1 text-ellipsis break-all">{children}</div>
  );
  const returnElement =
    (children?.toLocaleString().length || 0) > 18 && isDesktop
      ? cloneElement(<Tooltip label={children} />, { children: content })
      : content;
  //
  return returnElement;
}

function FilterCheckbox(props: FilterCheckboxProps) {
  const {
    collapseTitle,
    collapseDefaultOpen,
    items,
    searchable,
    searchPlaceholder,
    searchValue,
    initialSelectedItems,
    loading,
    onChange,
    isCollapsible = true,
  } = props;
  const [open, setOpen] = useState(collapseDefaultOpen || false);
  const [inputSearchValue, setInputSearchValue] = useState(searchValue ?? "");
  const [selectedItems, setSelectedItems] = useState<string[]>(
    initialSelectedItems ?? [],
  );
  const wrapperRef = useRef<HTMLDivElement>(null);
  const [visibleItems, setVisibleItems] = useState(
    filterItems(inputSearchValue, items.slice(0, itemVisibleLimit)),
  );
  const { t } = useLocale();

  const [showMore, setShowMore] = useState(
    visibleItems.length > itemVisibleLimit,
  );

  // show more events
  const handleShowMoreOnClick = useCallback(() => {
    if (!showMore) {
      setVisibleItems(filterItems(inputSearchValue, items));
      setShowMore(true);
    } else {
      setShowMore(false);
      setVisibleItems(
        filterItems(
          inputSearchValue,
          items
            .filter((item) => {
              if (inputSearchValue) {
                return item.label
                  .toLowerCase()
                  .includes(inputSearchValue.toLowerCase());
              }
              return true;
            })
            .slice(0, itemVisibleLimit),
        ),
      );
    }
  }, [showMore, inputSearchValue, items]);

  // search events
  const handleOnChange = (e: ChangeEvent<HTMLInputElement>) => {
    setInputSearchValue(e?.target.value);
    setVisibleItems(
      filterItems(e?.target.value, items).slice(
        0,
        showMore ? items.length : itemVisibleLimit,
      ),
    );
  };

  // checkbox events
  const handleOnChangeCheckbox = (e: FormEvent<HTMLButtonElement>) => {
    const { value } = e.currentTarget;
    const isItemExist = selectedItems.includes(value);
    let currentSelectedItems = [] as string[];
    if (isItemExist) {
      currentSelectedItems = selectedItems.filter((item) => item !== value);
    } else {
      currentSelectedItems = [...selectedItems, value];
    }
    if (onChange) {
      onChange(currentSelectedItems);
    }
    setSelectedItems(currentSelectedItems);
  };

  useUpdateEffect(() => {
    setSelectedItems(initialSelectedItems ?? []);
  }, [initialSelectedItems]);

  return (
    <Collapsible open={isCollapsible ? open : true} onOpenChange={setOpen}>
      <CollapsibleTrigger
        className={cn(
          "flex w-full items-center justify-between text-sm font-medium leading-tight text-slate-800",
          (open || !isCollapsible) && "mb-4",
          isCollapsible ? "cursor-pointer" : "cursor-default",
        )}
      >
        {collapseTitle}
        {isCollapsible && <LIcon name={open ? "ChevronUp" : "ChevronDown"} />}
      </CollapsibleTrigger>
      <CollapsibleContent>
        {searchable && (
          <div className="relative">
            <Input
              leftIcon={<LIcon name="Search" />}
              placeholder={searchPlaceholder}
              containerClassName="mb-3"
              className="pr-12"
              value={inputSearchValue}
              onChange={handleOnChange}
            />
            {inputSearchValue.length > 0 && (
              <LIcon
                name="X"
                className="absolute right-4 top-4 size-4 cursor-pointer"
                onClick={() =>
                  handleOnChange({
                    target: { value: "" },
                  } as ChangeEvent<HTMLInputElement>)
                }
              />
            )}
          </div>
        )}
        <div
          ref={wrapperRef}
          className={cn("w-full", "relative", showMore && "scroll-bar")}
          style={{
            // eslint-disable-next-line no-nested-ternary
            height: showMore
              ? visibleItems.length < 13
                ? `${visibleItems.length * 28}px`
                : "350px"
              : "auto",
          }}
        >
          <div className="w-full">
            {loading && (
              <div className="absolute left-0 top-0 flex size-full items-center justify-center bg-gray-500/50">
                <LIcon
                  className="animate-spin text-white"
                  name="Loader2"
                  size={30}
                />
              </div>
            )}
            {loading && (
              <div className="absolute left-0 top-0 flex size-full items-center justify-center bg-gray-500/50">
                <LIcon
                  className="animate-spin text-white"
                  name="Loader2"
                  size={30}
                />
              </div>
            )}
            {visibleItems.map((item) => (
              <div className="my-2" key={item.key}>
                <Checkbox
                  id={item.label}
                  label={
                    <div className="flex items-center justify-start gap-0.5">
                      <CheckboxLabel>{item.label}</CheckboxLabel>
                      <div className="">{item.labelExtra}</div>
                    </div>
                  }
                  value={item.value ?? item.key}
                  checked={selectedItems.includes(item.key)}
                  variant="primary"
                  rounded="md"
                  onClick={handleOnChangeCheckbox}
                  labelClassname="font-medium text-gray-500 my-.5"
                />
              </div>
            ))}
          </div>
        </div>
        {
          // show more button
          (filterItems(inputSearchValue, items).length > itemVisibleLimit ||
            visibleItems.length > itemVisibleLimit) && (
            <Button
              variant="primary-text"
              className="z-10 gap-1 !px-0"
              type="button"
              onClick={handleShowMoreOnClick}
            >
              {showMore ? (
                <>
                  {t("show less")}
                  <LIcon name="ChevronUp" />
                </>
              ) : (
                <>
                  {t("Show more.")} <LIcon name="ChevronDown" />
                </>
              )}
            </Button>
          )
        }
      </CollapsibleContent>
    </Collapsible>
  );
}

export default FilterCheckbox;
