/* eslint-disable no-nested-ternary */
/* eslint-disable no-console */
import dayjs from "dayjs";
import Link from "next/link";
import React from "react";

import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import type { STextProps } from "@/components/ui/status-text/status-text";
import { StatusText } from "@/components/ui/status-text/status-text";
import type { ReactQueryReturnProps } from "@/components/widget/booking-request-card-detail/booking-request-card-detail";
import { JoinItems } from "@/components/widget/booking-request-card-detail/booking-request-card-detail";
import BookingRequest from "@/components/widget/request-card/booking-request-card/booking-request-card";
import FlightMultipleWayCard from "@/components/widget/request-card/flight-request-card/flight-multiple-way-card";
import BookingRequestCardDetailConfirmRejectDeniedButtonList from "@/features/booking-request/components/booking-request-card-detail/booking-request-card-detail-button-list";
import PermGuard from "@/features/permission/components/perm-guard";
import { JsonParserHelper } from "@/libs/helpers/string-to-json-helper";
import { PriceManager } from "@/libs/price-manager/price-manager";
import { cn } from "@/libs/utils";
import type {
  CorpRequestHistories,
  IBookingRequest,
} from "@/models/booking-request.model";
import type { CancelReasonItem } from "@/services/eva-display/book-cancel-reason/booking-cancel-reason-service.types";
import type { ISearchRequest } from "@/services/eva-display/demand/display-information/display-information";
import type { IFlightBookedProduct } from "@/services/eva-display/demand/flight-booked-product/flight-booked-product";
import type { IFlightDisplayInformation } from "@/services/eva-display/demand/flight-display-information/flight-display-information";
import type { IPlaneReservationData } from "@/services/eva-display/demand/flight-reservation-data/flight-reservation-data";
import type { IPassengerInformation } from "@/services/eva-display/demand/passenger-information/passenger-information";
import { requestApprovalAllowPerm } from "@/validations/permissions/request-approval";

import styles from "./flight-request-card-detail.module.css";

interface BookingRequestCardDetailProps {
  id: number;
  className?: string;
  status: STextProps["status"];
  data: IBookingRequest;
  bookedProduct: IFlightBookedProduct;
  displayInformations: IFlightDisplayInformation;
  reservationData: IPlaneReservationData;
  passengerInformations: IPassengerInformation;
  searchRequest: ISearchRequest;
  onAccept: () => void;
  onReject: () => void;
  onDenied: () => void;
  bookingRequestStatusMutation: ReactQueryReturnProps;
  cancelReasonData: CancelReasonItem[];
  setRejectSelectValue: (e: { value: string; name: string }) => void;
  rejectSelectValue: {
    value: string;
    name: string;
  };
  corpRequestHistories: CorpRequestHistories[];
}

export function Undefined() {
  return <small className="text-error-400">bulunamadı</small>;
}

function FlightRequestCardDetail({
  id,
  className,
  status,
  data,
  bookedProduct,
  reservationData,
  displayInformations,
  passengerInformations,
  onAccept,
  onReject,
  onDenied,
  bookingRequestStatusMutation,
  searchRequest,
  cancelReasonData,
  setRejectSelectValue,
  rejectSelectValue,
  corpRequestHistories,
}: BookingRequestCardDetailProps) {
  const pricingCombinations =
    reservationData?.DetailResponse?.PricingCombinations;
  const flightSegments =
    reservationData &&
    pricingCombinations?.length > 0 &&
    pricingCombinations[0] &&
    pricingCombinations[0].CombinationGroups &&
    pricingCombinations[0].CombinationGroups[0] &&
    pricingCombinations[0].CombinationGroups[0].FlightSegments[0] &&
    pricingCombinations[0].CombinationGroups[0].FlightSegments.sort(
      (a, b) =>
        dayjs(a.DepartureTime).valueOf() - dayjs(b.DepartureTime).valueOf(),
    );

  if (!flightSegments) {
    return null;
  }

  // const originSegment = flightSegments[0];
  const arrivalSegment = flightSegments[flightSegments.length - 1];

  const mainCurrency =
    reservationData!.DetailResponse!.PricingCombinations[0]!.TotalPrice.slice(
      0,
      3,
    );
  const totalledPrice =
    reservationData.DetailResponse.PricingCombinations.length > 1
      ? reservationData.DetailResponse.PricingCombinations.reduce(
          (acc, curr) => acc + +curr.TotalPrice.slice(3),
          0,
        )
      : +reservationData.DetailResponse.PricingCombinations[0]!.TotalPrice.slice(
          3,
        );

  const totalPrice = PriceManager({
    currency: mainCurrency,
    value: totalledPrice,
  });

  const errorStatusText = corpRequestHistories?.find(
    (item: CorpRequestHistories) => item.newStatus === data.requestStatus,
  );

  const searchRequestData = JsonParserHelper.jsonParseOrUndefined(
    data.searchRequest,
  );

  return (
    <div className={cn(styles.container, className, "group")}>
      <div className={styles.wrapper}>
        <Link
          href="/booking-request"
          className="mr-auto flex items-center gap-1.5 text-primary-500"
        >
          <LIcon name="ArrowLeft" size={18} />
          <span className="font-base font-medium">Geri Dön</span>
        </Link>
        <h1 className="text-3xl font-semibold">
          {arrivalSegment?.Destination
            ? `${arrivalSegment?.Destination.split(" ")[0]}${
                data?.travelReasonName ? ` ${data?.travelReasonName}` : ""
              } Seyahati`
            : data?.travelReasonName}
          {data.createDate && (
            <p className="mt-2.5 text-base font-normal text-gray-500">
              {dayjs.utc(data.createDate).format("DD MMMM YYYY (HH:mm)")}{" "}
              tarihinde oluşturuldu
            </p>
          )}
        </h1>

        <Card className="p-8 shadow-lg">
          <div className="flex flex-col md:justify-between lg:flex-row lg:items-center">
            <div className="flex flex-col gap-4 md:flex-row md:gap-10">
              <div className="flex flex-col gap-5">
                <div>
                  <p className="mb-2.5 font-medium text-gray-500">
                    Onay Durumu
                  </p>
                  <StatusText
                    status={status}
                    text={`${dayjs
                      .utc(data.updateDate)
                      .format("DD MMMM YYYY (hh:mm)")} tarihinde ${
                      data.updatedByUserName
                        ? `${
                            errorStatusText?.createdByFullName
                              ? errorStatusText?.createdByFullName
                              : data.updatedByUserName
                          } tarafindan`
                        : ``
                    } ${
                      status === "denied"
                        ? "Reddedildi"
                        : status === "error"
                          ? "iptal edildi"
                          : "onaylandı"
                    }.`}
                  />
                </div>
                {status === "denied" && errorStatusText?.description && (
                  <div>
                    <p className="mb-2.5 font-medium text-gray-500">
                      Reddedilme Nedeni
                    </p>
                    <p className="text-base text-gray-800">
                      {errorStatusText?.description}
                    </p>
                  </div>
                )}
              </div>
              {status === "denied" ||
                (status !== "error" &&
                  data.bookedProduct &&
                  data.bookedProduct.OptionDate &&
                  dayjs((data.bookedProduct as any)?.OptionDate).isValid() && (
                    <div className="flex flex-col">
                      <p className="mb-2.5 font-medium text-gray-500">
                        Opsiyon Süresi
                      </p>
                      <div className="mt-0.5 text-sm font-medium text-warning-500">
                        {`${dayjs
                          .utc((data.bookedProduct as any)?.OptionDate)
                          .format("DD.MM.YYYY HH:mm")}'e kadar`}
                      </div>
                    </div>
                  ))}
            </div>
            <BookingRequestCardDetailConfirmRejectDeniedButtonList
              status={status}
              createdByUserId={data.createdByUserId}
              bookingRequestStatusMutation={bookingRequestStatusMutation}
              onAccept={onAccept}
              onReject={onReject}
              onDenied={onDenied}
              cancelReasonData={cancelReasonData}
              setRejectSelectValue={(value) => setRejectSelectValue(value)}
              rejectSelectValue={rejectSelectValue}
            />
          </div>
        </Card>

        <Card className="p-8">
          <h2 className={cn(styles.title, "mb-4 text-lg")}>
            Rezervasyon Bilgileri
          </h2>
          <div className="grid grid-cols-2 gap-x-2 gap-y-3 md:grid-cols-3 lg:grid-cols-5">
            <div className={styles.rowCell}>
              <p className="font-medium text-gray-500">Seyahat Yeri</p>
              <p className="mt-1">
                {arrivalSegment?.Destination ||
                  searchRequest?.DestinationName ||
                  Undefined()}
              </p>
            </div>
            <div className={styles.rowCell}>
              <p className="font-medium text-gray-500">Uçuş Tipi</p>
              <p className="mt-1">
                {(pricingCombinations?.[0]?.CombinationGroups &&
                  pricingCombinations?.[0]?.CombinationGroups?.length > 1) ||
                pricingCombinations?.length > 1
                  ? "Gidiş Dönüş"
                  : "Tek Yön"}
              </p>
            </div>
            <div className={styles.rowCell}>
              <p className="font-medium text-gray-500">
                {pricingCombinations?.[0]?.CombinationGroups &&
                pricingCombinations?.[0]?.CombinationGroups?.length > 1
                  ? "Gidiş-Dönüş Tarihleri"
                  : "Gidiş Tarihi"}
              </p>
              <p className="mt-1">
                {pricingCombinations?.[0]?.CombinationGroups &&
                pricingCombinations?.[0]?.CombinationGroups?.length > 1
                  ? `${dayjs(
                      searchRequestData?.SearchLegs[0].DepartureTime,
                    ).format("DD MMM")} - ${dayjs(
                      searchRequestData?.SearchLegs[1].DepartureTime,
                    ).format("DD MMM")}`
                  : bookedProduct.BookingInformation.RoundDepartureTime}
              </p>
            </div>
            <div className={styles.rowCell}>
              <p className="font-medium text-gray-500">Seyahat Nedeni</p>
              <p className="mt-1">
                {data?.travelReasonName ? data?.travelReasonName : ""}
              </p>
            </div>
            <div className={styles.rowCell}>
              <p className="font-medium text-gray-500">
                {bookedProduct.PriceSummary.TotalPriceTitle}
              </p>
              <p className="mt-1">{totalPrice}</p>
            </div>
          </div>
        </Card>
        <Card className="p-8">
          <div className="mb-4 flex items-center justify-between">
            <h2 className={cn(styles.title, "text-lg")}>Uçuş Bilgileri</h2>
            <PermGuard permission={requestApprovalAllowPerm}>
              {status === "warning" && (
                <Link href={`/booking-request-flight-change?id=${id}`}>
                  <Button variant="outline-primary">Uçuş Değiştir</Button>
                </Link>
              )}
            </PermGuard>
          </div>
          <div className="flex flex-row gap-4">
            <FlightMultipleWayCard
              reservationData={
                reservationData as unknown as IPlaneReservationData
              }
              totalPrice={totalPrice}
            />
          </div>
        </Card>
        <Card className="p-8">
          <div className="flex items-center justify-between">
            <h2 className={cn(styles.title, "text-lg")}>Kişi Bilgileri</h2>
          </div>

          {passengerInformations
            ? Object.values(passengerInformations).map((passanger) => {
                let phoneNumber;
                if (passanger?.PhoneNumbers) {
                  phoneNumber = JoinItems([
                    passanger?.PhoneNumbers[0]?.CountryCode,
                    passanger?.PhoneNumbers[0]?.AreaCode,
                    passanger?.PhoneNumbers[0]?.Number,
                  ]);
                }

                return (
                  <div
                    key={passanger.Key}
                    className={cn(
                      "grid grid-cols-1 gap-4 md:gap-0 mt-4 pb-4 border-b last:border-b-0 md:grid-cols-4",
                      {
                        "lg:grid-cols-5": !!passanger.MilesCode,
                      },
                    )}
                  >
                    <div className={styles.rowCell}>
                      <p className={styles.heading}>Ad Soyad</p>
                      <p
                        className="mt-1 truncate"
                        title={`${passanger?.Name} ${passanger?.Surname}`}
                      >
                        {passanger?.Name || Undefined()} {passanger?.Surname}
                      </p>
                    </div>
                    <div className={styles.rowCell}>
                      <p className={styles.heading}>Doğum Tarihi</p>
                      <p className="mt-1">
                        {dayjs(passanger?.Birthday).format("DD.MM.YYYY")}
                      </p>
                    </div>
                    <div className={styles.rowCell}>
                      <p className={styles.heading}>Telefon Numarası</p>
                      <p className="mt-1">
                        {`+${phoneNumber}` || <Undefined />}
                      </p>
                    </div>
                    <div className={styles.rowCell}>
                      <p className={styles.heading}>E-posta</p>
                      <p className="mt-1 truncate" title={passanger?.Email}>
                        {passanger?.Email || <Undefined />}
                      </p>
                    </div>
                    {passanger?.MilesCode && (
                      <div className={styles.rowCell}>
                        <p className={styles.heading}>Miles & Smiles No</p>
                        <p
                          className="mt-1 truncate"
                          title={passanger?.MilesCode}
                        >
                          {passanger?.MilesCode || <Undefined />}
                        </p>
                      </div>
                    )}
                  </div>
                );
              })
            : Undefined()}
        </Card>

        {reservationData.ReservationNote && (
          <Card className="p-8">
            <div className="flex items-center justify-between">
              <h2 className={cn(styles.title, "text-lg")}>Not</h2>
            </div>
            <div className="mt-3 text-gray-500">
              {reservationData.ReservationNote}
            </div>
          </Card>
        )}

        {reservationData.ProjectCode && (
          <Card className="p-8">
            <div className="flex items-center justify-between">
              <h2 className={cn(styles.title, "text-lg")}>Fatura Notu</h2>
            </div>
            <div className="mt-3 text-gray-500">
              {reservationData.ProjectCode}
            </div>
          </Card>
        )}

        {displayInformations?.Approvers &&
          displayInformations?.Approvers.length > 0 && (
            <Card className="p-8">
              <div className="flex items-center justify-between">
                <h2 className={cn(styles.title, "text-lg")}>
                  Onaylayacak Kişiler
                </h2>
              </div>
              <div className="mt-3 text-gray-500">
                {displayInformations.Approvers.map(
                  (item) => item.FullName,
                ).join(", ")}
              </div>
            </Card>
          )}

        <Card className="hidden p-8">
          <BookingRequest.Note
            className="bg-white"
            description="Lorem ipsum dolor sit amet lorem ipsum amet"
          />
        </Card>
      </div>
    </div>
  );
}

export default FlightRequestCardDetail;
