import dayjs from "dayjs";
import React, { useState } from "react";

import { Button } from "@/components/ui/button/button";
import { Calendar } from "@/components/ui/calendar/calendar";
import Card from "@/components/ui/card/card";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import { Modal } from "@/components/ui/modal/modal";
import { useModal } from "@/components/ui/modal/modal.hook";
import { useLocale } from "@/hooks/useLocale";
import type { HotelAvailabilityItems } from "@/services/eva-display/hotel/hotel-detail/hotel-detail-services.types";

export type RoomAvailableDatesModalProps = {
  availableDates: HotelAvailabilityItems[];
  onClickApply?: ({
    checkIn,
    checkOut,
  }: {
    checkIn: Date;
    checkOut?: Date;
  }) => void;
};

const RoomAvailableDatesModal = ({
  availableDates,
  onClickApply,
}: RoomAvailableDatesModalProps) => {
  const { t } = useLocale();
  const {
    closeModal: availabilityCloseModal,
    openModal: availabilityOpenModal,
    modalOpen: availabilityModalOpen,
  } = useModal({ open: false });
  const [selectedDates, setSelectedDates] = useState<
    | {
        checkIn: Date;
        checkOut?: Date;
      }
    | undefined
  >();

  const handleClickAvailableDates = () => {
    if (availableDates.length > 0) {
      availabilityOpenModal();
    }
  };

  const handleClickAvailableApply = () => {
    onClickApply?.(selectedDates!);
  };

  return (
    <>
      <Button onClick={handleClickAvailableDates} variant="outline-primary">
        {t("See Available Dates")}
      </Button>
      <Modal
        size="xl"
        open={availabilityModalOpen}
        onClose={availabilityCloseModal}
        title={t("Availability Calendar")}
        footer={
          <div className="mt-2 flex items-center justify-end gap-4">
            <Button
              variant="outline-primary"
              className="min-w-[150px]"
              onClick={availabilityCloseModal}
            >
              {t("Close")}
            </Button>
            <Button
              disabled={!selectedDates?.checkIn || !selectedDates.checkOut}
              variant="primary"
              className="min-w-[150px]"
              onClick={handleClickAvailableApply}
            >
              {t("Apply")}
            </Button>
          </div>
        }
      >
        <div>
          <Card className="my-2 grid grid-cols-2 !p-0 text-center text-xs !shadow-none">
            <div className="flex items-center justify-center border-r border-gray-200">
              <LIcon
                name="Dot"
                spacing={0}
                className="text-primary-500"
                size={35}
              />
              <span>{t("Available")}</span>
            </div>{" "}
            <div className="flex items-center justify-center border-r border-gray-200">
              <LIcon
                name="Dot"
                spacing={0}
                className="text-gray-500"
                size={35}
              />
              <span>{t("Not available")}</span>
            </div>
          </Card>
          <div className="flex justify-center">
            <Calendar
              numberOfMonths={2}
              disabled={availableDates.length === 0}
              mode="range"
              selected={{
                from: selectedDates?.checkIn,
                to: selectedDates?.checkOut,
              }}
              fromDate={
                availableDates?.[0]?.date && new Date(availableDates?.[0]?.date)
              }
              toDate={
                availableDates?.[availableDates.length - 1]?.date &&
                new Date(
                  availableDates?.[availableDates.length - 1]?.date as Date,
                )
              }
              modifiers={{
                available: availableDates
                  .filter((item) => item.status === 2)
                  .map((item) => new Date(item.date)),
                booked: availableDates
                  .filter((item) => item.status === 1)
                  .map((item) => new Date(item.date)),
              }}
              modifiersClassNames={{
                available:
                  "text-primary-500 [&:has(.day-range-start)]:!text-white",
                booked: "!text-gray-500",
              }}
              onDayClick={(day, modifiers) => {
                if (modifiers.available) {
                  if (!selectedDates?.checkIn) {
                    setSelectedDates({
                      checkIn: day,
                      checkOut: undefined,
                    });
                    return;
                  }

                  if (
                    selectedDates?.checkIn &&
                    !selectedDates?.checkOut &&
                    !dayjs(selectedDates?.checkIn).isBefore(day)
                  ) {
                    setSelectedDates({
                      checkIn: day,
                      checkOut: undefined,
                    });
                    return;
                  }

                  if (
                    selectedDates?.checkIn &&
                    !selectedDates?.checkOut &&
                    dayjs(selectedDates?.checkIn).isBefore(day)
                  ) {
                    setSelectedDates((prev) => ({
                      checkIn: prev!.checkIn,
                      checkOut: day,
                    }));
                    return;
                  }

                  if (selectedDates?.checkIn && selectedDates?.checkOut) {
                    setSelectedDates({
                      checkIn: day,
                      checkOut: undefined,
                    });
                  }
                }
              }}
            />
          </div>
        </div>
      </Modal>
    </>
  );
};

export default RoomAvailableDatesModal;
