/* eslint-disable jsx-a11y/no-static-element-interactions */
/* eslint-disable jsx-a11y/click-events-have-key-events */
/* eslint-disable @next/next/no-img-element */
import type { icons } from "lucide-react";
import Image from "next/image";
import type { ReactNode } from "react";
import React, { useCallback, useEffect } from "react";
import { useWindowSize } from "usehooks-ts";

import NoImage from "@/assets/svg/images/no-image.svg";
import { Badge } from "@/components/ui/badge/badge";
import { Button } from "@/components/ui/button/button";
import { Carousel } from "@/components/ui/carousel/carousel";
import { Tooltip } from "@/components/ui/tooltip/tooltip";
import { cn } from "@/libs/utils";

import styles from "./tour-card.module.css";

interface DiscountBadgeProps {
  price: string;
  rate: number;
}
interface IncludedSpecification {
  icon?: keyof typeof icons;
  text: string;
}
export interface TourCardProps {
  id: string;
  className?: string;
  tourName?: string;
  discount?: DiscountBadgeProps;
  isRecommended?: boolean;
  isLike?: boolean;
  likeButtonVisible?: boolean;
  images?: string[];
  priceInformation?: string;
  includedSpecifications?: IncludedSpecification[];
  SpecificationError?: string;
  selectDateForPrice?: boolean;
  swiperContainerModifierClass?: string;
  price?: string;
  availableDateButton?: boolean;
  detailItems: {
    label: string;
    value: string;
    tooltip?: TourCardTooltipProps;
  }[];
  renderPrimaryButton?: () => React.ReactNode;
  onClickPrimaryButton?: (id: string) => void;
  onClickLike?: (id: string) => void;
}

type AvailableButtonProps = {
  // eslint-disable-next-line react/no-unused-prop-types
  type: "desktop" | "mobile";
};

interface TourCardTooltipProps {
  label: ReactNode;
  children: ReactNode;
}

export function TourCardTooltip({ label, children }: TourCardTooltipProps) {
  return (
    <Tooltip position="bottom" label={label}>
      <div className="cursor-pointer items-center rounded-full border border-gray-100 bg-gray-100 px-2 py-0.5 text-xs font-medium text-slate-800">
        {children}
      </div>
    </Tooltip>
  );
}

function TourCardDetailItem({
  label,
  value,
  tooltip,
}: {
  label?: ReactNode;
  value?: ReactNode;
  tooltip?: TourCardTooltipProps;
}) {
  return (
    value && (
      <div>
        <div className="inline-block whitespace-nowrap text-[14px] font-normal leading-[20px] tracking-normal text-[#667084] [font-family:'Inter-Regular',Helvetica]">
          {label}:
        </div>
        <div className="ml-2 inline-block whitespace-nowrap text-[14px] font-normal leading-[20px] tracking-normal text-[#1d2838] [font-family:'Inter-Regular',Helvetica]">
          {value}
        </div>
        {!!tooltip && (
          <div className="ml-2 inline-block">
            <TourCardTooltip {...tooltip} />
          </div>
        )}
      </div>
    )
  );
}

function TourCard({
  id,
  tourName,
  discount,
  isRecommended,
  isLike,
  likeButtonVisible = false,
  images,
  priceInformation,
  includedSpecifications,
  price,
  selectDateForPrice,
  SpecificationError,
  availableDateButton,
  detailItems,
  renderPrimaryButton,
  onClickLike,
  onClickPrimaryButton,
  className,
  swiperContainerModifierClass,
}: TourCardProps) {
  const handleOnPrimaryClick = useCallback(
    () => onClickPrimaryButton?.(id),
    [id, onClickPrimaryButton],
  );
  const specWrapperRef = React.useRef<HTMLDivElement>(null);
  const [contentSlice, setContentSlice] = React.useState(3);
  const windowSize = useWindowSize();
  const isDesktop = windowSize.width > 640;

  const AvailableButton = useCallback(
    ({ type }: AvailableButtonProps) => (
      /* @todo add translation for 'İncele' & 'Müsait Tarihler' */
      <Button
        variant="outline"
        // className="mt-2 w-full text-sm text-gray-500 sm:!block"
        className={cn(
          "mt-2 w-full text-sm text-gray-500",
          // @todo - This field is not used currently. When service is ready, it will be used.
          type === "desktop" ? "!hidden sm:!hidden" : "!hidden sm:!hidden",
        )}
        onClick={handleOnPrimaryClick}
      >
        Müsait Tarihler
      </Button>
    ),
    [handleOnPrimaryClick],
  );

  const extendincludedSpecifications = includedSpecifications
    ?.slice(contentSlice)
    .map(({ text }, key) => <li key={+key}>{text}</li>);

  useEffect(() => {
    if ((specWrapperRef?.current?.offsetHeight || 0) > 60) {
      setContentSlice((prev) => prev - 1);
    }
  }, [specWrapperRef, contentSlice]);

  return (
    <div
      className={cn(styles.container, className, "group")}
      onClick={() =>
        !isDesktop && !!handleOnPrimaryClick && handleOnPrimaryClick()
      }
    >
      <div className={styles.left}>
        {/* @todo - add translation for 'Öneriliyor' */}
        {isRecommended && (
          <Badge
            leftIcon={{
              name: "ThumbsUp",
              size: 12,
              strokeWidth: 2,
              className: "ml-1",
            }}
            className={styles.recommended}
          >
            Öneriliyor
          </Badge>
        )}
        {likeButtonVisible && (
          <Badge
            leftIcon={{
              name: "Heart",
              size: 12,
              strokeWidth: 3,
              className: "ml-1 text-primary-500",
            }}
            className={cn(styles.like, isLike && "bg-primary-500")}
            onClick={() => onClickLike?.(id)}
          />
        )}

        {images && images?.length > 0 && (
          <Carousel
            wrapperClassName={styles["left-carousel"]}
            className="size-full"
            loop
            navigation={images.length > 1}
            containerModifierClass={swiperContainerModifierClass}
          >
            {images?.map((image) => (
              <img
                src={image}
                alt="tour card images"
                key={image}
                className="h-[214px] w-full object-cover md:h-full"
              />
            ))}
          </Carousel>
        )}
        {!images && (
          <Image
            src={NoImage}
            className="h-[214px] w-full object-cover md:h-full"
            alt="No picture image"
          />
        )}
      </div>
      <div className={styles.contentWrapper}>
        <div className={styles["left-content-wrapper"]}>
          {/* tour Name and Location */}
          <div className="flex flex-col gap-1">
            <h2 className={cn(styles.title, "group-hover:underline")}>
              {tourName}
            </h2>
            {!!detailItems?.length && (
              <div className="flex flex-col gap-1">
                {detailItems.map((item) => (
                  <TourCardDetailItem key={item.label} {...item} />
                ))}
              </div>
            )}
          </div>
          {/* tour Specifications  - @todo it'll be complete */}
          <div ref={specWrapperRef}>
            {SpecificationError ? (
              <Badge
                variant="error"
                leftIcon={{
                  name: "Clock",
                }}
              >
                {SpecificationError}
              </Badge>
            ) : (
              (includedSpecifications?.length || 0) > 0 && (
                <div className={styles.badgeGroup}>
                  {includedSpecifications
                    ?.slice(0, contentSlice)
                    .map(({ text }) => <Badge key={text}>{text}</Badge>)}
                  {(includedSpecifications?.length || 0) > contentSlice && (
                    <TourCardTooltip
                      label={
                        <div className="flex flex-col gap-2 text-xs">
                          <span className="font-semibold">
                            Kampanya ve fırsatlar
                          </span>
                          <ul className="!list-disc px-4 py-0">
                            {extendincludedSpecifications}
                          </ul>
                        </div>
                      }
                    >
                      +{(includedSpecifications?.length || 0) - contentSlice}
                    </TourCardTooltip>
                  )}
                </div>
              )
            )}
          </div>
        </div>
        <div className={styles.divider} />
        <div className={styles["right-content-wrapper"]}>
          {selectDateForPrice && !availableDateButton ? (
            <div className={styles["select-date-wrapper"]}>
              {/* @todo - add translation for 'Fiyat görebilmek için tarih seçin' */}
              <p
                className={styles["select-date-text"]}
                onClick={handleOnPrimaryClick}
                aria-hidden="true"
              >
                Fiyatları görebilmek için tarih seçin
              </p>
              {/* @todo add translation for 'Tarih Seç' */}
              <Button
                variant="primary"
                className="!hidden w-[105px] sm:!block"
                onClick={handleOnPrimaryClick}
              >
                Tarih Seç
              </Button>
            </div>
          ) : (
            <>
              {/* Price & Discount Info */}
              <div>
                <div className="whitespace-nowrap text-right text-[12px] font-medium leading-[18px] tracking-normal text-[#1d2838] [font-family:'Inter-Medium',Helvetica]">
                  Kişi başı
                </div>
                {price && (
                  <>
                    {discount && !availableDateButton && (
                      <>
                        {/* @todo - add translation for 'İndirim' */}
                        <Badge variant="grayGreen">
                          %{discount.rate} İndirim
                        </Badge>
                      </>
                    )}
                    <div className={styles.price}>
                      {discount && !availableDateButton && (
                        <>
                          {/* @todo - add dynamic currency */}
                          <p className={styles["discount-price"]}>
                            {/* @todo - add dynamic currency and format */}
                            {discount.price}
                          </p>
                        </>
                      )}
                      {/* @todo - add dynamic currency and format */}
                      <div>{price || 0.0}</div>
                      {/* @todo - add translation for 'gece toplam' */}
                    </div>
                    <p className={styles["night-info"]}>{priceInformation}</p>
                  </>
                )}
              </div>
              {/* Card Button and night info */}
              <div className={styles["card-button-wrapper"]}>
                {price && (
                  <>
                    {/* @todo add translation for 'İncele' */}
                    {/* eslint-disable-next-line no-nested-ternary */}
                    {renderPrimaryButton ? (
                      renderPrimaryButton()
                    ) : !availableDateButton ? (
                      <Button
                        variant="primary"
                        className="!hidden w-[105px] sm:!block"
                        onClick={handleOnPrimaryClick}
                      >
                        İncele
                      </Button>
                    ) : (
                      availableDateButton && <AvailableButton type="desktop" />
                    )}
                  </>
                )}
              </div>
            </>
          )}
        </div>
        {availableDateButton && (
          <div className={styles["available-button-wrapper"]}>
            <AvailableButton type="mobile" />
          </div>
        )}
      </div>
    </div>
  );
}

export default TourCard;
