/* eslint-disable no-console */
import * as Select from "@radix-ui/react-select";
import dayjs from "dayjs";
import Link from "next/link";
import React, { useState } from "react";

import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import {
  Dialog,
  DialogClose,
  DialogContent,
  DialogDescription,
  DialogFooter,
  DialogHeader,
  DialogTitle,
  DialogTrigger,
} from "@/components/ui/dialog/dialog";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import { SelectItem } from "@/components/ui/select";
import type { STextProps } from "@/components/ui/status-text/status-text";
import { StatusText } from "@/components/ui/status-text/status-text";
import type { ReactQueryReturnProps } from "@/components/widget/booking-request-card-detail/booking-request-card-detail";
import BookingRequest from "@/components/widget/request-card/booking-request-card/booking-request-card";
import { useAuth } from "@/features/auth/hooks/use-auth";
import PermGuard from "@/features/permission/components/perm-guard";
import { PriceManager } from "@/libs/price-manager/price-manager";
import { cn } from "@/libs/utils";
import type { IBookingRequest } from "@/models/booking-request.model";
import type { IPassengerInformation } from "@/services/eva-display/demand/passenger-information/passenger-information";
import type { ITransferBookedProduct } from "@/services/eva-display/demand/transfer-booked-product/transfer-booked-product";
import type { ITransferReservationData } from "@/services/eva-display/demand/transfer-reservation-data/transfer-reservation-data";
import { requestApprovalAllowPerm } from "@/validations/permissions/request-approval";

import styles from "../flight-request-card-detail/flight-request-card-detail.module.css";
import TransferMultipleWayCard from "../request-card/transfer-request-card/transfer-multiple-way-card";
import TransferRequestCardDetailPersonalInformation from "./transfer-request-card-detail-personal-information";
import TransferRequestCardDetailReservationInformation from "./transfer-request-card-detail-reservation-information";

export interface DeniedReasonType {
  value: string;
  name: string;
}

interface BookingRequestCardDetailProps {
  className?: string;
  status: STextProps["status"];
  data: IBookingRequest;
  bookedProduct: ITransferBookedProduct;
  reservationData: ITransferReservationData;
  passengerInformations: IPassengerInformation;
  onAccept: () => void;
  onReject: () => void;
  onDenied: (reason: DeniedReasonType) => void;
  bookingRequestStatusMutation: ReactQueryReturnProps;
}

export function Undefined() {
  return <small className="text-error-400">bulunamadı</small>;
}

function statusTextCondition(status: string) {
  switch (status) {
    case "error":
      return "iptal edildi";
    case "denied":
      return "reddedildi";
    default:
      return "onaylandı";
  }
}

const deniedReasons: DeniedReasonType[] = [
  {
    value: "1",
    name: "Bütçe limiti yetersiz",
  },
  {
    value: "2",
    name: "Kurala uymuyor",
  },
  {
    value: "3",
    name: "Departman kısıtı",
  },
  {
    value: "4",
    name: "Kişi kısıtı",
  },
  {
    value: "5",
    name: "Diğer",
  },
];
const initialDeniedReason: DeniedReasonType = deniedReasons[0]!;

function TransferRequestCardDetail({
  className,
  status,
  data,
  reservationData,
  passengerInformations,
  onAccept,
  onReject,
  onDenied,
  bookingRequestStatusMutation,
}: BookingRequestCardDetailProps) {
  const [rejectSelectValue, setRejectSelectValue] =
    useState<DeniedReasonType>(initialDeniedReason);
  const [valueError, setValueError] = useState("");
  const { user } = useAuth();

  const selectValueChanged = (e: string) => {
    const item = deniedReasons.find((x) => x.value === e);
    setRejectSelectValue({
      name: item?.value === "5" ? "" : item?.name!,
      value: item?.value || "",
    });
  };

  let transferSegmentGroupItems =
    reservationData.DetailResponse.TransferSegmentGroupItem.filter((x) =>
      reservationData.ProductKeys.includes(x.Key),
    );

  transferSegmentGroupItems = transferSegmentGroupItems.sort(
    (a, b) =>
      dayjs(a.Items[0]?.DepartureTime).valueOf() -
      dayjs(b.Items[0]?.DepartureTime).valueOf(),
  );

  const originSegmentGroup = transferSegmentGroupItems[0]?.Items[0];
  const destinationSegmentGroup =
    transferSegmentGroupItems.length === 1
      ? null
      : transferSegmentGroupItems[1]?.Items[0];

  return (
    <div className={cn(styles.container, className, "group")}>
      <div className={styles.wrapper}>
        <Link
          href="/booking-request"
          className="mr-auto flex items-center gap-1.5 text-primary-500"
        >
          <LIcon name="ArrowLeft" size={18} />
          <span className="font-base font-medium">Geri Dön</span>
        </Link>
        <h1 className="text-3xl font-semibold">
          {originSegmentGroup?.Name}
          {data.createDate && (
            <p className="mt-2.5 text-base font-normal text-gray-500">
              {dayjs(data.createDate).format("DD MMMM YYYY (HH:mm)")} tarihinde
              oluşturuldu
            </p>
          )}
        </h1>

        <Card className="p-8 shadow-lg">
          <div className="flex flex-col md:flex-row md:items-center md:justify-between">
            <div className="flex gap-10">
              <div className="flex flex-col">
                <p className="mb-2.5 font-medium text-gray-500">Onay Durumu</p>
                <StatusText
                  status={status}
                  text={`${dayjs(data.updateDate).format(
                    "DD MMMM YYYY (HH:mm)",
                  )} tarihinde ${
                    data.updatedByUserName
                      ? `${data.updatedByUserName} tarafindan`
                      : ``
                  } ${statusTextCondition(status)}.`}
                />
                {/* @TODO - bind dynamic data  */}
                {status === "denied" && (
                  <>
                    <p className="mt-4 font-medium text-gray-500">
                      Reddedilme Nedeni
                    </p>
                    {data.corpRequestHistories
                      ?.filter(
                        (item) => item.newStatus === 2 && !!item.description,
                      )
                      .map((item) => (
                        <p key={item.id} className="text-gray-800">
                          {item.description}
                        </p>
                      ))}
                  </>
                )}
              </div>
              {data.bookedProduct &&
                data.bookedProduct.OptionDate &&
                dayjs((data.bookedProduct as any)?.OptionDate).isValid() && (
                  <div className="flex flex-col">
                    <p className="mb-2.5 font-medium text-gray-500">
                      Option Süresi
                    </p>
                    <div className="mt-0.5 text-sm font-medium text-warning-500">
                      {`${dayjs((data.bookedProduct as any)?.OptionDate).format(
                        "DD.MM.YYYY HH:mm",
                      )}'e kadar`}
                    </div>
                  </div>
                )}
            </div>
            <div className="mt-3 flex flex-col gap-3 sm:flex-row md:mt-0">
              {status === "warning" && (
                <>
                  {status === "warning" &&
                    user?.userGuidId === data.createdByUserId && (
                      <Dialog>
                        <DialogTrigger asChild>
                          <Button
                            type="button"
                            variant="outline-error"
                            disabled={bookingRequestStatusMutation?.isLoading}
                          >
                            İptal Et
                          </Button>
                        </DialogTrigger>
                        <DialogContent className="max-w-[408px]">
                          <DialogHeader>
                            <div className="mr-auto inline-flex rounded-full border-8 border-error-50 bg-error-100 p-3">
                              <LIcon name="Info" className="text-error-500" />
                            </div>
                            <DialogTitle className="!mt-5">
                              Talebi iptal etmek istediğine emin misin?
                            </DialogTitle>
                            <DialogDescription>
                              İptal Etme işlemini geri alamazsınız.
                            </DialogDescription>
                          </DialogHeader>
                          <DialogFooter className="mt-4 !flex-col gap-3 sm:!flex-row">
                            <DialogClose className="w-full">
                              <Button
                                className="w-full"
                                data-state="closed"
                                variant="outline"
                              >
                                Vazgeç
                              </Button>
                            </DialogClose>
                            <Button
                              className="!ml-0 w-full"
                              variant="error"
                              onClick={() => onReject()}
                            >
                              İptal Et
                            </Button>
                          </DialogFooter>
                        </DialogContent>
                      </Dialog>
                    )}
                  <PermGuard permission={requestApprovalAllowPerm}>
                    {status === "warning" && (
                      <>
                        <Dialog>
                          <DialogTrigger asChild>
                            <Button
                              type="button"
                              variant="outline-error"
                              disabled={bookingRequestStatusMutation?.isLoading}
                            >
                              Reddet
                            </Button>
                          </DialogTrigger>
                          <DialogContent className="max-w-[408px]">
                            <DialogHeader>
                              <div className="mr-auto inline-flex rounded-full border-8 border-error-50 bg-error-100 p-3">
                                <LIcon name="Info" className="text-error-500" />
                              </div>
                              <DialogTitle className="!mt-5">
                                Talebi reddetmek istediğine emin misin?
                              </DialogTitle>
                              <DialogDescription>
                                Reddetme işlemini geri alamazsınız.
                              </DialogDescription>
                            </DialogHeader>
                            <div className="flex flex-col gap-1.5">
                              <p className="text-sm font-medium text-gray-800">
                                Reddetme Nedeni
                              </p>

                              <Select.Root
                                defaultValue={rejectSelectValue?.value || ""}
                                onValueChange={selectValueChanged}
                              >
                                <Select.Trigger className="inline-flex items-center justify-between rounded-lg border border-gray-200 bg-white px-3.5 py-2.5 text-base shadow-xs outline-none">
                                  <Select.Value placeholder="Neden seçiniz" />
                                  <Select.Icon>
                                    <LIcon size={20} name="ChevronDown" />
                                  </Select.Icon>
                                </Select.Trigger>
                                <Select.Portal>
                                  <Select.Content
                                    position="popper"
                                    className="!z-50 w-[358px] max-w-[80vw] overflow-hidden rounded-primary bg-white shadow-md"
                                  >
                                    <Select.ScrollUpButton className="flex h-6 items-center justify-center bg-white">
                                      <LIcon size={20} name="ChevronUp" />
                                    </Select.ScrollUpButton>
                                    <Select.Viewport className="p-1">
                                      {deniedReasons.map((item) => {
                                        return (
                                          <SelectItem
                                            key={item.value}
                                            className="cursor-pointer px-3.5 text-base"
                                            value={item.value}
                                          >
                                            {item.name}
                                          </SelectItem>
                                        );
                                      })}
                                    </Select.Viewport>
                                    <Select.ScrollDownButton className="flex h-6 cursor-default items-center justify-center bg-white">
                                      <LIcon size={20} name="ChevronDown" />
                                    </Select.ScrollDownButton>
                                  </Select.Content>
                                </Select.Portal>
                              </Select.Root>

                              {rejectSelectValue?.value === "5" && ( // 5 is "Diğer"
                                <div className="mt-3.5 flex flex-col gap-1.5">
                                  <p className="text-sm font-medium text-gray-800">
                                    Diğer
                                  </p>
                                  <Card className="p-3">
                                    <textarea
                                      className="w-full outline-none"
                                      rows={5}
                                      placeholder="Talebi reddetme nedeniniz nedir?"
                                      onChange={(e) => {
                                        setRejectSelectValue({
                                          value: "5",
                                          name: e.target.value,
                                        });
                                      }}
                                      value={rejectSelectValue.name}
                                    />
                                  </Card>
                                </div>
                              )}
                              {!!valueError && (
                                <p className="text-xs font-medium text-red-500">
                                  {valueError}
                                </p>
                              )}
                            </div>
                            <DialogFooter className="mt-4 !flex-col gap-3 sm:!flex-row">
                              <Button
                                className="w-full"
                                data-state="closed"
                                variant="outline"
                              >
                                Vazgeç
                                <DialogClose />
                              </Button>
                              <Button
                                className="!ml-0 w-full"
                                variant="error"
                                onClick={() => {
                                  if (rejectSelectValue.name.length < 5) {
                                    setValueError(
                                      "Reddetme nedeni en az 5 karakter olmalıdır.",
                                    );
                                    return;
                                  }
                                  onDenied(rejectSelectValue);
                                }}
                              >
                                Reddet
                              </Button>
                            </DialogFooter>
                          </DialogContent>
                        </Dialog>

                        <Dialog>
                          <DialogTrigger asChild>
                            <Button
                              type="button"
                              variant="primary"
                              disabled={bookingRequestStatusMutation?.isLoading}
                            >
                              Onayla
                            </Button>
                          </DialogTrigger>
                          <DialogContent className="max-w-[408px]">
                            <DialogHeader>
                              <div className="mr-auto inline-flex rounded-full border-8 border-success-50 bg-success-100 p-3">
                                <LIcon
                                  name="Check"
                                  className="text-success-500"
                                />
                              </div>
                              <DialogTitle className="!mt-5">
                                Talebi onaylamak istediğine emin misin?
                              </DialogTitle>
                            </DialogHeader>
                            <DialogFooter className="mt-4 !flex-col gap-3 sm:!flex-row">
                              <Button
                                className="w-full"
                                data-state="closed"
                                variant="outline"
                                disabled={
                                  bookingRequestStatusMutation?.isLoading
                                }
                              >
                                Vazgeç
                                <DialogClose />
                              </Button>
                              <Button
                                className="!ml-0 w-full"
                                variant="primary"
                                loading={
                                  bookingRequestStatusMutation?.isLoading
                                }
                                onClick={() => onAccept()}
                              >
                                Onayla
                              </Button>
                            </DialogFooter>
                          </DialogContent>
                        </Dialog>
                      </>
                    )}
                  </PermGuard>
                </>
              )}
            </div>
          </div>
        </Card>

        <TransferRequestCardDetailReservationInformation
          data={data}
          reservationData={reservationData}
        />

        <Card className="p-8">
          <div className="mb-4 flex items-center justify-between">
            <h2 className={cn(styles.title, "text-lg")}>Transfer Bilgileri</h2>
            {status === "warning" && (
              <PermGuard permission={requestApprovalAllowPerm}>
                <Link href={`/booking-request-transfer-change?id=${data.id}`}>
                  <Button variant="outline-primary">Transfer Değiştir</Button>
                </Link>
              </PermGuard>
            )}
          </div>
          <div>
            <TransferMultipleWayCard
              hasAddress
              reservationData={reservationData as ITransferReservationData}
            />
          </div>
        </Card>

        <TransferRequestCardDetailPersonalInformation
          passengerInformations={passengerInformations}
        />

        <Card className="space-y-4">
          <h3 className="text-xl font-semibold">Fiyat Özeti</h3>
          <div className="space-y-2">
            {originSegmentGroup && (
              <div className="flex items-center justify-between">
                <span className="text-sm">Gidiş Transferi</span>
                <span className="text-base font-semibold">
                  {PriceManager({
                    currency: originSegmentGroup.TotalPrice.slice(
                      0,
                      3,
                    ) as string,
                    value: +originSegmentGroup.TotalPrice.slice(3),
                  })}
                </span>
              </div>
            )}
            {destinationSegmentGroup && (
              <div className="flex items-center justify-between">
                <span className="text-sm">Dönüş Transferi</span>
                <span className="text-base font-semibold">
                  {PriceManager({
                    currency: destinationSegmentGroup.TotalPrice.slice(
                      0,
                      3,
                    ) as string,
                    value: +destinationSegmentGroup.TotalPrice.slice(3),
                  })}
                </span>
              </div>
            )}
          </div>

          <div className="rounded-primary border border-gray-200 p-4">
            <div className="flex items-center justify-between">
              <span className="text-sm">Toplam Tutar</span>
              <span className="text-xl font-semibold">
                {PriceManager({
                  currency: (originSegmentGroup?.TotalPrice, "TRY").slice(
                    0,
                    3,
                  ) as string,
                  value: +data.totalPrice,
                })}
              </span>
            </div>
          </div>
        </Card>

        {/* Reservation Note */}
        <Card className="p-8">
          <BookingRequest.Note
            className="bg-white"
            description={reservationData?.ReservationNote || "-"}
          />
        </Card>
        {/* Invoice Note */}
        <Card className="p-8">
          <BookingRequest.Note
            title="Fatura Notu"
            className="bg-white"
            description={reservationData?.ProjectCode || "-"}
          />
        </Card>
        {/* Approvers */}
        <Card className="p-8">
          <BookingRequest.Note
            title="Onaylayacak Kişiler"
            className="bg-white"
            description={
              data.displayInformations?.Approvers?.map(
                (item) => item.FullName,
              ).join(", ") || "-"
            }
          />
        </Card>
      </div>
    </div>
  );
}

export default TransferRequestCardDetail;
