/* eslint-disable no-nested-ternary */
import dayjs from "dayjs";
import { type PropsWithChildren, useState } from "react";
import { useBoolean } from "usehooks-ts";

import { Calendar } from "@/components/ui/calendar/calendar";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import {
  Popover,
  PopoverContent,
  PopoverTrigger,
} from "@/components/ui/popover/popover";
import { useLocale } from "@/hooks/useLocale";
import { useMessageByDates } from "@/libs/get-message-by-dates/get-message-by-dates";
import { DateHelper } from "@/libs/helpers/date-time-helper";
import { cn } from "@/libs/utils";

import TpBottomBox from "../tp-bottom-box/tp-bottom-box";
import TpValidationMessage from "../tp-validation-message/tp-validation-message";
import styles from "./tp-date-range-picker.module.css";
import type { TpDateRangePickerProps } from "./tp-date-range-picker.types";
import { TpDateRangePickerTriggerButton } from "./tp-date-range-picker-trigger-button";

let memClassName = "";
const selectedStartDayButtonSelector = ".day-range-end.day-range-start";
const hoverDayButtonClass = "border border-gray-200 rounded-primary";

const TpDateRangePicker = ({ className, ...props }: TpDateRangePickerProps) => {
  const boolean = useBoolean();
  const resetBoolean = useBoolean();
  const { t } = useLocale();
  const [message, setMessage] = useState("");
  const dateMessage = useMessageByDates(props.value, props.showNightInfo);
  const isBeforeCheck = (e: Date) =>
    props.value?.from &&
    !props.value?.to &&
    DateHelper.dayjs(e).isBefore(props.value?.from);
  const isAfterCheck = (e: Date) =>
    props.value?.from &&
    !props.value?.to &&
    DateHelper.dayjs(props.value?.from).isBefore(e);
  const addBorderToElement = (element: MouseEvent) => {
    const target = element?.target as HTMLDivElement;
    target.className = cn(target.className, hoverDayButtonClass);
  };
  const removeBorderFromElement = (element: MouseEvent) => {
    const target = element?.target as HTMLDivElement;
    target.className = target.className.replace(hoverDayButtonClass, "");
  };
  return (
    <div className={cn("grid gap-2 relative", className)}>
      <Popover open={boolean.value} onOpenChange={boolean.setValue}>
        <PopoverTrigger className="w-full">
          <TpDateRangePickerTriggerButton bool={boolean} {...props} />
        </PopoverTrigger>
        <PopoverContent
          className={styles.popoverContent}
          align="end"
          side="bottom"
          avoidCollisions={false}
        >
          {props.children}
          <div>
            <Calendar
              initialFocus
              mode="range"
              min={props.minDate}
              fromDate={dayjs().toDate()}
              toDate={
                props.untilDay
                  ? dayjs().add(props.untilDay, "day").toDate()
                  : undefined
              }
              toYear={!props.untilDay ? dayjs().year() + 1 : undefined}
              defaultMonth={props.value?.from}
              selected={props.value}
              onSelect={(value) => {
                if (
                  !resetBoolean.value &&
                  value?.from &&
                  !isBeforeCheck(value?.from)
                ) {
                  if (value?.to) {
                    resetBoolean.setTrue();
                  }
                  props.onChange({
                    from: value?.from,
                    to: value?.to,
                  });
                } else {
                  let changeValue = null;
                  if (props.value.from !== value?.from) {
                    changeValue = value?.from;
                  } else if (props.value.to !== value?.to) {
                    changeValue = value?.to;
                  } else {
                    changeValue = value?.to;
                  }

                  resetBoolean.setFalse();
                  props.onChange({
                    from: changeValue,
                    to: undefined,
                  });
                }
              }}
              numberOfMonths={2}
              onDayMouseEnter={(e, _, z) => {
                if (isBeforeCheck(e)) {
                  const selectedButton = document.querySelector(
                    selectedStartDayButtonSelector,
                  );
                  if (selectedButton) {
                    memClassName = selectedButton.className;
                    selectedButton.className = cn(
                      memClassName,
                      "!bg-transparent !text-primary-500 rounded-primary",
                    );
                  }
                  if (z) {
                    addBorderToElement(z as unknown as MouseEvent);
                  }
                  setMessage(t("You are about to change the login date"));
                }

                if (props.value?.from && isAfterCheck(e)) {
                  if (z) {
                    addBorderToElement(z as unknown as MouseEvent);
                  }
                  setMessage(
                    `${t("Total")} ${DateHelper.getNightCount(props.value.from, e)} ${t("night")}`,
                  );
                }
              }}
              onDayMouseLeave={(e, _, z) => {
                if (isBeforeCheck(e)) {
                  const selectedButton = document.querySelector(
                    selectedStartDayButtonSelector,
                  );
                  if (selectedButton) {
                    selectedButton.className = memClassName;
                  }
                }

                if (z) {
                  removeBorderFromElement(z as unknown as MouseEvent);
                }
                setMessage("");
              }}
            />
          </div>
          <TpBottomBox
            onApplyClick={boolean.setFalse}
            label={
              <div className={styles.popoverFooterContent}>
                {message || dateMessage}
              </div>
            }
            buttonText={t("Apply")}
          />
        </PopoverContent>
      </Popover>
      {props.errorMessage && (
        <TpValidationMessage message={t("This field is required")} />
      )}
    </div>
  );
};

export const TpDateRangeMessageBox = (props: PropsWithChildren) => {
  return (
    <div className={styles.popoverHeaderWrapper}>
      <div className={styles.popoverHeaderContent}>
        <div className={styles.popoverHeaderIconWrapper}>
          <LIcon name="AlertCircle" className={styles.popoverHeaderIcon} />
        </div>
        <div className={styles.popoverHeaderContentText}>{props.children}</div>
      </div>
    </div>
  );
};

TpDateRangePicker.MessageBox = TpDateRangeMessageBox;

export default TpDateRangePicker;
