"use client";

import { CommandInput } from "cmdk";
import type { SetStateAction } from "react";
import { useRef, useState } from "react";
import { useBoolean, useOnClickOutside } from "usehooks-ts";

import { Button } from "@/components/ui/button/button";
import { Command, CommandItem, CommandList } from "@/components/ui/command";
import { Dialog, DialogContent, DialogTrigger } from "@/components/ui/dialog";
import { Input } from "@/components/ui/input/input";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import TriggerButton from "@/components/ui/trigger-button/trigger-button";
import { useBreakpoints } from "@/hooks/useBreakpoints";
import { useLocale } from "@/hooks/useLocale";
import { cn } from "@/libs/utils";

import TpCloseButton from "../shareds/tp-close-button";

interface TpSearchProps<T extends { key: string }> {
  renderItem: (item: T, index: number) => React.ReactNode;
  onSelect: (item: T, index: number) => void;
  onChangeQuery?: (query: string) => void;
  data: T[];
  isLoading?: boolean;
  defaultQuery?: string;
  placeholder?: string;
  title?: string;
  shouldFilter?: boolean;
  leftIcon?: React.ReactNode;
  queryLengthLimit?: 0 | number;
}

const TpSearch = <
  T extends {
    value: SetStateAction<string>;
    key: string;
  },
>({
  shouldFilter = false,
  queryLengthLimit = 3,
  ...props
}: TpSearchProps<T>) => {
  const [query, setQuery] = useState(props.defaultQuery ?? "");
  const inputRef = useRef<HTMLInputElement>(null);
  const boolean = useBoolean(false);
  const containerRef = useRef<HTMLDivElement>(null);
  const { isDesktop } = useBreakpoints();
  const { t } = useLocale();

  useOnClickOutside(containerRef, () => {
    boolean.setFalse();
  });

  const isEmpty = !props.isLoading && (!props.data || props.data?.length === 0);

  if (!isDesktop) {
    return (
      <Dialog modal open={boolean.value} onOpenChange={boolean.setValue}>
        <DialogTrigger asChild>
          <TriggerButton
            size="lg"
            label={props.placeholder || t("Travel destination")}
            leftIcon={props.leftIcon}
            fullWidth
            onClick={() => {
              boolean.setTrue();
            }}
            value={query && props.defaultQuery}
          />
        </DialogTrigger>
        <DialogContent
          noShowClose
          className="flex size-full h-full max-h-full max-w-full flex-col overflow-auto !rounded-none p-0"
        >
          <div className="shrink-0 basis-[100px] px-4 pt-1">
            <TpCloseButton onClick={boolean.setFalse} />
            <div className="relative">
              <Input
                key="mobile-search"
                autoFocus
                value={query}
                ref={inputRef}
                defaultValue={props.defaultQuery}
                fullWidth
                size="lg"
                placeholder={props.placeholder || t("Travel destination")}
                title={props.title || t("Travel destination")}
                className="!w-full border-0 p-0 text-base outline-none ring-0 placeholder:!text-base"
                containerClassName="!border-0 !shadow-none !ring-0 !p-0"
                onFocus={() => {
                  boolean.setTrue();
                }}
                onChange={(e) => {
                  props.onChangeQuery?.(e.target.value);
                  setQuery(e.target.value);
                }}
              />
              {query !== "" && (
                <div className="absolute right-0 top-1">
                  <Button
                    onClick={() => {
                      setQuery("");
                      props.onChangeQuery?.("");
                    }}
                    size="icon"
                    className="text-gray-400"
                  >
                    <LIcon name="X" />
                  </Button>
                </div>
              )}
            </div>
            <hr />
          </div>
          <div className="grow overflow-y-auto">
            {props.data?.length > 0 &&
              props.data.map((item, index) => (
                <div
                  onClick={() => {
                    props.onSelect(item, index);
                    setQuery(item.value);
                    boolean.setFalse();
                  }}
                  onKeyDown={() => {
                    props.onSelect(item, index);
                    setQuery(item.value);
                    boolean.setFalse();
                  }}
                  aria-hidden="true"
                  className="mb-2 p-0"
                  key={item.key}
                >
                  {props.renderItem(item, index)}
                </div>
              ))}
            {props.isLoading && query.length >= 3 && (
              <div className="flex h-full min-h-[50px] flex-1 items-center justify-center">
                <LIcon name="Loader" className="animate-spin" />
              </div>
            )}
            {isEmpty && <div className="p-5 text-center">Sonuç Bulunamadı</div>}
          </div>
        </DialogContent>
      </Dialog>
    );
  }

  return (
    <div ref={containerRef}>
      <Command shouldFilter={shouldFilter}>
        <Input
          key="desktop-search"
          value={query}
          ref={inputRef}
          defaultValue={props.defaultQuery}
          fullWidth
          label={props.placeholder || t("Travel destination")}
          leftIcon={props.leftIcon}
          size="lg"
          title={props.title || t("Travel destination")}
          as={CommandInput}
          className="!w-full truncate border-0 p-0 outline-none ring-0"
          onFocus={() => {
            boolean.setTrue();
          }}
          asProps={{
            onValueChange: (text: string) => {
              props.onChangeQuery?.(text);
              setQuery(text);
            },
          }}
        />

        {query !== "" && (
          <div className="absolute right-[4px] top-[8px]">
            <Button
              onClick={() => {
                setQuery("");
                props.onChangeQuery?.("");
              }}
              size="icon"
              className="text-[#101828]"
            >
              <LIcon size={16} name="X" />
            </Button>
          </div>
        )}

        {boolean.value &&
          (queryLengthLimit === 0 || query.length >= queryLengthLimit) && (
            <CommandList
              className={cn(
                "scroll-bar absolute z-20 mt-[60px] max-h-[300px] w-full md:min-w-[350px] max-w-[100%] md:max-w-[350px] rounded-primary border border-gray-200 bg-white",
              )}
            >
              {props.data?.length > 0 &&
                props.data.map((item, index) => (
                  <CommandItem
                    onSelect={() => {
                      props.onSelect(item, index);
                      setQuery(item.value);
                      boolean.setFalse();
                    }}
                    className="mb-2 p-0"
                    key={item.key}
                  >
                    {props.renderItem(item, index)}
                  </CommandItem>
                ))}
              {props.isLoading && query.length >= 3 && (
                <div className="flex h-full min-h-[50px] flex-1 items-center justify-center">
                  <LIcon name="Loader" className="animate-spin" />
                </div>
              )}
              {isEmpty && (
                <div className="p-5 text-center">Sonuç Bulunamadı</div>
              )}
            </CommandList>
          )}
      </Command>
    </div>
  );
};

export default TpSearch;
