/* eslint-disable no-console */
import type { Dispatch, SetStateAction } from "react";
import React, { useCallback, useEffect, useState } from "react";

import {
  DropdownMenu,
  DropdownMenuContent,
  DropdownMenuItem,
  DropdownMenuTrigger,
} from "@/components/ui/dropdown-menu/dropdown-menu";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import TriggerButton from "@/components/ui/trigger-button/trigger-button";
import type { EventPayload } from "@/components/widget/core-list/core-list";
import CoreList from "@/components/widget/core-list/core-list";
import RentACarCard from "@/components/widget/rent-a-car-card/rent-a-car-card";
import type { Car } from "@/services/eva-display/rent-a-car/rent-a-car-detail/rent-a-car-detail.types";
import { useRentACarDetail } from "@/services/eva-display/rent-a-car/rent-a-car-detail/rent-a-car-detail-service";
import type {
  IRentACarRequest,
  RentACarItem,
} from "@/services/eva-display/rent-a-car/rent-a-car-search/rent-a-car-search.types";
import { useRentACarSearch } from "@/services/eva-display/rent-a-car/rent-a-car-search/rent-a-car-search-service";

import { useBookingRequestRentACarChangeContext } from "../context/booking-request-car-rental-context";

interface BookingRequestCarRentalCarSelectProps {
  car: Car;
  searchData: IRentACarRequest;
  setSelectedCar: Dispatch<SetStateAction<Car | undefined>>;
}

const dropdownItems = [
  {
    title: "Artan Fiyat",
    value: "asc",
  },
  {
    title: "Azalan Fiyat",
    value: "desc",
  },
];

const BookingRequestCarRentalCarSelect = ({
  car,
  searchData,
  setSelectedCar,
}: BookingRequestCarRentalCarSelectProps) => {
  const { mutateAsync, isLoading, data } = useRentACarSearch();
  const [requestData, setRequestData] = useState<IRentACarRequest>({
    CarRentalSearchRequest: {
      ...searchData.CarRentalSearchRequest,
      Filter: {
        PageSize: 10,
        Page: 1,
        IsPriceSortedByDesc: false,
      },
      SearchId: null,
    },
  });
  const { setActiveStep, setSearchId, searchId } =
    useBookingRequestRentACarChangeContext();
  const [itemLoadingKey, setItemLoadingKey] = useState("");
  const { mutateAsync: detailMutation, isLoading: detailLoading } =
    useRentACarDetail();
  const [listData, setListData] = useState<RentACarItem[]>([]);

  const getData = useCallback(() => {
    const payload = { ...requestData };

    mutateAsync({
      CarRentalSearchRequest: {
        PickupStationId: `${payload.CarRentalSearchRequest.SearchLeg.PickupStationId}`,
        PickupDateTime: payload.CarRentalSearchRequest.PickupDateTime,
        ReturnDateTime: payload.CarRentalSearchRequest.ReturnDateTime,
        ReturnStationId: `${payload.CarRentalSearchRequest.SearchLeg.ReturnStationId}`,
        DriverAge: payload.CarRentalSearchRequest.DriverAge,
        SearchLeg: payload.CarRentalSearchRequest.SearchLeg,
        Filter: {
          PageSize: 10,
          Page: payload.CarRentalSearchRequest.Filter?.Page,
          IsPriceSortedByDesc:
            payload.CarRentalSearchRequest.Filter?.IsPriceSortedByDesc || false,
        },
        SearchId: payload.CarRentalSearchRequest.SearchId,
      },
    }).then((e) => {
      setSearchId(e.searchId);

      if (requestData?.CarRentalSearchRequest?.SearchId) {
        setListData((prev) => [...prev, ...(e?.items || [])]);
      } else {
        setListData(e?.items || []);
      }
    });
  }, [mutateAsync, requestData, setSearchId]);

  const handleClickItem = async (item: Car) => {
    setItemLoadingKey(item.key);
    const detail = await detailMutation({
      ProductKey: item.key,
      searchId,
      InsuranceOptions: [],
      ExtraOptions: [],
    });
    if (detail?.car?.key) {
      setSelectedCar(item);
      setActiveStep(1);
    } else {
      console.log("Error: ", detail);
    }
    setItemLoadingKey("");
  };

  const handleChangeFilter = (event: EventPayload) => {
    if (event.type === "loadMore") {
      setRequestData((prev) => ({
        CarRentalSearchRequest: {
          ...prev.CarRentalSearchRequest,
          Filter: {
            ...prev.CarRentalSearchRequest?.Filter,
            Page: data?.hasMorePage
              ? (data?.filter?.page || 0) + 1
              : data?.filter.page,
          },
          SearchId: searchId,
        },
      }));
    }
  };

  useEffect(() => {
    getData();
  }, [getData]);

  return (
    <>
      {/* Selected Car */}
      <div className="overflow-hidden rounded-primary border border-blue-500">
        <div className="bg-blue-50 px-4 py-2 font-semibold text-gray-800">
          Seçili araç
        </div>
        <RentACarCard
          id={`${car.key}`}
          item={car}
          hideDetail
          hidePrimaryButton
          onClickPrimaryButton={() => {}}
        />
      </div>
      {/* Search Result */}
      <div>
        <CoreList
          renderHeader={() => (
            <div className="flex items-center justify-between">
              <div className="text-xl font-semibold text-gray-800">
                Alternatif Araçlar
              </div>
              <div>
                <DropdownMenu>
                  <DropdownMenuTrigger className="w-full max-w-[218px] flex-1 md:flex-auto">
                    <TriggerButton
                      /**
                       * @todo - add translation fot these text
                       */
                      label="Sıralama"
                      fullWidth
                      size="lg"
                      value={
                        dropdownItems.find(
                          (item) =>
                            item.value ===
                            (requestData.CarRentalSearchRequest.Filter
                              ?.IsPriceSortedByDesc
                              ? "desc"
                              : "asc"),
                        )?.title
                      }
                      className="bg-white"
                      rightIcon={
                        <LIcon name="ChevronDown" height={24} width={24} />
                      }
                    />
                  </DropdownMenuTrigger>
                  <DropdownMenuContent className="w-[218px]">
                    {dropdownItems.map((item) => (
                      <DropdownMenuItem
                        key={item.value}
                        onClick={() => {
                          setRequestData((prev) => ({
                            CarRentalSearchRequest: {
                              ...prev.CarRentalSearchRequest,
                              Filter: {
                                ...prev.CarRentalSearchRequest?.Filter,
                                IsPriceSortedByDesc: item.value === "desc",
                                Page: 1,
                              },
                              SearchId: null,
                            },
                          }));
                          setListData([]);
                        }}
                        className="px-[14px] py-[10px]"
                      >
                        {item.title}
                      </DropdownMenuItem>
                    ))}
                  </DropdownMenuContent>
                </DropdownMenu>
              </div>
            </div>
          )}
          totalCount={data?.totalCount || 0}
          data={(listData || []) as Car[]}
          isLoading={isLoading}
          isEmpty={false}
          hasMore={data?.hasMorePage}
          filterOptions={[]}
          hideFilter
          hiderSort
          onChange={handleChangeFilter}
          renderItem={(item: Car) => {
            return (
              <RentACarCard
                id={item.key}
                item={item}
                primaryButtonText="Seç"
                onClickPrimaryButton={() => handleClickItem(item)}
                loading={detailLoading && itemLoadingKey === item.key}
              />
            );
          }}
        />
      </div>
    </>
  );
};

export default BookingRequestCarRentalCarSelect;
