/* eslint-disable no-unsafe-optional-chaining */
import dayjs from "dayjs";
import { useState } from "react";
import { Swiper, SwiperSlide } from "swiper/react";

import Container from "@/components/shared/container";
import TinyHeader from "@/components/shared/tiny-header/tiny-header";
import { Breadcrumb } from "@/components/ui/breadcrumb/breadcrumb";
import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import CircleProgressBar from "@/components/ui/circle-progress-bar/circle-progress-bar";
import { DUMMY_FLIGHT_LIST_REQUEST } from "@/data/dummy/flight-list-request";
import FlightListItemContent from "@/features/flight/components/flight-list/components/flight-list-item/components/flight-list-item-content";
import {
  FlightListContextProvider,
  useFlightListContext,
} from "@/features/flight/components/flight-list/context/flight-list-context";
import {
  getCityName,
  getWaitingTime,
  sortCombinationGroupsForBookingRequest,
  sortFlightSegmentsForBookingRequest,
} from "@/libs/helpers/flight-helper";
import lcfc from "@/libs/helpers/lower-cased-first-character";
import { PriceManager } from "@/libs/price-manager/price-manager";
import type { IFlightBookingRequest } from "@/models/booking-request.model";
import type {
  CombinationGroup,
  IFlightSearchRequest,
} from "@/services/eva-display/flight/flight-search/flight-search-services.types";
import type {
  IFlightSearchResultSerializer,
  PricingCombinationWithCombinationGroups,
} from "@/services/eva-display/flight/flight-search/flight-search-services-serializer";

import ChoosedRoomCard from "../shared/choosed-room-card";
import BRFCDetailData from "./components/brfc-detail-data";
import FlightSearchResult from "./components/brfc-flight-search-result";

interface BookingRequestFlightChangeProps {
  flightBookingRequest: IFlightBookingRequest;
}

const BookingRequestFlightChangeContent = (
  props: BookingRequestFlightChangeProps,
) => {
  const [state, setState] = useState({
    flightDetail: null as {
      productKey: string;
      searchId: string;
      passengers: IFlightSearchResultSerializer["passengerList"];
    } | null,
    combinationKeys: [] as string[],
  });
  const { departureSegmentKey, setDepartureSegmentKey } =
    useFlightListContext();
  const ProgressBarItems = [
    {
      label: "Biletler",
    },
    {
      label: "Onay",
    },
  ];

  const [progressBarActiveItem, setProgressBarActiveItem] = useState(0);
  const [mySwiper, setMySwiper] = useState<any>({});

  const pricingCombinations = [
    ...(props.flightBookingRequest.reservationData?.DetailResponse
      .PricingCombinations || []),
  ];

  const combinationGroups = sortCombinationGroupsForBookingRequest(
    pricingCombinations?.flatMap((item) => item.CombinationGroups)!,
  ).map((combinationGroupItem) => ({
    ...combinationGroupItem,
    FlightSegments: sortFlightSegmentsForBookingRequest(
      combinationGroupItem.FlightSegments,
    ),
  }));

  if (pricingCombinations.length === 0 || !combinationGroups) {
    return null;
  }

  const title = `${getCityName(
    combinationGroups[0]?.FlightSegments[
      (combinationGroups[0]?.FlightSegments.length ?? 1) - 1
    ]?.Destination,
  )} Seyahati`;

  const segments = {
    departure: combinationGroups[0]?.FlightSegments[0],
    arrival:
      combinationGroups[0]?.FlightSegments[
        (combinationGroups[0]?.FlightSegments.length ?? 1) - 1
      ],
  };

  const description = `${dayjs(segments.departure?.DepartureTime).format(
    "DD MMM YYYY HH:mm",
  )} - ${dayjs(segments.arrival?.ArrivalTime).format(
    "DD MMM YYYY HH:mm",
  )} (${getWaitingTime(
    segments.departure!.DepartureTime,
    segments.arrival!.ArrivalTime,
  )}), ${
    props.flightBookingRequest.passengerInformations?.length
  } Yolcu, ${combinationGroups[0]?.CabinClassTitle?.toLowerCase()}`;

  const isRoundTrip = combinationGroups.length > 1;

  const selectedCombinationGroup = lcfc(
    pricingCombinations?.length > 1
      ? pricingCombinations
          .find((x) =>
            x.CombinationGroups.find(
              (y) => y.GroupId === (!departureSegmentKey ? 0 : 1),
            ),
          )
          ?.CombinationGroups.find((x) => x.GroupId === 1)
      : pricingCombinations[0]?.CombinationGroups.find(
          (x) => x.GroupId === (!departureSegmentKey ? 0 : 1),
        ),
  ) as unknown as CombinationGroup;

  return (
    <div>
      <div className="border-b border-gray-200 bg-white">
        <Container>
          <TinyHeader href="booking-request" title="Talep Değiştir" />
        </Container>
      </div>
      <Container className="mt-4 pb-5">
        <div className="flex flex-col gap-3">
          <ChoosedRoomCard
            title={title}
            description={description}
            directionInfo={
              combinationGroups.length > 1 ? "Gidiş Dönüş" : "Tek Yön"
            }
          />
          <div className="mx-auto w-6/12 md:w-2/12">
            <CircleProgressBar
              items={ProgressBarItems}
              activeIndex={progressBarActiveItem}
              itemOnClick={(e) => {
                if (e === 0) {
                  setState((prev) => ({ ...prev, flightDetail: null }));
                }
                mySwiper.slideTo(e, 700);
              }}
              className="my-8"
            />
          </div>
          <div>
            <Swiper
              spaceBetween={50}
              slidesPerView={1}
              allowTouchMove={false}
              onSlideChange={(e) => {
                window.scrollTo({
                  left: 0,
                  top: 0,
                  behavior: "instant",
                });
                setProgressBarActiveItem(e.activeIndex);
              }}
              onInit={(ev) => {
                setMySwiper(ev);
              }}
            >
              <SwiperSlide>
                {props.flightBookingRequest.displayInformations!.SearchRequest
                  ?.FlightDisplayType === 1 && (
                  <Breadcrumb
                    steps={[
                      {
                        key: "departure",
                        label: "Gidiş",
                      },
                      {
                        key: "arrival",
                        label: "Dönüş",
                      },
                    ]}
                    activeSteps={
                      departureSegmentKey ? ["arrival"] : ["departure"]
                    }
                    completedSteps={departureSegmentKey ? ["departure"] : []}
                  />
                )}
                <div className="mt-4 flex flex-col gap-10">
                  <div className="rounded-primary border border-primary-500">
                    <div className="rounded-t-primary bg-primary-50 px-4 py-2">
                      <span className="font-semibold">Seçili Bilet</span>
                    </div>
                    <Card>
                      <FlightListItemContent
                        isNoDetail
                        hasLocationInfo
                        passengerCount={
                          props.flightBookingRequest.passengerInformations
                            ?.length ?? 1
                        }
                        combinationGroup={selectedCombinationGroup}
                        renderActionButton={() => (
                          <Button
                            variant="outline-primary"
                            onClick={() => {
                              if (
                                state?.combinationKeys?.[0] ===
                                  pricingCombinations[0]?.Key ||
                                !isRoundTrip
                              ) {
                                window.location.href = `/flight-request-detail?id=${props.flightBookingRequest.id}`;
                              }

                              // If there is no any change in the selected flight, redirect to the flight request detail page
                              else if (
                                !departureSegmentKey &&
                                setDepartureSegmentKey &&
                                pricingCombinations
                              ) {
                                setDepartureSegmentKey(
                                  pricingCombinations?.[0]?.Key ?? "",
                                );
                                setState((prev) => ({
                                  ...prev,
                                  flightDetail: null,
                                  selectedDepartureItem: lcfc(
                                    pricingCombinations?.[0],
                                  ) as any,
                                  combinationKeys: [
                                    pricingCombinations?.[0]?.Key ?? "",
                                  ],
                                }));
                              } else {
                                const returnItem =
                                  pricingCombinations?.[1] ||
                                  pricingCombinations?.[0]
                                    ?.CombinationGroups?.[1];
                                setState((prev) => ({
                                  ...prev,
                                  flightDetail: null,
                                  combinationKeys: [
                                    departureSegmentKey ?? "",
                                    returnItem?.Key ?? "",
                                  ],
                                }));
                                mySwiper.slideTo(2, 700);
                              }
                            }}
                          >
                            Değiştirmeden İlerle
                          </Button>
                        )}
                        pricingCombination={
                          lcfc(
                            pricingCombinations?.length > 1
                              ? pricingCombinations[
                                  !departureSegmentKey ? 0 : 1
                                ]
                              : pricingCombinations[0],
                          ) as unknown as PricingCombinationWithCombinationGroups
                        }
                        price={PriceManager({
                          currency: selectedCombinationGroup?.totalPrice.slice(
                            0,
                            3,
                          ),
                          value:
                            +selectedCombinationGroup?.totalPrice.slice(3) /
                            (props.flightBookingRequest.passengerInformations
                              ?.length ?? 1),
                        })}
                      />
                    </Card>
                  </div>
                  {/* List of flights based on initial search criteria */}
                  <FlightSearchResult
                    segments={segments}
                    onItemSelect={(_, flightDetail) => {
                      setState((prev) => ({
                        ...prev,
                        flightDetail,
                        combinationKeys:
                          flightDetail.combinationGroupKeys ?? [],
                      }));
                      mySwiper.slideTo(2, 700);
                    }}
                  />
                </div>
              </SwiperSlide>
              <SwiperSlide>
                {state.flightDetail && (
                  <BRFCDetailData
                    pricingCombinationKeys={state.combinationKeys}
                    productKey={state.flightDetail.productKey}
                    searchId={state.flightDetail.searchId}
                    flightBookingRequest={{
                      ...props.flightBookingRequest,
                      passengerInformations:
                        props.flightBookingRequest.passengerInformations?.map(
                          (passenger, index) => ({
                            ...passenger,
                            Key:
                              state.flightDetail?.passengers[index]?.key ?? "",
                          }),
                        ),
                    }}
                  />
                )}
              </SwiperSlide>
            </Swiper>
          </div>
        </div>
      </Container>
    </div>
  );
};

const BookingRequestFlightChange = (props: BookingRequestFlightChangeProps) => {
  const urlVariables: IFlightSearchRequest = {
    ...DUMMY_FLIGHT_LIST_REQUEST,
    searchLegs:
      props.flightBookingRequest.displayInformations!.SearchRequest.SearchLegs.map(
        (searchLeg) =>
          ({
            cabinClass: searchLeg.CabinClass,
            departureTime: searchLeg.DepartureTime,
            destinationId: searchLeg.DestinationId,
            destinationType: searchLeg.DestinationType,
            originId: searchLeg.OriginId,
            originType: searchLeg.OriginType,
          }) as any,
      ),
    Filter: DUMMY_FLIGHT_LIST_REQUEST.Filter,
    searchPassengers:
      props.flightBookingRequest?.passengerInformations?.map((x) => ({
        age: x.Age ? 22 : 1,
        passengerType: x.PassengerType,
      })) ?? [],
    FlightDisplayType:
      props.flightBookingRequest.displayInformations!.SearchRequest
        ?.FlightDisplayType || 0,
    FlightTripKind:
      props.flightBookingRequest.displayInformations!.SearchRequest
        ?.FlightTripKind || 0,
  };

  return (
    <FlightListContextProvider urlVariables={urlVariables}>
      <BookingRequestFlightChangeContent {...props} />
    </FlightListContextProvider>
  );
};
export default BookingRequestFlightChange;
