/* eslint-disable react/no-unused-prop-types */
import dayjs from "dayjs";
import Link from "next/link";
import { useEffectOnce } from "usehooks-ts";

import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import { Skeleton } from "@/components/ui/skeleton/skeleton";
import Spinner from "@/components/ui/spinner";
import {
  RezervationSummaryDivider,
  RezervationSummaryFlightInfo,
  RezervationSummaryFlightPrice,
  RezervationSummaryFlightTotalPrice,
  RezervationSummaryProvider,
  RezervationSummaryTitle,
} from "@/components/widget/rezervation-summary/rezervation-summary";
import { getFormatedFlightTime } from "@/libs/helpers/date-time-helper";
import {
  getAirportCode,
  sortCombinationGroups,
  sortFlightSegments,
} from "@/libs/helpers/flight-helper";
import { PriceManager } from "@/libs/price-manager/price-manager";
import type { IFlightBookingRequest } from "@/models/booking-request.model";
import { useFlightAddToCartMutation } from "@/services/eva-display/flight/flight-add-to-cart/flight-add-to-cart.service";
import type { IFlightDetailResponse } from "@/services/eva-display/flight/flight-detail/flight-detail.service";

import { useBookingRequestFlightChange } from "../hooks/use-booking-request-flight-change";

interface BRFCCartDataProps {
  flightBookingRequest: IFlightBookingRequest;
  searchId: string;
  cartId: string;
  pricingCombinationKeys: string[];
  detailData: IFlightDetailResponse;
}

const BRFCCartData = (props: BRFCCartDataProps) => {
  const addToCartMutation = useFlightAddToCartMutation();
  const { changeBookingRequest, isLoading } = useBookingRequestFlightChange({
    detailData: props.detailData,
    flightBookingRequest: props.flightBookingRequest,
    pricingCombinationKeys: props.pricingCombinationKeys,
    searchId: props.searchId,
  });

  function addToCart() {
    addToCartMutation.mutate({
      FlightCartProductRequest: {
        PreBook: 0,
        PricingCombinationKeys: props.pricingCombinationKeys,
        searchId: props.searchId,
        cardId: null,
      },
    });
  }

  useEffectOnce(() => {
    addToCart();
  });

  if (addToCartMutation.isLoading || !addToCartMutation.data) {
    return (
      <div className="flex items-center justify-center">
        <Spinner />
      </div>
    );
  }

  if (addToCartMutation.isError) {
    return (
      <div>
        <div>Error...</div>
      </div>
    );
  }

  const combinationGroups = sortCombinationGroups(
    props.detailData.pricingCombinations.length > 1
      ? props.detailData.pricingCombinations.flatMap(
          (pricingCombination) => pricingCombination.combinationGroups,
        )
      : (props.detailData.pricingCombinations[0]?.combinationGroups ?? []),
  );

  if (!combinationGroups) {
    return (
      <div>
        <Skeleton className="h-[100px]" />
      </div>
    );
  }

  return (
    <div>
      <RezervationSummaryProvider>
        <RezervationSummaryTitle
          title={
            props.detailData.pricingCombinations[0]!.combinationGroups.length >
              1 || props.detailData.pricingCombinations.length > 1
              ? "Gidiş Dönüş"
              : "Tek Yön"
          }
        />
        <RezervationSummaryDivider />
        {combinationGroups.map((combinationGroup) => {
          const flightSegmentsItem = sortFlightSegments(
            combinationGroup.flightSegments,
          );
          const originSegmentItem = flightSegmentsItem[0]!;
          const departureSegmentItem =
            flightSegmentsItem[flightSegmentsItem.length - 1]!;
          return (
            <RezervationSummaryFlightInfo
              key={combinationGroup.key}
              destination={
                getAirportCode(departureSegmentItem.destination) ?? ""
              }
              origin={getAirportCode(originSegmentItem.origin) ?? ""}
              date={dayjs(
                combinationGroup.flightSegments[0]?.departureTime,
              ).format("DD MMM YYYY")}
              time={`${dayjs(
                combinationGroup.flightSegments[0]?.departureTime,
              ).format("HH:mm")} - ${dayjs(
                combinationGroup.flightSegments[0]?.arrivalTime,
              ).format("HH:mm")}`}
              flightTime={
                getFormatedFlightTime(combinationGroup.travelTime ?? "") ?? ""
              }
              operatingAirline={
                combinationGroup.flightSegments[0]?.operatingAirline ?? ""
              }
              flightNumber={
                combinationGroup.flightSegments[0]?.flightNumber ?? ""
              }
            />
          );
        })}
        <RezervationSummaryDivider />
        <RezervationSummaryFlightPrice
          priceDescription={[
            {
              description: "Uçuş Bedeli",
              price: props.detailData.pricingCombinations[0]?.basePrice ?? "",
            },
            {
              description: "Vergi",
              price: props.detailData.pricingCombinations[0]?.taxes ?? "",
            },
          ]}
        />
        <RezervationSummaryFlightTotalPrice
          price={PriceManager({
            currency: props.detailData.pricingCombinations[0]!.totalPrice.slice(
              0,
              3,
            ),
            value:
              +props.detailData.pricingCombinations[0]!.totalPrice.slice(3),
          })}
        />
      </RezervationSummaryProvider>
      <Card className="mt-6">
        <div className="flex items-center justify-between">
          <div className="flex flex-col items-start gap-1">
            <span className="text-base font-medium text-gray-500">
              Toplam Tutar
            </span>
            <span className="text-lg font-semibold">
              {PriceManager({
                currency:
                  props.detailData.pricingCombinations[0]!.totalPrice.slice(
                    0,
                    3,
                  ),
                value:
                  +props.detailData.pricingCombinations[0]!.totalPrice.slice(3),
              })}
            </span>
          </div>
          <div className="flex gap-3">
            <Link
              href={`/flight-request-detail?id=${props.flightBookingRequest.id}`}
            >
              <Button variant="outline">Vazgeç</Button>
            </Link>
            <Button
              variant="primary"
              onClick={() => {
                if (addToCartMutation.data) {
                  changeBookingRequest({
                    cartId: addToCartMutation.data.cartId,
                  });
                }
              }}
              loading={isLoading}
            >
              Değiştir ve Onayla
            </Button>
          </div>
        </div>
      </Card>
    </div>
  );
};

export default BRFCCartData;
