/* eslint-disable no-nested-ternary */
/* eslint-disable react-hooks/exhaustive-deps */
import type { PropsWithChildren } from "react";
import { useCallback, useState } from "react";

import {
  Accordion,
  AccordionContent,
  AccordionItem,
  AccordionTrigger,
} from "@/components/ui/accordion/accordion";
import { Button } from "@/components/ui/button/button";
import { Input } from "@/components/ui/input/input";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import { useBreakpoints } from "@/hooks/useBreakpoints";
import { useLocale } from "@/hooks/useLocale";
import { getDiscountCouponAddon } from "@/libs/helpers/addon-helper";
import { cn } from "@/libs/utils";
import { useServerDataCtx } from "@/providers/server-data-provider/server-data-provider";
import {
  useAddCouponCodeMutation,
  useRemoveCouponCodeMutation,
} from "@/services/eva-booking/payment/payment.service";

import { useCoreSummaryCtx } from "../../context/core-summary-context";

type StatusStateType = {
  message?: string;
  status: "idle" | "loading" | "success" | "error";
};

interface CoreSummaryCouponCodeProps {
  transactionKey: string;
  cartId: string;
}

const MobileWrapper = ({ children }: PropsWithChildren) => {
  const { t } = useLocale();
  return (
    <Accordion type="single" collapsible defaultValue="coupon">
      <AccordionItem
        value="coupon"
        className="rounded-primary border bg-gray-100 px-4 py-0 md:w-[744px]"
      >
        <AccordionTrigger className="hover:no-underline">
          {t("Coupon Code")}
        </AccordionTrigger>
        <AccordionContent>{children}</AccordionContent>
      </AccordionItem>
    </Accordion>
  );
};

const DesktopWrapper = ({ children }: PropsWithChildren) => (
  <div>{children}</div>
);

function CoreSummaryCouponCode(props: CoreSummaryCouponCodeProps) {
  const { t } = useLocale();
  const { addonList } = useServerDataCtx();
  const DEFAULT_ERROR_MESSAGE = t("Coupon code canceled");
  const DEFAULT_SUCCESS_MESSAGE = t("Coupon code added");

  const coreSummary = useCoreSummaryCtx();
  const [couponCodeState, setCouponCodeState] = useState(
    !coreSummary.state.hasCouponCode
      ? ""
      : (coreSummary.state.discounts?.[0]?.code ?? ""),
  );
  const [statusState, setStatusState] = useState<StatusStateType>(
    !coreSummary.state.hasCouponCode
      ? {
          status: "idle",
        }
      : {
          status: "success",
          message: DEFAULT_SUCCESS_MESSAGE,
        },
  );

  const applyDiscountCouponMutation = useAddCouponCodeMutation();
  const removeDiscountCouponMutation = useRemoveCouponCodeMutation();
  const { isDesktop } = useBreakpoints();

  const isLoading = statusState.status === "loading";
  const isCouponAdded =
    statusState.status === "success" && coreSummary.state.hasCouponCode;

  async function applyCoupon(code: string) {
    setStatusState({ status: "loading" });

    try {
      const response = await applyDiscountCouponMutation.mutateAsync({
        cartId: props.cartId,
        transactionKey: props.transactionKey,
        code,
      });

      if (response.isSuccess) {
        setStatusState({
          status: "success",
          message: DEFAULT_SUCCESS_MESSAGE,
        });
        const discount = response.discountCoupons?.[0];
        if (discount) {
          coreSummary.actions.addDiscount({
            type: "CouponCode",
            code: couponCodeState,
            currency: discount.price.slice(0, 3),
            price: +discount.price.slice(3),
            title: discount.description,
            extras:
              response.loyalities && response.loyalities.length > 0
                ? response.loyalities.map((loyaltyItem) => ({
                    productCode: loyaltyItem.key,
                    type: "Loyalty",
                    value: loyaltyItem.value,
                  }))
                : undefined,
          });
        }
      } else {
        setStatusState({
          status: "error",
          message: DEFAULT_ERROR_MESSAGE,
        });
      }
    } catch (error) {
      setStatusState({ status: "error", message: DEFAULT_ERROR_MESSAGE });
    }
  }

  async function removeCoupon() {
    setStatusState({ status: "loading" });

    try {
      await removeDiscountCouponMutation.mutateAsync({
        cartId: props.cartId,
        transactionKey: props.transactionKey,
        code: couponCodeState,
      });
      setStatusState({ status: "idle" });
      coreSummary.actions.removeDiscount(couponCodeState);
      setCouponCodeState("");
    } catch (error) {
      /* empty */
    }
  }

  const handleClick = useCallback(() => {
    if (isCouponAdded) {
      removeCoupon();
    } else {
      applyCoupon(couponCodeState);
    }
  }, [couponCodeState, isCouponAdded, removeCoupon, applyCoupon]);

  const discountCouponAddon = getDiscountCouponAddon(addonList);
  if (!discountCouponAddon) {
    return null;
  }

  if (coreSummary.state.hasCreditCardCampaign) {
    return null;
  }

  const Wrapper = isDesktop ? DesktopWrapper : MobileWrapper;

  return (
    <Wrapper>
      <div className="relative">
        <Input
          value={couponCodeState}
          onChange={(e) => setCouponCodeState(e.target.value)}
          maxLength={100}
          placeholder={t("Add coupon code")}
          className="text-sm placeholder:text-sm"
          disabled={isCouponAdded}
        />
        <div className="absolute right-3 top-0 flex h-full items-center">
          <Button
            size="sm"
            disabled={couponCodeState === ""}
            variant={isCouponAdded ? "outline-primary" : "primary"}
            className="!h-6 !w-6 !p-0"
            onClick={handleClick}
            loading={isLoading}
          >
            {isLoading ? null : isCouponAdded ? (
              <LIcon name="Trash2" className="w-4" />
            ) : (
              <LIcon name="Plus" className="w-4" />
            )}
          </Button>
        </div>
      </div>
      {statusState.message && (
        <div className="mt-3 flex items-center gap-1">
          <LIcon
            name="Info"
            className={cn("w-3", {
              "text-green-500": statusState.status === "success",
              "text-red-500": statusState.status === "error",
            })}
          />
          <p
            className={cn("text-xs", {
              "text-green-500": statusState.status === "success",
              "text-red-500": statusState.status === "error",
            })}
          >
            {statusState.message}
          </p>
        </div>
      )}
    </Wrapper>
  );
}

export default CoreSummaryCouponCode;
