import { useEffect, useRef } from "react";

import FormTextInput from "@/components/form/components/basic/form-text-input";
import FormTextAreaInput from "@/components/form/components/basic/form-textarea-input";
import FormBirthDateInput from "@/components/form/components/related/form-birthdate-input";
import type { FormCitiesHandles } from "@/components/form/components/related/form-cities-select";
import FormCitiesSelect, {
  useFormCitiesSelect,
} from "@/components/form/components/related/form-cities-select";
import FormCountrySelect from "@/components/form/components/related/form-country-select";
import FormEmailInput from "@/components/form/components/related/form-email-input";
import FormGenderRadio from "@/components/form/components/related/form-gender-radio";
import FormIdentityOrPassport from "@/components/form/components/related/form-identity-or-passport-input";
import FormPassportDateInput from "@/components/form/components/related/form-passport-date-input";
import FormPassportNoInput from "@/components/form/components/related/form-passport-no-inpuıt";
import FormPhoneInput from "@/components/form/components/related/form-phone-input";
import { useLocale } from "@/hooks/useLocale";
import {
  nameValidation,
  surNameValidation,
} from "@/validations/form-rules/rules/name-rule";
import { requiredRule } from "@/validations/form-rules/rules/required-rule";

interface CustomerInfoFormItemsProps {
  customerPrefix: string;
  hasPassportInfos?: boolean;
  showMilesCodeInput?: boolean;
  showIdentityWarning?: boolean;
  hideGender?: boolean;
  noContactInfo?: boolean;
  isBillingInfo?: boolean;
}

const CustomerInfoFormItems = (props: CustomerInfoFormItemsProps) => {
  const citiesRef = useRef<FormCitiesHandles>(null);
  const { t } = useLocale();
  const { initFormCitiesSelect, getCitiesData } = useFormCitiesSelect(
    citiesRef,
    props.customerPrefix,
  );

  useEffect(() => {
    initFormCitiesSelect();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  return (
    <div>
      {!props.hideGender && (
        <FormGenderRadio
          name={`${props.customerPrefix}-gender`}
          rules={{
            required: requiredRule,
          }}
        />
      )}
      <div className="mt-2 grid grid-cols-1 gap-3 md:grid-cols-2 ">
        <FormTextInput
          fullWidth
          name={`${props.customerPrefix}-name`}
          label={`${t("Name")}*`}
          size="lg"
          rules={{
            required: requiredRule,
            validate: (value) => {
              return nameValidation(value);
            },
          }}
        />
        <FormTextInput
          fullWidth
          name={`${props.customerPrefix}-surname`}
          label={`${t("Surname")}*`}
          size="lg"
          rules={{
            required: requiredRule,
            validate: (value) => {
              return surNameValidation(value);
            },
          }}
        />
        <FormBirthDateInput
          fullWidth
          name={`${props.customerPrefix}-birthDate`}
          size="lg"
          label={`${t("Date of Birth")}* ${t("(dd.mm.yyyy)")}`}
        />
        <FormIdentityOrPassport
          fullWidth
          name={`${props.customerPrefix}-identity`}
          size="lg"
          label={`${t("TC Identification Number")}*`}
          rules={{
            required: requiredRule,
          }}
          showWarning={props.showIdentityWarning}
        />
        {!props.noContactInfo && (
          <>
            <FormEmailInput
              fullWidth
              name={`${props.customerPrefix}-email`}
              size="lg"
              label={`${t("Email")}*`}
              rules={{
                required: requiredRule,
              }}
            />
            <FormPhoneInput
              fullWidth
              name={`${props.customerPrefix}-phone`}
              size="lg"
              label={`${t("Telephone")}*`}
              rules={{
                required: {
                  value: true,
                  message: t("This field is required"),
                },
                validate: (value) => {
                  if (!value || value?.phoneNumber === "")
                    return t("This field is required");

                  return true;
                },
              }}
            />
          </>
        )}
        {props.isBillingInfo && (
          <>
            <FormCountrySelect
              name={`${props.customerPrefix}-country`}
              onChange={(countriesId) => getCitiesData(countriesId)}
              rules={{
                required: requiredRule,
              }}
              placeholder={`${t("Country")}*`}
            />
            <FormCitiesSelect
              ref={citiesRef}
              name={`${props.customerPrefix}-city`}
              placeholder={`${t("City")}*`}
              rules={{
                required: requiredRule,
              }}
            />
            <div className="md:col-span-2">
              <FormTextAreaInput
                name={`${props.customerPrefix}-address`}
                placeholder={`${t("Billing address")}*`}
                rules={{
                  required: requiredRule,
                }}
              />
            </div>
          </>
        )}
        {props.showMilesCodeInput && (
          <div className="md:col-span-2">
            <FormTextInput
              size="lg"
              placeholder={t("Miles&Smiles Number")}
              name={`${props.customerPrefix}-milesCode`}
              fullWidth
              leftIcon={<div>TK</div>}
              maxLength={9}
              className="text-sm placeholder:text-sm"
              rules={{
                validate: (value) => {
                  if (value?.length > 0) {
                    const sum = (arr: any) =>
                      arr.reduce((b: any, a: any) => b + a, 0);
                    if (value.length !== 9)
                      return t("The entered TK number is incorrect.");
                    if (
                      (sum(value.slice(0, 8).split("").map(Number)) -
                        Number(value.slice(-1))) %
                        9 !==
                      0
                    )
                      return t("The entered TK number is incorrect.");
                    return true;
                  }
                  return true;
                },
              }}
            />
          </div>
        )}
      </div>
      {props.hasPassportInfos && (
        <div className="mt-4">
          <h4 className="mb-4 text-sm font-semibold">Pasaport Bilgileri</h4>
          <div className="mt-2 grid grid-cols-2 gap-3">
            <FormPassportNoInput
              fullWidth
              name={`${props.customerPrefix}-passport`}
              size="lg"
              label={`${t("Passport number")}*`}
            />
            <FormPassportDateInput
              fullWidth
              name={`${props.customerPrefix}-passportDate`}
              size="lg"
              label="Pasaport Geçerlilik Tarihi"
            />
            <FormCountrySelect
              name={`${props.customerPrefix}-passportCountry`}
              onChange={() => {}}
              rules={{
                required: requiredRule,
              }}
              placeholder="Pasaport Verilen Ülke"
            />
          </div>
        </div>
      )}
    </div>
  );
};

export default CustomerInfoFormItems;
