/* eslint-disable react-hooks/exhaustive-deps */

"use client";

import { useState } from "react";

import { Button } from "@/components/ui/button/button";
import {
  Modal,
  ModalDescription,
  ModalHeader,
  ModalTitle,
} from "@/components/ui/modal/modal";
import { useModal } from "@/components/ui/modal/modal.hook";
import Spinner from "@/components/ui/spinner";
import type { ReactQueryReturnProps } from "@/components/widget/booking-request-card-detail/booking-request-card-detail";
import BookingRequestCardDetail from "@/components/widget/booking-request-card-detail/booking-request-card-detail";
import BookingRequestDetailSkeleton from "@/features/demand/components/demand-detail/components/booking-request-skeleton";
import type { IBookingRequest } from "@/models/booking-request.model";
import { useBookingCancelReasonQuery } from "@/services/eva-display/book-cancel-reason/booking-cancel-reason-service";
import type { CancelReasonItem } from "@/services/eva-display/book-cancel-reason/booking-cancel-reason-service.types";
import { useBookingDetailQuery } from "@/services/eva-display/booking-detail-request/booking-detail-service";
import { useBookingRequestStatusMutation } from "@/services/eva-display/demand/booking-request-status/booking-request-status.services";
import { CorpRequestStatusEnum } from "@/services/eva-display/demand/booking-request-status/booking-request-status-services.types";
import type { IBookingRequestItem } from "@/services/eva-display/demand/demand-service.types";
import { statusEnumTypes } from "@/services/eva-display/demand/demand-service.types";
import type {
  IDisplayInformations,
  ISearchRequest,
} from "@/services/eva-display/demand/display-information/display-information";
import type { IPassengerInformation } from "@/services/eva-display/demand/passenger-information/passenger-information";
import type { IReservationData } from "@/services/eva-display/demand/reservation-data/reservation-data";

interface DemandDetailProps {
  id: number;
}

interface IApproveReason {
  approveReasonItems: ApproveReasonItem[];
}

interface ApproveReasonItem {
  approveReasonType: number;
  request: string;
  approve: string;
  message: string;
}

const BookingRequestDetail = ({ id }: DemandDetailProps) => {
  const bookingRequestDetailQuery = useBookingDetailQuery({
    RequestId: id,
  });
  const bookingCancelReason = useBookingCancelReasonQuery();
  const [newPrice, setNewPrice] = useState<string | null>(null);
  const [rejectSelectValue, setRejectSelectValue] = useState({
    value: "",
    name: "",
  });

  const { closeModal, openModal, modalOpen } = useModal({
    open: false,
  });
  const { isLoading, isSuccess, isError, data } = bookingRequestDetailQuery;

  const bookingRequestStatusMutation = useBookingRequestStatusMutation();

  const detailData = data as unknown as IBookingRequest;

  const handleAccept = async () => {
    if (!detailData) {
      return;
    }

    const resp = await bookingRequestStatusMutation.mutateAsync({
      CorpBaseRequest: {
        CorpRequestStatus: CorpRequestStatusEnum.Approved,
        RequestId: detailData.id,
        ApprovePriceChange: false,
      },
    });

    if (resp.data?.result?.serviceResponse?.result?.approveReason) {
      const respData = resp.data?.result?.serviceResponse?.result
        ?.approveReason as IApproveReason;

      if (respData?.approveReasonItems) {
        setNewPrice(respData.approveReasonItems[0]?.approve ?? null);
        return;
      }
    }

    if (resp.data?.result?.serviceResponse?.errors) {
      openModal();
    } else {
      await bookingRequestDetailQuery.refetch();
    }
  };
  const handleReject = async () => {
    if (!detailData) {
      return;
    }

    const statusResult = await bookingRequestStatusMutation.mutateAsync({
      CorpBaseRequest: {
        CorpRequestStatus: CorpRequestStatusEnum.Cancelled,
        RequestId: detailData.id,
      },
    });

    // await bookingRequestDetailQuery.refetch();
    if (
      statusResult.data?.success &&
      statusResult.data?.result.serviceResponse.result.isSuccess
    ) {
      await bookingRequestDetailQuery.refetch();
    } else {
      openModal();
    }
  };
  const handleDenied = async () => {
    if (!detailData) {
      return;
    }

    await bookingRequestStatusMutation.mutateAsync({
      CorpBaseRequest: {
        CorpRequestStatus: CorpRequestStatusEnum.Denied,
        RequestId: detailData.id,
        DenyReasonId: +rejectSelectValue.value,
        StatusChangeDescription: rejectSelectValue.name,
      },
    });

    // await bookingRequestDetailQuery.refetch();
    if (bookingRequestStatusMutation.data?.data.success) {
      await bookingRequestDetailQuery.refetch();
    } else {
      openModal();
    }
  };

  return (
    <div>
      {isError && (
        <h1 className="my-5 rounded-lg bg-error-100 p-2 text-error-500">
          Detay bilgileri alinirken bir sorun ile karsilasildi.
        </h1>
      )}
      {isLoading && <BookingRequestDetailSkeleton />}
      {detailData && isSuccess && (
        <>
          <BookingRequestCardDetail
            id={detailData.id}
            data={detailData as unknown as IBookingRequestItem}
            status={statusEnumTypes[detailData?.requestStatus] || "warning"}
            displayInformations={
              detailData?.displayInformations as unknown as IDisplayInformations
            }
            reservationData={
              detailData?.reservationData as unknown as IReservationData
            }
            searchRequest={
              detailData?.searchRequest as unknown as ISearchRequest
            }
            passengerInformations={
              detailData?.passengerInformations as unknown as IPassengerInformation[]
            }
            className="pt-10"
            onAccept={handleAccept}
            onReject={handleReject}
            onDenied={handleDenied}
            bookingRequestStatusMutation={
              bookingRequestStatusMutation as ReactQueryReturnProps
            }
            cancelReasonData={
              (bookingCancelReason.data?.items as CancelReasonItem[]) || []
            }
            setRejectSelectValue={(value) => setRejectSelectValue(value)}
            rejectSelectValue={rejectSelectValue}
            corpRequestHistories={detailData?.corpRequestHistories || []}
          />
          <Modal
            open={!!newPrice}
            variant="warning"
            header={
              <ModalHeader>
                <ModalTitle>Fiyat Değişti</ModalTitle>
                <ModalDescription>
                  {bookingRequestStatusMutation.isLoading ? (
                    <div className="flex items-center gap-3">
                      {`${newPrice} Yeni fiyat ile onaylama işlemi yapılıyor...`}{" "}
                      <Spinner />
                    </div>
                  ) : (
                    `Yeni oda fiyatı ${newPrice} olarak değişti. Yeni fiyatı onaylıyor
                  musunuz?`
                  )}
                </ModalDescription>
              </ModalHeader>
            }
            footer={
              bookingRequestStatusMutation.isLoading ? (
                <div className="mt-3 flex items-center gap-3" />
              ) : undefined
            }
            onClose={() => {
              setNewPrice(null);
            }}
            onCancel={() => {
              setNewPrice(null);
            }}
            onOk={async () => {
              const resp = await bookingRequestStatusMutation.mutateAsync({
                CorpBaseRequest: {
                  CorpRequestStatus: CorpRequestStatusEnum.Approved,
                  RequestId: detailData.id,
                  ApprovePriceChange: true,
                },
              });
              if (resp.data?.result?.serviceResponse?.errors) {
                openModal();
              } else {
                await bookingRequestDetailQuery.refetch();
              }

              setNewPrice(null);
            }}
          />
        </>
      )}

      <Modal
        size="lg"
        open={modalOpen}
        onClose={closeModal}
        title="Uyarı"
        footer
      >
        <div className="flex flex-col gap-4">
          {bookingRequestStatusMutation.data?.data?.result?.serviceResponse
            ?.errors && (
            <>
              <div className="mt-10 flex items-center justify-center">
                {bookingRequestStatusMutation.data?.data?.result
                  ?.serviceResponse?.errors[0].message ===
                "Strategy access denied" ? (
                  <span>Bu işlem için gerekli yetkiniz bulunmamaktadır.</span>
                ) : (
                  <span>
                    {
                      bookingRequestStatusMutation.data?.data?.result
                        ?.serviceResponse?.errors[0].message
                    }
                  </span>
                )}
              </div>
              <div className="flex justify-end">
                <Button variant="error" onClick={closeModal}>
                  Kapat
                </Button>
              </div>
            </>
          )}
        </div>
      </Modal>
    </div>
  );
};

export default BookingRequestDetail;
