"use client";

import { useState } from "react";

import { Button } from "@/components/ui/button/button";
import { Modal } from "@/components/ui/modal/modal";
import { useModal } from "@/components/ui/modal/modal.hook";
import type { ReactQueryReturnProps } from "@/components/widget/booking-request-card-detail/booking-request-card-detail";
import FlightRequestCardDetail from "@/components/widget/flight-request-card-detail/flight-request-card-detail";
import BookingRequestDetailSkeleton from "@/features/demand/components/demand-detail/components/booking-request-skeleton";
import type { IBookingRequest } from "@/models/booking-request.model";
import { useBookingCancelReasonQuery } from "@/services/eva-display/book-cancel-reason/booking-cancel-reason-service";
import type { CancelReasonItem } from "@/services/eva-display/book-cancel-reason/booking-cancel-reason-service.types";
import { useBookingDetailQuery } from "@/services/eva-display/booking-detail-request/booking-detail-service";
import { useBookingRequestStatusMutation } from "@/services/eva-display/demand/booking-request-status/booking-request-status.services";
import { CorpRequestStatusEnum } from "@/services/eva-display/demand/booking-request-status/booking-request-status-services.types";
import { statusEnumTypes } from "@/services/eva-display/demand/demand-service.types";
import type { ISearchRequest } from "@/services/eva-display/demand/display-information/display-information";
import type { IFlightBookedProduct } from "@/services/eva-display/demand/flight-booked-product/flight-booked-product";
import type { IFlightDisplayInformation } from "@/services/eva-display/demand/flight-display-information/flight-display-information";
import type { IPlaneReservationData } from "@/services/eva-display/demand/flight-reservation-data/flight-reservation-data";
import type { IPassengerInformation } from "@/services/eva-display/demand/passenger-information/passenger-information";

interface DemandDetailProps {
  id: number;
}

const FlightRequestDetail = ({ id }: DemandDetailProps) => {
  const bookingRequestDetailQuery = useBookingDetailQuery({
    RequestId: id,
  });
  const bookingCancelReason = useBookingCancelReasonQuery();
  const [rejectSelectValue, setRejectSelectValue] = useState({
    value: "",
    name: "",
  });
  const { isLoading, isSuccess, isError, data } = bookingRequestDetailQuery;
  const { closeModal, openModal, modalOpen } = useModal({
    open: false,
  });

  const bookingRequestStatusMutation = useBookingRequestStatusMutation();

  const detailData = data as unknown as IBookingRequest;

  const handleAccept = async () => {
    if (!detailData) {
      return;
    }

    const resp = await bookingRequestStatusMutation.mutateAsync({
      CorpBaseRequest: {
        CorpRequestStatus: CorpRequestStatusEnum.Approved,
        RequestId: detailData.id,
        ApprovePriceChange: false,
      },
    });

    if (resp.data?.result?.serviceResponse?.errors) {
      openModal();
    } else {
      await bookingRequestDetailQuery.refetch();
    }
  };

  const handleReject = async () => {
    if (!detailData) {
      return;
    }

    const resp = await bookingRequestStatusMutation.mutateAsync({
      CorpBaseRequest: {
        CorpRequestStatus: CorpRequestStatusEnum.Cancelled,
        RequestId: detailData.id,
      },
    });

    if (resp.data?.result?.serviceResponse?.errors) {
      openModal();
    } else {
      await bookingRequestDetailQuery.refetch();
    }
  };

  const handleDenied = async () => {
    if (!detailData) {
      return;
    }

    const resp = await bookingRequestStatusMutation.mutateAsync({
      CorpBaseRequest: {
        CorpRequestStatus: CorpRequestStatusEnum.Denied,
        RequestId: detailData.id,
        DenyReasonId: +rejectSelectValue.value,
        StatusChangeDescription: rejectSelectValue.name,
      },
    });

    if (resp.data?.result?.serviceResponse?.errors) {
      openModal();
    } else {
      await bookingRequestDetailQuery.refetch();
    }
  };

  return (
    <div>
      {isError && (
        <h1 className="my-5 rounded-lg bg-error-100 p-2 text-error-500">
          Detay bilgileri alinirken bir sorun ile karsilasildi.
        </h1>
      )}
      {isLoading && <BookingRequestDetailSkeleton />}
      {detailData && isSuccess && (
        <FlightRequestCardDetail
          id={detailData.id}
          data={detailData}
          displayInformations={
            detailData?.displayInformations as unknown as IFlightDisplayInformation
          }
          status={statusEnumTypes[detailData?.requestStatus] || "warning"}
          reservationData={
            detailData?.reservationData as unknown as IPlaneReservationData
          }
          searchRequest={detailData?.searchRequest as unknown as ISearchRequest}
          bookedProduct={
            detailData?.bookedProduct as unknown as IFlightBookedProduct
          }
          passengerInformations={
            detailData?.passengerInformations as unknown as IPassengerInformation
          }
          cancelReasonData={
            (bookingCancelReason.data?.items as CancelReasonItem[]) || []
          }
          setRejectSelectValue={(value) => setRejectSelectValue(value)}
          rejectSelectValue={rejectSelectValue}
          corpRequestHistories={detailData?.corpRequestHistories || []}
          className="pt-10"
          onAccept={handleAccept}
          onReject={handleReject}
          onDenied={handleDenied}
          bookingRequestStatusMutation={
            bookingRequestStatusMutation as ReactQueryReturnProps
          }
        />
      )}

      <Modal
        size="lg"
        open={modalOpen}
        onClose={closeModal}
        title="Uyarı"
        footer
      >
        <div className="flex flex-col gap-4">
          {bookingRequestStatusMutation.data?.data?.result?.serviceResponse
            ?.errors && (
            <>
              <div className="mt-10 flex items-center justify-center">
                {bookingRequestStatusMutation.data?.data?.result
                  ?.serviceResponse?.errors[0].message ===
                "Strategy access denied" ? (
                  <span>Bu işlem için gerekli yetkiniz bulunmamaktadır.</span>
                ) : (
                  <span>
                    {
                      bookingRequestStatusMutation.data?.data?.result
                        ?.serviceResponse?.errors[0].message
                    }
                  </span>
                )}
              </div>
              <div className="flex justify-end">
                <Button variant="error" onClick={closeModal}>
                  Kapat
                </Button>
              </div>
            </>
          )}
        </div>
      </Modal>
    </div>
  );
};

export default FlightRequestDetail;
