"use client";

import { useState } from "react";

import { Button } from "@/components/ui/button/button";
import { Modal } from "@/components/ui/modal/modal";
import { useModal } from "@/components/ui/modal/modal.hook";
import RentACarRequestDetailWidget from "@/components/widget/rent-a-car-request-detail/rent-a-car-request-detail";
import BookingRequestDetailSkeleton from "@/features/demand/components/demand-detail/components/booking-request-skeleton";
import type { IRentACarBookingRequest } from "@/models/booking-request.model";
import { useBookingCancelReasonQuery } from "@/services/eva-display/book-cancel-reason/booking-cancel-reason-service";
import { useBookingRequestDetailQuery } from "@/services/eva-display/booking-detail-request/booking-detail-service";
import { useBookingRequestStatusMutation } from "@/services/eva-display/demand/booking-request-status/booking-request-status.services";
import { CorpRequestStatusEnum } from "@/services/eva-display/demand/booking-request-status/booking-request-status-services.types";

interface DemandDetailProps {
  id: number;
}
export interface RejectSelectValue {
  value: "";
  name: "";
}
const RentACarRequestDetail = ({ id }: DemandDetailProps) => {
  const bookingRequestDetailQuery = useBookingRequestDetailQuery({
    RequestId: id,
  });
  const bookingCancelReason = useBookingCancelReasonQuery();
  const bookingRequestStatusMutation = useBookingRequestStatusMutation();
  const { closeModal, openModal, modalOpen } = useModal({
    open: false,
  });
  const [rejectSelectValue, setRejectSelectValue] = useState<RejectSelectValue>(
    {
      value: "",
      name: "",
    },
  );
  const { isLoading, isSuccess, isError, data, refetch } =
    bookingRequestDetailQuery;

  const detailData = data as unknown as IRentACarBookingRequest;

  if (!isLoading && detailData?.moduleName !== "CarRental") {
    window.location.href = "/booking-request";
  }
  if (!detailData) return;

  const handleAccept = async () => {
    if (!data) {
      return;
    }
    const resp = await bookingRequestStatusMutation.mutateAsync({
      CorpBaseRequest: {
        CorpRequestStatus: CorpRequestStatusEnum.Approved,
        RequestId: detailData.id,
        ApprovePriceChange: false,
      },
    });

    if (resp.data?.result?.serviceResponse?.errors) {
      openModal();
    } else {
      refetch();
    }
  };

  const handleReject = async () => {
    if (!data) {
      return;
    }

    await bookingRequestStatusMutation.mutateAsync({
      CorpBaseRequest: {
        CorpRequestStatus: CorpRequestStatusEnum.Cancelled,
        RequestId: detailData.id,
      },
    });
    refetch();
  };

  const handleDenied = async () => {
    if (!data) {
      return;
    }

    await bookingRequestStatusMutation.mutateAsync({
      CorpBaseRequest: {
        CorpRequestStatus: CorpRequestStatusEnum.Denied,
        RequestId: detailData.id,
        DenyReasonId: +rejectSelectValue.value,
        StatusChangeDescription: rejectSelectValue.name,
      },
    });
    refetch();
  };

  // eslint-disable-next-line consistent-return
  return (
    <>
      <div>
        {isError && (
          <h1 className="my-5 rounded-lg bg-error-100 p-2 text-error-500">
            Detay bilgileri alinirken bir sorun ile karsilasildi.
          </h1>
        )}
        {isLoading && <BookingRequestDetailSkeleton />}
        {detailData && isSuccess && (
          <RentACarRequestDetailWidget
            data={detailData}
            onAccept={handleAccept}
            onReject={handleReject}
            onDenied={handleDenied}
            rejectSelectValue={rejectSelectValue}
            onRejectSelectValue={setRejectSelectValue}
            bookingCancelReason={bookingCancelReason}
            bookingRequestStatusMutation={bookingRequestStatusMutation}
          />
        )}
      </div>

      <Modal
        size="lg"
        open={modalOpen}
        onClose={closeModal}
        title="Uyarı"
        footer
      >
        <div className="flex flex-col gap-4">
          {bookingRequestStatusMutation.data?.data?.result?.serviceResponse
            ?.errors && (
            <>
              <div className="mt-10 flex items-center justify-center">
                {bookingRequestStatusMutation.data?.data?.result
                  ?.serviceResponse?.errors[0].message ===
                "Strategy access denied" ? (
                  <span>Bu işlem için gerekli yetkiniz bulunmamaktadır.</span>
                ) : (
                  <span>
                    {
                      bookingRequestStatusMutation.data?.data?.result
                        ?.serviceResponse?.errors[0].message
                    }
                  </span>
                )}
              </div>
              <div className="flex justify-end">
                <Button variant="error" onClick={closeModal}>
                  Kapat
                </Button>
              </div>
            </>
          )}
        </div>
      </Modal>
    </>
  );
};

export default RentACarRequestDetail;
