import dayjs from "dayjs";
import { useMediaQuery } from "usehooks-ts";

import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import { Skeleton } from "@/components/ui/skeleton/skeleton";
import {
  FlightBrand,
  FlightPrice,
  FlightTime,
} from "@/components/widget/flight-card/flight-card";
import { getFormatedFlightTime } from "@/libs/helpers/date-time-helper";
import { getAirportCode, getCityName } from "@/libs/helpers/flight-helper";
import { PriceManager } from "@/libs/price-manager/price-manager";
import type { GroupedCombinationGroup } from "@/services/eva-display/flight/flight-search/flight-search-services.types";

import { useFlightListContext } from "../context/flight-list-context";
import FlightListOrderDropdown from "./flight-list-order-dropdown";

export interface FlightListHeaderProps {
  renderHeader?: () => React.ReactNode;
}

const FlightListHeaderOneWay = (props: FlightListHeaderProps) => {
  const {
    data,
    isLoading,
    departureSegmentKey,
    setDepartureSegmentKey,
    paginatedData,
  } = useFlightListContext();
  const isDesktop = useMediaQuery("(min-width: 768px)");

  if (isLoading || !paginatedData) {
    return (
      <Skeleton
        style={{
          height: 60,
        }}
      />
    );
  }

  // groupedCombinationGroupByGroupIds uses always 0 index, because this means is departure flights
  const groupedCombinationGroups = paginatedData?.reduce(
    (prev: GroupedCombinationGroup[], current) => {
      return [
        ...prev,
        ...(current.groupedCombinationGroupByGroupIds?.find(
          (item) => item.groupId === 0,
        )?.groupedCombinationGroups || []),
      ];
    },
    [],
  );

  let combinationGroup = groupedCombinationGroups?.[0]?.combinationGroups?.[0];

  const selectedCombinationGroupIndex = groupedCombinationGroups?.findIndex(
    (x) => x.combinationGroups?.some((y) => y.key === departureSegmentKey),
  );

  const selectedCombinationGroup = groupedCombinationGroups?.[
    selectedCombinationGroupIndex || 0
  ]?.combinationGroups.find((item) => item.key === departureSegmentKey);

  if (selectedCombinationGroup)
    combinationGroup = { ...selectedCombinationGroup };

  const departureSegment = combinationGroup?.flightSegments[0];
  const arrivalSegment =
    combinationGroup?.flightSegments[
      (combinationGroup?.flightSegments?.length || 0) - 1
    ];
  const flightSegments = combinationGroup?.flightSegments.sort(
    (a, b) =>
      dayjs(a.departureTime).valueOf() - dayjs(b.departureTime).valueOf(),
  );

  const renderListHeader = () => {
    return (
      <div className="flex w-full items-center gap-1 text-base font-semibold leading-8 text-gray-800 md:text-2xl">
        <div className="">
          {`${getCityName(
            departureSegmentKey
              ? arrivalSegment?.destination!
              : departureSegment!.origin!,
          )} (${getAirportCode(
            departureSegmentKey
              ? arrivalSegment?.destination!
              : departureSegment!.origin!,
          )})`}
        </div>
        <LIcon name="ArrowRight" height={24} width={24} />
        <div className="">
          {`${getCityName(
            departureSegmentKey
              ? departureSegment!.origin!
              : arrivalSegment?.destination!,
          )} (${getAirportCode(
            departureSegmentKey
              ? departureSegment!.origin!
              : arrivalSegment?.destination!,
          )})`}
        </div>
      </div>
    );
  };

  return (
    <>
      {!props?.renderHeader && departureSegmentKey && (
        <Card id="selectedFlight" className="mb-2">
          <div className="flex items-center justify-between">
            <span className="text-sm font-semibold text-gray-800">
              Gidiş uçuşu seçildi
            </span>
            <span>
              <Button
                variant="primary-text"
                className="!h-auto !p-0"
                onClick={() => setDepartureSegmentKey!("")}
              >
                Değiştir
              </Button>
            </span>
          </div>
          <div className="grid grid-cols-1 py-4 md:grid-cols-3">
            <FlightBrand
              logoSrc={`https://concorecdn.jollytur.com/concore/media/site/assets/img/theme/${
                departureSegment?.marketingAirlineCode || ""
              }-42x42.gif?v1040`}
              brandName={departureSegment?.marketingAirline || ""}
              flightCode={departureSegment?.flightNumber || ""}
            />
            <FlightTime
              totalFlightHours={getFormatedFlightTime(
                combinationGroup?.travelTime || "",
              )}
              flightStartTime={dayjs(departureSegment?.departureTime).format(
                "HH:mm",
              )}
              startIATACode={getAirportCode(departureSegment?.origin || "")!}
              endStartTime={dayjs(arrivalSegment?.arrivalTime).format("HH:mm")}
              endIATACode={getAirportCode(arrivalSegment?.destination || "")!}
              transferInformation={`${
                combinationGroup?.flightSegments?.length || 0 >= 2
                  ? `${
                      combinationGroup?.flightSegments?.length || 1 - 1
                    } Aktarmalı ( ${[
                      ...new Set(
                        (flightSegments || [])
                          ?.filter((_, i) => i !== 0)
                          ?.map((x) => getAirportCode(x.origin)),
                      ),
                    ].join(", ")} )`
                  : "Aktarmasız"
              }, ${
                selectedCombinationGroup?.cabinClassTitle?.toLowerCase() ?? ""
              }`}
            />

            <div className="mt-4 flex justify-end border-t pt-4 md:mt-0 md:justify-end md:border-t-0 md:pt-0">
              <FlightPrice
                price={PriceManager({
                  currency:
                    selectedCombinationGroup?.totalPrice.slice(0, 3) || "",
                  value:
                    +(selectedCombinationGroup?.totalPrice || "").slice(3) /
                    (data?.passengerList?.length || 1),
                })}
                isOptimal
              />
            </div>
            {/* <div className="col-span-5 flex flex-col items-center justify-start gap-1 overflow-hidden">
              <p className="text-xs text-gray-500">
                {getFormatedFlightTime(combinationGroup?.travelTime || "") ||
                  Undefined()}
              </p>
              <p className="text-sm">
                {`${
                  combinationGroup?.flightSegments?.length || 0 >= 2
                    ? `${
                        combinationGroup?.flightSegments?.length || 1 - 1
                      } Aktarmalı ( ${[
                        ...new Set(
                          (flightSegments || [])
                            ?.filter((_, i) => i !== 0)
                            ?.map((x) => getAirportCode(x.origin)),
                        ),
                      ].join(", ")} )`
                    : "Aktarmasız"
                }`}
                <span className="capitalize">
                  {`, ${
                    combinationGroup?.cabinClassTitle?.toLowerCase() ?? ""
                  }`}
                </span>
              </p>
            </div> */}
          </div>
        </Card>
      )}
      <div className="flex justify-between">
        {props.renderHeader ? (
          props.renderHeader()
        ) : (
          <div className="flex w-full flex-col">
            {data !== null &&
              data?.combinationGroups[0]?.flightSegments[0]?.destination &&
              paginatedData.length > 0 &&
              renderListHeader()}
            <div className="mb-4 flex md:mb-0">
              <p className="text-sm font-semibold leading-5 text-gray-800">
                {/* @todo - add translation for these text */}
              </p>
              <p className="w-full text-sm text-gray-500">
                <span className="font-semibold text-gray-800">
                  {departureSegmentKey ? "Dönüş Uçusu" : "Gidiş Uçuşu"}
                </span>{" "}
                {data?.groupedCombinationGroupByGroupIds?.[
                  departureSegmentKey ? 1 : 0
                ]?.totalRecord || 0}{" "}
                Sonuç Bulundu
              </p>
            </div>
          </div>
        )}

        {isDesktop && <FlightListOrderDropdown />}
      </div>
    </>
  );
};

export default FlightListHeaderOneWay;
