import { useMediaQuery } from "usehooks-ts";

import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import { Skeleton } from "@/components/ui/skeleton/skeleton";
import {
  getAirportCode,
  getCityName,
  sortCombinationGroups,
  sortFlightSegments,
} from "@/libs/helpers/flight-helper";

import { useFlightListContext } from "../context/flight-list-context";
import FlightListOrderDropdown from "./flight-list-order-dropdown";

export interface FlightListHeaderProps {
  renderHeader?: () => React.ReactNode;
}

const FlightListHeader = (props: FlightListHeaderProps) => {
  const { data, isLoading } = useFlightListContext();
  const isDesktop = useMediaQuery("(min-width: 768px)");

  if (isLoading) {
    return (
      <Skeleton
        style={{
          height: 60,
        }}
      />
    );
  }

  const renderListHeader = () => {
    const pricingCombination = data?.pricingCombinations[0];

    if (!pricingCombination) return null;

    const combinationGroups = sortCombinationGroups(
      pricingCombination!.combinationGroups,
    )!;

    if (combinationGroups?.length === 0) return null;

    const flightSegments = sortFlightSegments(
      combinationGroups![0]!.flightSegments,
    );

    const departureSegment = flightSegments[0];
    const arrivalSegment = flightSegments[flightSegments.length - 1];

    return (
      <div className="flex w-full items-center gap-1 text-base font-semibold leading-8 text-gray-800 md:text-2xl">
        <div className="">
          {`${getCityName(departureSegment!.origin!)} (${getAirportCode(
            departureSegment!.origin!,
          )})`}
        </div>
        <LIcon name="ArrowRight" height={24} width={24} />
        <div className="">
          {`${getCityName(arrivalSegment?.destination!)} (${getAirportCode(
            arrivalSegment?.destination!,
          )})`}
        </div>
      </div>
    );
  };

  return (
    <div className="flex justify-between">
      {props.renderHeader ? (
        props.renderHeader()
      ) : (
        <div className="flex w-full flex-col">
          {data !== null &&
            data?.combinationGroups[0]?.flightSegments[0]?.destination &&
            renderListHeader()}
          <div className="mb-4 flex md:mb-0">
            <p className="text-sm font-semibold leading-5 text-gray-800">
              {/* @todo - add translation for these text */}
            </p>
            <p className="w-full text-sm text-gray-500">
              {data?.totalFlightCount || 0} Sonuç Bulundu
            </p>
          </div>
        </div>
      )}

      {isDesktop && <FlightListOrderDropdown />}
    </div>
  );
};

export default FlightListHeader;
