import { useRouter } from "next/navigation";
import React from "react";

import ListSkeleton from "@/components/widget/list-skeleton/list-skeleton";
import { localeStorageHelper } from "@/libs/helpers/locale-storage-helper";
import { ILoaderKeys } from "@/models/loader.model";
import { useFlightDetailMutation } from "@/services/eva-display/flight/flight-detail/flight-detail.service";
import type {
  GroupedCombinationGroup,
  PricingCombination,
} from "@/services/eva-display/flight/flight-search/flight-search-services.types";
import type {
  GroupedPricingCombinationWithCombinationGroups,
  IFlightSearchResultSerializer,
  PricingCombinationWithCombinationGroups,
} from "@/services/eva-display/flight/flight-search/flight-search-services-serializer";
import { useLoaderStore } from "@/store/loader-store/use-loader-store";
import { useNotifyStore } from "@/store/notify-store/use-notify-store";

import { useFlightListContext } from "../context/flight-list-context";
import FlightListAppliedFilters from "./flight-list-applied-filters";
import type { FlightListHeaderProps } from "./flight-list-header";
import FlightListHeader from "./flight-list-header";
import FlightListHeaderOneWay from "./flight-list-header-one-way";
import { FlightListGroupItem } from "./flight-list-item/flight-list-group-item";
import FlightListLoadMoreButton from "./flight-list-load-more-button";
import FlightListNoData from "./flight-list-no-data";

interface FlightListResultProps {
  headerProps?: FlightListHeaderProps;
  onItemSelect?: (
    item: PricingCombinationWithCombinationGroups,
    variables: {
      productKey: string;
      searchId: string;
      passengers: IFlightSearchResultSerializer["passengerList"];
      combinationGroupKeys?: string[];
    },
  ) => void;
}

const FlightListResult = (props: FlightListResultProps) => {
  const router = useRouter();
  const { setNotify } = useNotifyStore();
  const { openLoader, removeLoader } = useLoaderStore();
  const {
    data,
    isSuccess,
    isLoading,
    isFetching,
    paginatedData,
    requestVariables,
    departureSegmentKey,
    setDepartureSegmentKey,
  } = useFlightListContext();
  const flightDetailMutation = useFlightDetailMutation({
    onSuccess: (response) => {
      if (!response.success) {
        if (response.errors && response.errors.length > 0) {
          setNotify({
            title: "Bir Hata Oluştu",
            message: response.errors[0]?.message ?? "",
            variant: "error",
          });
        }
        removeLoader(ILoaderKeys.FlightDetailLoaderKey);
        return;
      }

      const item = response.result;

      if (item.pricingCombinations?.length > 0) {
        localeStorageHelper.set("flight-cart", {
          passengers: data?.passengerList,
          searchId: data?.metadata.searchId,
          productKey: item.pricingCombinations[0]!.key,
          pricingCombinationKeys: [
            ...item.pricingCombinations.map((x) => x.key),
          ],
          isAbroad: Boolean(data?.metadata.isAbroad),
          combinationGroupKeys:
            item.pricingCombinations?.length > 1
              ? item.pricingCombinations
                  .map((x) => x.combinationGroups.map((y) => y.key).flat())
                  .flat()
              : item.pricingCombinations[0]!.combinationGroups.map(
                  (y) => y.key,
                ).flat(),
        });
        router.push("/flight-summary");
      }
    },
  });

  const listData = paginatedData?.reduce(
    (prev: GroupedPricingCombinationWithCombinationGroups[], current) => {
      return [...prev, ...current.pricingCombinationGroups];
    },
    [],
  );

  const listDataOneWay = paginatedData?.reduce(
    (prev: GroupedCombinationGroup[], current) => {
      return [
        ...prev,
        ...(current.groupedCombinationGroupByGroupIds?.find(
          (item) => item.groupId === (departureSegmentKey ? 1 : 0),
        )?.groupedCombinationGroups || []),
      ];
    },
    [],
  );

  return (
    <div className="flex grow flex-col gap-2">
      <div className="flex grow flex-col gap-2">
        {(isLoading || isFetching) && <ListSkeleton size={5} />}
        {/* If this is not Round-Trip type, it may be visible */}
        {requestVariables?.FlightDisplayType !== 1 &&
          isSuccess &&
          !(isLoading || isFetching) &&
          data &&
          (listData || [])?.length > 0 && (
            <>
              <FlightListHeader {...props.headerProps} />
              <FlightListAppliedFilters />
              {listData?.map((combinationItem) => (
                <FlightListGroupItem
                  key={combinationItem.key}
                  item={combinationItem}
                  passengerCount={data.passengerList.length}
                  onPricingCombinationSelect={(pricingCombination) => {
                    if (props.onItemSelect) {
                      props.onItemSelect(pricingCombination, {
                        productKey: pricingCombination.key,
                        searchId: data?.metadata.searchId,
                        passengers: data?.passengerList,
                      });
                      return;
                    }
                    openLoader({
                      key: ILoaderKeys.FlightDetailLoaderKey,
                      open: true,
                    });
                    // Main flight list flow is using this block.
                    if (pricingCombination.combinationGroups.length > 0) {
                      flightDetailMutation.mutateAsync({
                        FlightDetailRequest: {
                          ProductKey: pricingCombination.key,
                          searchId: data?.metadata.searchId,
                        },
                      });
                    }
                  }}
                />
              ))}
              <div className="flex flex-col items-center gap-2 text-sm text-gray-800">
                <span>
                  {data.totalFlightCount || 0} sonuçtan {listData?.length}{" "}
                  tanesini görüntülediniz
                </span>
              </div>
              <FlightListLoadMoreButton />
            </>
          )}
        {/* New type one way display method */}
        {requestVariables?.FlightDisplayType === 1 &&
          isSuccess &&
          data &&
          (listDataOneWay || [])?.length > 0 && (
            <>
              <FlightListHeaderOneWay {...props.headerProps} />{" "}
              <FlightListAppliedFilters />
              {listDataOneWay?.map((combinationItem) => (
                <FlightListGroupItem
                  key={combinationItem?.combinationGroups?.[0]?.key || ""}
                  item={{
                    key: combinationItem?.combinationGroups?.[0]?.key || "",
                    pricingCombinations: combinationItem?.combinationGroups.map(
                      (group) => ({
                        ...(group as unknown as PricingCombination),
                        combinationGroups: [group],
                        combinationGroupKeys: [group.key],
                      }),
                    ),
                  }}
                  passengerCount={data.passengerList.length}
                  onPricingCombinationSelect={(pricingCombination) => {
                    // One Way Flight List flow is using this block.
                    if (!departureSegmentKey) {
                      setDepartureSegmentKey!(pricingCombination.key);
                      setTimeout(() => {
                        const selectedElement =
                          document.getElementById("selectedFlight");
                        if (selectedElement) {
                          selectedElement.scrollIntoView({
                            behavior: "smooth",
                          });
                        }
                      }, 150);
                    } else if (departureSegmentKey) {
                      if (props.onItemSelect) {
                        props.onItemSelect(pricingCombination, {
                          productKey: "",
                          searchId: data?.metadata.searchId,
                          passengers: data?.passengerList,
                          combinationGroupKeys: [
                            departureSegmentKey,
                            pricingCombination.key,
                          ],
                        });
                        return;
                      }
                      openLoader({
                        key: ILoaderKeys.FlightDetailLoaderKey,
                        open: true,
                      });
                      flightDetailMutation.mutateAsync({
                        FlightDetailRequest: {
                          ProductKey: "",
                          searchId: data?.metadata.searchId,
                          CombinationGroupKeys: [
                            departureSegmentKey,
                            pricingCombination.key,
                          ],
                        },
                      });
                    }
                  }}
                />
              ))}
              <div className="flex flex-col items-center gap-2 text-sm text-gray-800">
                <span>
                  {/* Toplam sonuç sayısı */}
                  {data.groupedCombinationGroupByGroupIds?.[
                    departureSegmentKey ? 1 : 0
                  ]?.totalRecord || 0}{" "}
                  sonuçtan&nbsp;
                  {/* Listelenen sonuç sayısı */}
                  {listDataOneWay?.length || 0} tanesini görüntülediniz
                </span>
              </div>
              {(listDataOneWay?.length || 0) <
                (data.groupedCombinationGroupByGroupIds?.[
                  departureSegmentKey ? 1 : 0
                ]?.totalRecord || 0) && <FlightListLoadMoreButton />}
            </>
          )}
        {!isLoading &&
          !isFetching &&
          listData?.length === 0 &&
          listDataOneWay?.length === 0 && <FlightListNoData />}
      </div>
    </div>
  );
};

export default FlightListResult;
