/* eslint-disable no-unsafe-optional-chaining */

"use client";

import { useRef } from "react";
import { useUpdateEffect } from "usehooks-ts";

import Container from "@/components/shared/container";
import type { MenuItem } from "@/components/widget/hotel-product-detail/hotel-product-detail-menu/hotel-product-detail-menu";
import { HotelProductDetailMenu } from "@/components/widget/hotel-product-detail/hotel-product-detail-menu/hotel-product-detail-menu";
import { HotelOtherInfoType } from "@/components/widget/hotel-product-detail/hotel-product-detail-other-info/hotel-product-detail-other-info";
import ProductDetailCoverGallery from "@/components/widget/product-detail/product-detail-cover-gallery/product-detail-cover-gallery";
import ProductDetailGallery from "@/components/widget/product-detail/product-detail-gallery/product-detail-gallery";
import { useCoreSummary } from "@/features/core/summary/hooks/use-core-summary";
import { useHotelSummary } from "@/features/modules/hotel/summary/hooks/use-hotel-summary";
import Room from "@/features/room";
import { usePageInfo } from "@/hooks/use-page-info";
import { useApp } from "@/hooks/useApp";
import { useBreakpoints } from "@/hooks/useBreakpoints";
import { useLocale } from "@/hooks/useLocale";
import { localeStorageHelper } from "@/libs/helpers/locale-storage-helper";
import { ILoaderKeys } from "@/models/loader.model";
import { useHotelDetailListSearchMutation } from "@/services/eva-display/hotel/hotel-detail/hotel-detail-services";
import { usePureHotelListSearchQuery } from "@/services/eva-display/hotel/hotel-search/hotel-search.services";
import type { IHotelSearchRequest } from "@/services/eva-display/hotel/hotel-search/hotel-search-services.types";
import { useLoaderStore } from "@/store/loader-store/use-loader-store";

import HotelDetailAbout from "./components/hotel-detail-about";
import HotelDetailEvents from "./components/hotel-detail-events";
import HotelDetailTitle, {
  HotelDetailTitleSubBar,
} from "./components/hotel-detail-header";
import HotelDetailOtherInfo from "./components/hotel-detail-other-info";
import HotelDetailOverview from "./components/hotel-detail-overview";
import HotelDetailSearchNotFound from "./components/hotel-detail-search-not-found";
import HotelDetailServicesAndFacilities from "./components/hotel-detail-services-and-facilities";
import HotelDetailSkeleton from "./components/hotel-detail-skeleton";

interface HotelDetailProps {
  parameters: IHotelSearchRequest;
}

const HotelDetail = (props: HotelDetailProps) => {
  const { info } = usePageInfo();
  const { isB2C } = useApp();
  const { isDesktop } = useBreakpoints();
  const { resetCoreSummary } = useCoreSummary();
  const { setSearchData } = useHotelSummary();
  const { removeLoader } = useLoaderStore();
  const hotelDetailMutation = useHotelDetailListSearchMutation({
    onSuccess() {
      // eslint-disable-next-line @typescript-eslint/no-use-before-define
      removeLoader(ILoaderKeys.HotelDetailLoaderKey);
    },
    onError() {
      removeLoader(ILoaderKeys.HotelDetailLoaderKey);
    },
  });
  const { t } = useLocale();
  const headerWrapperRef = useRef<HTMLDivElement>(null);
  const menuWrapperRef = useRef<HTMLDivElement>(null);

  const includedSpesification = hotelDetailMutation?.data
    ? hotelDetailMutation.data.roomGroupResult.reduce((prev, curr) => {
        const campaignInformations = curr?.campaignInformations || [];
        return [
          ...campaignInformations.filter(
            (item) => !prev.some((prevItem: any) => prevItem.key === item.key),
          ),
          ...prev,
        ];
      }, [] as any)
    : [];

  const hoteListQuery = usePureHotelListSearchQuery(props.parameters, {
    onSuccess: (data) => {
      resetCoreSummary();
      setSearchData({
        searchId: data.metadata.searchId,
        isAbroad: data.hotelList[0]?.passportRequired ?? false,
        searchItemKey: data.hotelList[0]?.key as string,
        passengersKeys: data.passengerList.map((item) => ({
          age: item.age,
          key: item.key,
          type: item.type,
        })),
      });
      localeStorageHelper.set("passengers", {
        passengers: data.passengerList,
        searchProductKey: data.hotelList?.[0]?.key,
      });
      hotelDetailMutation.mutate({
        searchId: data.metadata.searchId,
        ProductKey: data.hotelList?.[0]?.key as string,
      });
    },
    onError() {
      removeLoader(ILoaderKeys.HotelDetailLoaderKey);
    },
  });

  function getMenuItem() {
    const temp: MenuItem[] = [];
    temp.push({ text: t("Overview"), id: "overview" });
    temp.push({ text: t("Rooms"), id: "rooms" });

    if (
      hotelDetailMutation.isSuccess &&
      hotelDetailMutation.data &&
      hotelDetailMutation.data.seasons.find((item) => item.facilities?.length)
    ) {
      temp.push({ text: t("Facilities"), id: "facilities" });
    }

    const eatingDringkingElemId = `${HotelOtherInfoType.EatingDrinking}-accordion`;
    const hasEatingDringkingElem = document.getElementById(
      eatingDringkingElemId,
    );
    if (hasEatingDringkingElem) {
      temp.push({
        text: t("Eat/Drinks"),
        id: eatingDringkingElemId,
      });
    }

    if (hotelDetailMutation?.data?.hotelAddons) {
      temp.push({ text: t("Events"), id: "events" });
    }

    temp.push({ text: t("About"), id: "about" });
    let otherInformation = false;
    if (hotelDetailMutation.isSuccess && hotelDetailMutation.data) {
      hotelDetailMutation.data.seasons.forEach((item: any) => {
        if (item.descriptions?.length > 0) {
          item.descriptions.forEach((description: any) => {
            if (description.descriptionTypeId !== 1) {
              if (description.description !== "") {
                otherInformation = true;
              }
            }
          });
        }
      });
      if (otherInformation) {
        temp.push({ text: t("Other informations"), id: "otherInformation" });
      }
    }

    return temp;
  }

  function getInformationText() {
    if (hotelDetailMutation.isSuccess && hotelDetailMutation.data) {
      const data = hotelDetailMutation.data.seasons.find(
        (item) => item.isCurrent === true,
      );
      if (data?.descriptions) {
        return (
          data.descriptions.find((item) => item.descriptionTypeId === 1)
            ?.description ?? ""
        );
      }
      return "";
    }
    return "";
  }

  useUpdateEffect(() => {
    if (info?.queryParams) {
      hotelDetailMutation.reset();

      hoteListQuery.refetch();
    }
  }, [info]);

  const headerWrapper = headerWrapperRef.current;
  const menuWrapper = menuWrapperRef.current;

  if (isDesktop && headerWrapper && menuWrapper) {
    // note: why I did this?
    // hotel title word length is no matter what,
    // menu wrapper's top value'll be equal according to title wrapper height
    menuWrapperRef.current.style.top = `${headerWrapper.offsetHeight}px`;
  }

  return (
    <div className="flex flex-col">
      {hoteListQuery.isError && <HotelDetailSearchNotFound />}
      {hotelDetailMutation.isLoading && !hotelDetailMutation.isSuccess && (
        <HotelDetailSkeleton />
      )}
      {hotelDetailMutation.isSuccess && hotelDetailMutation.data && (
        <>
          <div
            ref={headerWrapperRef}
            className="top-0 z-50 flex w-full flex-col bg-white md:sticky md:flex-col-reverse"
          >
            <div className="z-50 !mx-0 w-full bg-white">
              {isB2C && !isDesktop && (
                <Container>
                  <ProductDetailGallery
                    images={hotelDetailMutation.data.images}
                  />
                </Container>
              )}
              <div className="mt-4 md:mt-10">
                <HotelDetailTitle
                  title={hotelDetailMutation.data.metadata.name}
                />
              </div>
            </div>
          </div>
          <HotelDetailTitleSubBar
            hasHygienicalMeasure={
              hotelDetailMutation.data?.hygienicalMeasures?.length > 0
            }
            nightlyRateDescription={
              hotelDetailMutation.data.roomCollectionResult?.length > 0
                ? hotelDetailMutation.data.roomCollectionResult[0]?.rooms[0]
                    ?.nightlyRateDescription
                : ""
            }
            price={
              !hoteListQuery.data?.hotelList?.[0]?.onDemand
                ? hoteListQuery.data?.hotelList?.[0]?.totalPrice
                : null
            }
            recommended={hoteListQuery.data?.hotelList?.[0]?.recommended}
            mealTypes={hoteListQuery.data?.hotelList?.[0]?.mealTypes}
          />
          <div className="hidden w-full bg-white pt-4 md:block">
            <ProductDetailCoverGallery
              images={hotelDetailMutation.data.images}
              noImageText={t(
                "This hotel's images are in the process of being prepared.",
              )}
            />
          </div>
          <div
            ref={menuWrapperRef}
            className="sticky top-0 z-50 !mx-0 w-full bg-white"
          >
            <Container>
              <HotelProductDetailMenu menuItems={getMenuItem()} />
            </Container>
          </div>
          <Container className="px-0 md:px-5">
            <div className="my-4">
              <HotelDetailOverview
                informationText={getInformationText()}
                season={hotelDetailMutation.data.seasons}
                address={hotelDetailMutation.data.generalInformation.address}
                nearDistances={hotelDetailMutation.data.nearDistances}
                includedSpecifications={includedSpesification}
                documentNumber={
                  hoteListQuery.data?.hotelList?.[0]?.documentNumber ?? ""
                }
              />
            </div>
          </Container>

          <Container className="px-0 md:px-5">
            <div className="my-4">
              <Room
                searchData={hoteListQuery.data}
                data={hotelDetailMutation.data.roomCollectionResult}
                roomGroupResult={hotelDetailMutation.data.roomGroupResult}
                searchId={hoteListQuery.data?.metadata.searchId as string}
              />
            </div>
          </Container>
          <Container className="px-0 md:px-5">
            <div className="my-4">
              <HotelDetailServicesAndFacilities
                seasons={hotelDetailMutation.data.seasons}
              />
            </div>
          </Container>

          <Container className="px-0 md:px-5">
            <div className="my-4">
              <HotelDetailEvents items={hotelDetailMutation.data.hotelAddons} />
            </div>
          </Container>
          <Container className="px-0 md:px-5">
            <div className="my-4">
              <HotelDetailAbout
                informationText={getInformationText()}
                name={hotelDetailMutation.data.metadata.name}
                generalInformation={hotelDetailMutation.data.generalInformation}
                hotelThemes={hoteListQuery.data?.filterResult.themes}
                documentNumber={
                  hoteListQuery.data?.hotelList?.[0]?.documentNumber ?? ""
                }
              />
            </div>
          </Container>
          <Container className="px-0 md:px-5">
            <div className="my-4 overflow-x-hidden">
              <HotelDetailOtherInfo season={hotelDetailMutation.data.seasons} />
            </div>
          </Container>
        </>
      )}
    </div>
  );
};

export default HotelDetail;
