import Link from "next/link";
import { useId } from "react";

import { Button } from "@/components/ui/button/button";
import HotelCard from "@/components/widget/hotel-card/hotel-card";
import { useHotelTravelPlanner } from "@/features/modules/hotel/travel-planner/hooks/use-hotel-travel-planner";
import { useApp } from "@/hooks/useApp";
import { useLocale } from "@/hooks/useLocale";
import { getLoyaltyAddon } from "@/libs/helpers/addon-helper";
import { PriceManager } from "@/libs/price-manager/price-manager";
import { cn } from "@/libs/utils";
import { useServerDataCtx } from "@/providers/server-data-provider/server-data-provider";
import type { IHotelListItem } from "@/services/eva-display/hotel/hotel-search/hotel-search-services.types";

interface HotelListItemProps {
  item: IHotelListItem;
  className?: string;
  swiperContainerModifierClass?: string;
  renderPrimaryButton?(): any;
  onClickPrimaryButton?(item: any): any;
}

const HotelListItem = ({
  item,
  className,
  swiperContainerModifierClass,
  onClickPrimaryButton,
  renderPrimaryButton,
}: HotelListItemProps) => {
  const { isB2E } = useApp();
  const id = useId();
  const { t } = useLocale();
  const { addonList } = useServerDataCtx();

  const { data, navigateToHotelList, getLinkGenerator } =
    useHotelTravelPlanner();

  function getSpecifications() {
    const Specifications = [];
    if (item.cancellationPolicy) {
      Specifications.push({ text: item.cancellationPolicy.description });
    }
    if (item.campaignInformations) {
      item.campaignInformations.forEach((subItem) => {
        Specifications.push({
          text: subItem.description,
          icon: subItem?.icon,
          tooltip: subItem?.tooltip,
        });
      });
    }
    if (item.priceDifferenceBackGuarantee) {
      Specifications.push({ text: t("Price protection policy") });
    }
    if (item.availableInstallment && !isB2E) {
      Specifications.push({ text: item.availableInstallment });
    }

    return Specifications;
  }

  const Specifications = getSpecifications();

  const itemBasePrice = item.basePrice.slice(3);
  const itemTotalPrice = item.totalPrice.slice(3);
  const itemTotalPriceWithDiscount = item.totalPriceWithDiscount.slice(3);

  const loyaltyAddon = getLoyaltyAddon(addonList);

  const handleClick = () => {
    if (data.payload) {
      navigateToHotelList({
        ...data.payload,
        destination: {
          id: +item.hotelId,
          value: item.name.toString(),
          type: 0,
          typeName: "otel",
          destinationBreadCrumb: "otel",
        },
      });
    }
  };

  const itemLink = getLinkGenerator({
    destination: {
      id: +item.hotelId,
      value: item.name.toString(),
      type: 0,
      typeName: "otel",
      destinationBreadCrumb: "otel",
    },
  });

  return (
    <div id={id}>
      <HotelCard
        loyaltyValue={
          loyaltyAddon && !item.onDemand
            ? item.loyaltyItem?.loyalities[0]?.value
            : undefined
        }
        availableDateButton={itemBasePrice === "0" || item.onDemand === true}
        className={cn(className)}
        id={item.hotelId.toString()}
        mainSpesisification={item?.mealTypes && item.mealTypes[0]?.description}
        hotelName={item.name}
        images={item.images?.map((imgItem) => imgItem.url)}
        durationNight={item.accommodationDayCountInfo}
        price={
          itemBasePrice !== "0" && !item.onDemand
            ? (PriceManager({
                currency: item.totalPrice.slice(0, 3),
                value: +itemTotalPrice,
              }) as unknown as number)
            : undefined
        }
        includedSpecifications={Specifications}
        discountInformations={item?.discountInformations}
        hasHygienicalMeasure={item.hasHygienicalMeasure}
        hotelLocation={item.location.replaceAll(", ", " / ")}
        isRecommended={item.recommended}
        discount={
          item.discountRate > 0
            ? {
                rate: item.discountRate,
                price: PriceManager({
                  currency: item.totalPriceWithDiscount.slice(0, 3),
                  value: +itemTotalPriceWithDiscount,
                }) as unknown as number,
              }
            : undefined
        }
        SpecificationError={
          itemBasePrice === "0" || item.onDemand
            ? t("There are no rooms left for the dates you selected.")
            : undefined
        }
        renderPrimaryButton={renderPrimaryButton}
        renderHotelSelectButton={() => (
          <Link href={itemLink} target="_blank" className="select-none">
            <Button variant="primary" className="!hidden w-[105px] sm:!block">
              {t("View")}
            </Button>
          </Link>
        )}
        swiperContainerModifierClass={
          swiperContainerModifierClass || `slider-${item?.hotelId}`
        }
        onClickPrimaryButton={() =>
          onClickPrimaryButton ? onClickPrimaryButton(item) : handleClick()
        }
      />
    </div>
  );
};

export default HotelListItem;
