/* eslint-disable no-unsafe-optional-chaining */
/* eslint-disable no-restricted-globals */
import type { PropsWithChildren } from "react";

import { Skeleton } from "@/components/ui/skeleton/skeleton";
import {
  EarlyBookingDiscount,
  RezervationSummaryDivider,
  RezervationSummaryFlightInfo,
  RezervationSummaryFlightTotalPrice,
  RezervationSummaryProvider,
  RezervationSummaryTitle,
} from "@/components/widget/rezervation-summary/rezervation-summary";
import { CoreSummaryAdditionalPrices } from "@/features/core/summary/components/core-summary-additional-prices";
import { useCoreSummaryCtx } from "@/features/core/summary/context/core-summary-context";
import { useSummaryCtxState } from "@/features/modules/hotel/summary/context/summary-context";
import {
  DateHelper,
  getFormatedFlightTime,
} from "@/libs/helpers/date-time-helper";
import {
  getAirportCode,
  sortCombinationGroups,
  sortFlightSegments,
} from "@/libs/helpers/flight-helper";
import { PriceManager } from "@/libs/price-manager/price-manager";
import type { IFlightDetailResponse } from "@/services/eva-display/flight/flight-detail/flight-detail.service";

const FlightSummaryReservationInfo = (
  props: PropsWithChildren<{
    detailData: IFlightDetailResponse;
  }>,
) => {
  const combinationGroups = sortCombinationGroups(
    props.detailData.pricingCombinations.length > 1
      ? props.detailData.pricingCombinations.flatMap(
          (pricingCombination) => pricingCombination.combinationGroups,
        )
      : (props.detailData.pricingCombinations[0]?.combinationGroups ?? []),
  );

  const summaryState = useSummaryCtxState();
  const {
    state: { discounts },
  } = useCoreSummaryCtx();

  if (!combinationGroups) {
    return (
      <div>
        <Skeleton className="h-[100px]" />
      </div>
    );
  }

  const mainCurrency =
    props.detailData.pricingCombinations[0]!.totalPrice.slice(0, 3);

  const totaledPrice =
    props.detailData.pricingCombinations?.length > 1
      ? props.detailData.pricingCombinations.reduce((acc, curr) => {
          return acc + +curr.totalPrice.slice(3);
        }, 0)
      : +props.detailData.pricingCombinations[0]!.totalPrice.slice(3);

  const totalPrice = PriceManager({
    currency: mainCurrency,
    value:
      totaledPrice -
      (isNaN(+summaryState.campaing?.discountAmount.slice(3))
        ? 0
        : +summaryState.campaing?.discountAmount.slice(3)) -
      discounts.reduce((acc, curr) => acc + +curr.price, 0),
  });

  const isRoundTrip =
    props.detailData.pricingCombinations[0]!.combinationGroups.length > 1 ||
    props.detailData.pricingCombinations?.length > 1;

  return (
    <RezervationSummaryProvider>
      <RezervationSummaryTitle
        title={isRoundTrip ? "Gidiş Dönüş" : "Tek Yön"}
      />
      <RezervationSummaryDivider />
      {combinationGroups.map((combinationGroup) => {
        const flightSegmentsItem = sortFlightSegments(
          combinationGroup.flightSegments,
        );
        const originSegmentItem = flightSegmentsItem[0]!;
        const departureSegmentItem =
          flightSegmentsItem[flightSegmentsItem.length - 1]!;
        return (
          <RezervationSummaryFlightInfo
            key={combinationGroup.key}
            destination={getAirportCode(departureSegmentItem.destination) ?? ""}
            origin={getAirportCode(originSegmentItem.origin) ?? ""}
            date={DateHelper.formatDateToString(
              originSegmentItem.departureTime as unknown as Date,
            )}
            time={`${DateHelper.dateToTime(
              originSegmentItem.departureTime as unknown as Date,
            )} - ${DateHelper.dateToTime(
              departureSegmentItem.arrivalTime as unknown as Date,
            )}`}
            flightTime={
              getFormatedFlightTime(combinationGroup.travelTime ?? "") ?? ""
            }
            operatingAirline={
              combinationGroup.flightSegments[0]?.operatingAirline ?? ""
            }
            flightNumber={
              combinationGroup.flightSegments[0]?.flightNumber ?? ""
            }
          />
        );
      })}
      <RezervationSummaryDivider />
      <CoreSummaryAdditionalPrices />
      {Boolean(summaryState.campaing) &&
        summaryState.campaing?.name &&
        summaryState.campaing?.discountAmount && (
          <EarlyBookingDiscount
            key={summaryState.campaing?.name}
            price={PriceManager({
              currency: summaryState.campaing?.discountAmount.slice(0, 3),
              // eslint-disable-next-line no-unsafe-optional-chaining
              value: +summaryState.campaing?.discountAmount.slice(3),
            })}
            title={summaryState.campaing?.name}
          />
        )}
      {/* TODO:Total price ile toplamları eşit gelmiyor be problemi fixlenirse burası gösterilir */}
      {/* <RezervationSummaryFlightPrice
      priceDescription={[
        {
          description: "Uçuş Bedeli",
          price:
            props.detailData.pricingCombinations[0]?.basePrice ?? "",
        },
        {
          description: "Vergi",
          price: props.detailData.pricingCombinations[0]?.taxes ?? "",
        },
      ]}
    /> */}
      <RezervationSummaryFlightTotalPrice price={totalPrice} />
      {props.children}
    </RezervationSummaryProvider>
  );
};

export default FlightSummaryReservationInfo;
