/* eslint-disable react-hooks/exhaustive-deps */
/* eslint-disable radix */
import dayjs from "dayjs";
import { isEmpty } from "lodash-es";
import { useEffect, useState } from "react";

import { usePageInfo } from "@/hooks/use-page-info";
import useRoom from "@/hooks/use-room/useRoom";
import type { RoomType } from "@/hooks/use-room/useRoom.types";
import { DateHelper } from "@/libs/helpers/date-time-helper";
import { storageManager } from "@/libs/storage-manager";
import { useServerDataCtx } from "@/providers/server-data-provider/server-data-provider";
import type { IHotelRoomSearchRequest } from "@/services/eva-display/hotel/hotel-room-search/hotel-room-search-services.types";
import { useLocalizationStore } from "@/store/localization-store/use-localization-store";
import { useTravelPlannerStore } from "@/store/travel-planner-store/use-travel-planner-store";

import type { IHotelTravelPlannerPayload } from "../../travel-planner/types/hotel-travel-planner.types";

type SearchPayload = Partial<
  Pick<IHotelTravelPlannerPayload, "rooms" | "dates" | "destinationId">
>;

const defaultRoomsDefinition: RoomType[] = [
  {
    passenger: [
      {
        count: 2,
      },
    ],
  },
];

export const useHotelWidgetTravelPlanner = () => {
  const { setModuleData, travelPlannerState, setData } =
    useTravelPlannerStore();
  const { setQueryParams, info, isDone } = usePageInfo();
  const [state, setState] = useState({
    isReady: false,
  });
  const { clientConfig } = useServerDataCtx();
  const isSingleFacility = clientConfig?.domainProducts.length === 1;
  const {
    tp: { roomsToString, roomsFromString },
    roomsFromString: roomsFromStringB2C,
    getRoom,
  } = useRoom();
  const { localizationState } = useLocalizationStore();

  function setSearchData(payload: SearchPayload) {
    storageManager.set("core-travel-planner", {
      moduleName: "Hotel",
      payload,
    });
    setModuleData({
      moduleName: "Hotel",
      payload,
    });
  }

  function navigateToRoomList(payload: SearchPayload) {
    setSearchData(payload);

    setQueryParams(
      {
        destinationId: payload.destinationId?.toString(),
        checkinDate: DateHelper.isoStringToFullYearString(
          payload.dates!.checkIn!,
          "-",
        ),
        checkoutDate: DateHelper.isoStringToFullYearString(
          payload.dates!.checkOut!,
          "-",
        ),
        rooms: roomsToString(payload.rooms!),
      },
      "/",
      "_self",
    );
  }

  function getSearchPayloadFromQueryParams() {
    const { queryParams } = info!;
    if (isEmpty(Object.entries(queryParams)) || !queryParams?.checkinDate) {
      return null;
    }
    const storagedTpData = storageManager.get("core-travel-planner");
    const payload: SearchPayload = {
      ...storagedTpData?.payload,
      destinationId: queryParams.destinationId ?? "",
      dates: {
        checkIn: DateHelper.dateToISOString(
          DateHelper.dayjs(queryParams.checkinDate as any)
            .utc(true)
            .toDate(),
        ),
        checkOut: DateHelper.dateToISOString(
          DateHelper.dayjs(queryParams.checkoutDate as any)
            .utc(true)
            .toDate(),
        ),
      },
      rooms: roomsFromString(queryParams.rooms ?? ""),
    };

    if (payload) {
      setSearchData(payload);
    }

    return payload;
  }

  function getRoomSearchPayload(): IHotelRoomSearchRequest | null {
    if (!info) return null;
    const { queryParams } = info;

    if (
      !isSingleFacility &&
      (!queryParams.destinationId || !queryParams.rooms)
    ) {
      return null;
    }

    const domainProduct = clientConfig?.domainProducts[0];

    const checkInDate = queryParams.checkinDate
      ? DateHelper.dateToISOString(
          dayjs(queryParams.checkinDate).utc(true).toDate(),
        )
      : null;

    const checkOutDate = queryParams.checkoutDate
      ? DateHelper.dateToISOString(
          dayjs(queryParams.checkoutDate ?? "")
            .utc(true)
            .toDate(),
        )
      : null;

    const tpRooms = roomsFromString(queryParams.rooms!);

    const rooms =
      tpRooms.length === 0 && isSingleFacility
        ? getRoom(defaultRoomsDefinition)
        : getRoom(roomsFromStringB2C(queryParams.rooms!));

    const destinationId = +(
      queryParams?.destinationId ||
      domainProduct?.id ||
      0
    );

    const payload: IHotelRoomSearchRequest = {
      searchId: null,
      requestKey: null,
      countryCode: "TR",
      currency: localizationState.data.currency ?? "TRY",
      checkInDate,
      checkOutDate,
      pageSize: 1,
      pageNumber: 1,
      orderBy: "Average",
      destinationId,
      destinationType: 0,
      destinationSubType: 1,
      filter: {
        hotelName: "",
        facilities: [],
        locations: [],
        mealTypes: [],
        themes: [],
      },
      rooms,
    };

    if (
      isSingleFacility &&
      !queryParams.checkinDate &&
      !queryParams.checkoutDate
    ) {
      storageManager.set("core-travel-planner", {
        payload: {
          moduleName: "Hotel",
          dates: {
            checkIn: checkInDate,
            checkOut: checkOutDate,
          },
          rooms: defaultRoomsDefinition,
          destinationId,
        },
      });
    }

    return payload;
  }

  useEffect(() => {
    if (isDone && travelPlannerState?.data?.moduleName !== "Hotel") {
      setData(undefined);
    }
    if (isDone && travelPlannerState.status === "done") {
      if (info?.path === "/" || info?.path === "") {
        getSearchPayloadFromQueryParams();
      }
      setState(() => ({
        isReady: true,
      }));
    }
  }, [travelPlannerState.status, isDone]);

  return {
    navigateToRoomList,
    getSearchPayloadFromQueryParams,
    getRoomSearchPayload,
    data: travelPlannerState.data as {
      moduleName: "Hotel";
      payload: IHotelTravelPlannerPayload;
    },
    isReady: state.isReady,
  };
};
