import dayjs from "dayjs";
import React, { useMemo } from "react";
import { useForm } from "react-hook-form";

import Form from "@/components/form/form";
import FormItem from "@/components/form/form-item";
import { Button } from "@/components/ui/button/button";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import {
  Select,
  SelectContent,
  SelectItem,
  SelectTrigger,
  SelectValue,
} from "@/components/ui/select";
import {
  DefaultPBTpPassenger,
  DefaultRBTpPassenger,
} from "@/data/default/tp-passenger";
import { TpDateRangePickerWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-date-range-picker-form";
import { TpRoomSelectWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-room-select-form";
import { useApp } from "@/hooks/useApp";
import { useLocale } from "@/hooks/useLocale";
import { DateHelper } from "@/libs/helpers/date-time-helper";
import { cn } from "@/libs/utils";
import { ILoaderKeys } from "@/models/loader.model";
import { useServerDataCtx } from "@/providers/server-data-provider/server-data-provider";
import { useLoaderStore } from "@/store/loader-store/use-loader-store";

import { validateDateRange } from "../../../travel-planner/libs/hotel-travel-planner-validations";
import { useHotelWidgetTravelPlanner } from "../../hooks/use-hotel-widget-travel-planner";
import type { IHotelWidgetTravelPlannerForm } from "../types/hotel-widget-travel-planner.types";

const HotelWidgetTravelPlanner = () => {
  const { t } = useLocale();
  const { isOtel } = useApp();

  const { navigateToRoomList, data } = useHotelWidgetTravelPlanner();
  const { clientConfig } = useServerDataCtx();
  const { openLoader } = useLoaderStore();
  const selectedBaseHotel =
    clientConfig?.domainProducts.find((item) =>
      data?.payload?.destinationId
        ? item.id === Number(data?.payload?.destinationId)
        : item.name === clientConfig?.clientScope,
    ) || clientConfig?.domainProducts[0];

  const defaultValues = useMemo(() => {
    const defaultValue: Partial<IHotelWidgetTravelPlannerForm> = {
      "date-range":
        data?.payload?.dates &&
        data.payload.dates.checkIn &&
        data.payload.dates.checkOut
          ? {
              from: DateHelper.dateFromISOString(data.payload.dates?.checkIn),
              to: DateHelper.dateFromISOString(data.payload.dates?.checkOut),
            }
          : undefined,
      hotel: {
        id: selectedBaseHotel?.id ?? 0,
        value: selectedBaseHotel?.name ?? "",
        destinationBreadCrumb: selectedBaseHotel?.name || "",
        type: 0,
        typeName: "Otel",
      },
      room: data?.payload?.rooms
        ? data.payload.rooms?.map((item) => ({
            guest: { count: item.guest?.count ?? 0 },
            child: {
              count: item.child?.count ?? 0,
              ages: item.child?.ages ?? [],
            },
          }))
        : [isOtel ? DefaultPBTpPassenger : DefaultRBTpPassenger],
    };

    return defaultValue;
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  const tpHotelForm = useForm<IHotelWidgetTravelPlannerForm>({
    defaultValues,
  });
  const hasMoreHotel = (clientConfig?.domainProducts?.length || 0) > 1;

  const handleSubmit = (values: IHotelWidgetTravelPlannerForm) => {
    openLoader({ open: true, key: ILoaderKeys.HotelWidgetTPLoaderKey });
    setTimeout(() => {
      let destinationId = values.hotel?.id;
      if (!values.hotel?.id && selectedBaseHotel?.id) {
        tpHotelForm.setValue("hotel", {
          id: selectedBaseHotel?.id,
          value: selectedBaseHotel?.name,
          destinationBreadCrumb: selectedBaseHotel?.name,
          type: 0,
          typeName: "Otel",
        });
        destinationId = selectedBaseHotel?.id;
      }
      navigateToRoomList({
        destinationId,
        dates: {
          checkIn: DateHelper.dateToISOString(
            dayjs(values["date-range"].from!).toDate(),
          ),
          checkOut: DateHelper.dateToISOString(
            dayjs(values["date-range"].to!).toDate(),
          ),
        },
        rooms: values.room.map((item) => ({
          child: item.child,
          employee: item.employee,
          guest: item.guest,
        })),
      });
    }, 750);
  };

  return (
    <div className="flex w-full items-center justify-center bg-gray-100 lg:h-20">
      <div className="w-full max-w-[1080px] p-5">
        {/* Travel Planner */}
        <div>
          <Form methods={tpHotelForm} onSubmit={handleSubmit}>
            <div
              className={cn("grid grid-cols-1 gap-3", {
                "lg:grid-cols-12": isOtel,
              })}
            >
              {hasMoreHotel && (
                <div className={cn(hasMoreHotel ? "col-span-3" : "col-span-6")}>
                  <FormItem name="hotel">
                    {({ field, setValue, fieldState }) => (
                      <div>
                        <Select
                          value={field.value?.value}
                          onValueChange={(itemName) => {
                            const item = clientConfig?.domainProducts.find(
                              (x) => x.name === itemName,
                            );
                            if (item)
                              setValue({
                                id: item.id,
                                value: item.name,
                                destinationBreadCrumb: item.name,
                                type: "hotel",
                                typeName: "Otel",
                              });
                          }}
                          defaultValue={field.value}
                        >
                          <SelectTrigger
                            className={cn(
                              "h-[52px] rounded-primary border border-gray-200 bg-white py-6 shadow-xs",
                              {
                                "border-error-500": fieldState.error,
                              },
                            )}
                          >
                            <div className="flex items-center justify-start gap-1 text-gray-500">
                              <LIcon name="Home" size={20} className="" />
                              <SelectValue
                                defaultValue={field.value}
                                placeholder={t("Choose a facility")}
                              />
                            </div>
                          </SelectTrigger>
                          <SelectContent className="scroll-bar max-h-[200px] text-sm text-gray-500">
                            {clientConfig?.domainProducts.map((item) => (
                              <SelectItem
                                key={`${item.name}-${item.id}`}
                                className="cursor-pointer px-3.5"
                                value={item.name.toString()}
                              >
                                {item.name}
                              </SelectItem>
                            ))}
                          </SelectContent>
                        </Select>
                        {fieldState.error && (
                          <div className="mt-1">
                            <span className="text-sm text-error-500">
                              {fieldState.error?.message}
                            </span>
                          </div>
                        )}
                      </div>
                    )}
                  </FormItem>
                </div>
              )}
              <div className={cn(hasMoreHotel ? "col-span-3" : "col-span-6")}>
                <TpDateRangePickerWrappedFormItem
                  name="date-range"
                  showNightInfo
                  untilDay={29}
                  rules={{
                    validate: () =>
                      validateDateRange(
                        tpHotelForm.watch("date-range")?.from,
                        tpHotelForm.watch("date-range")?.to,
                      ),
                  }}
                />
              </div>
              <div className="col-span-3">
                <TpRoomSelectWrappedFormItem
                  name="room"
                  checkInDate={tpHotelForm.watch("date-range")?.from}
                />
              </div>
              <div className="col-span-3">
                <Button
                  size="lg"
                  variant="primary"
                  className="size-full min-h-[54px]"
                >
                  {`${t("Search Room")}`}
                </Button>
              </div>
            </div>
          </Form>
        </div>
      </div>
    </div>
  );
};

export default HotelWidgetTravelPlanner;
