/* eslint-disable react-hooks/exhaustive-deps */
import dayjs from "dayjs";
import { useMemo } from "react";
import { useForm } from "react-hook-form";

import Form from "@/components/form/form";
import OverlayWrapper from "@/components/shared/overlay/overlay-wrapper";
import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import {
  Collapsible,
  CollapsibleContent,
  CollapsibleTrigger,
} from "@/components/ui/collapsible/collapsible";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import {
  DefaultPBTpPassenger,
  DefaultRBTpPassenger,
} from "@/data/default/tp-passenger";
import { usePermGuard } from "@/features/permission/components/perm-guard";
import { TpBranchOfficeDropdownWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-branch-office-dropdown-form";
import { CustomerSelectWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-customer-select-form";
import { TpDateRangePickerWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-date-range-picker-form";
import { TpHotelSearchWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-hotel-search";
import { TpReasonDropdownWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-reason-dropdown-form";
import { TpRoomSelectWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-room-select-form";
import type { ITravelPlannerForm } from "@/features/travel-planner/components/travel-planner/travel-planner-form";
import { useApp } from "@/hooks/useApp";
import { useBreakpoints } from "@/hooks/useBreakpoints";
import { useLocale } from "@/hooks/useLocale";
import { DateHelper } from "@/libs/helpers/date-time-helper";
import { useLocalStateCache } from "@/libs/helpers/local-state-cache-helper";
import { localeStorageHelper } from "@/libs/helpers/locale-storage-helper";
import { cn } from "@/libs/utils";
import { ILoaderKeys } from "@/models/loader.model";
import { useLoaderStore } from "@/store/loader-store/use-loader-store";
import { requiredRule } from "@/validations/form-rules/rules/required-rule";
import { requestAddGuestPassengersAllowPerm } from "@/validations/permissions/request-add-guest-passengers";

import { useHotelTravelPlanner } from "../hooks/use-hotel-travel-planner";
import {
  validateDateRange,
  validatePassengersAndCustumer,
} from "../libs/hotel-travel-planner-validations";

interface HotelTravelPlannerViewProps {
  hideShadow?: boolean;
  isCollapsible?: boolean;
}

const HotelTravelPlannerView = (props: HotelTravelPlannerViewProps) => {
  const { isB2C, isB2E } = useApp();
  const { data, navigateToHotelList } = useHotelTravelPlanner();
  const { t } = useLocale();
  const { isDesktop } = useBreakpoints();
  const { setSelectedEmployees } = useLocalStateCache();
  const { openLoader } = useLoaderStore();
  const defaultValues = useMemo(() => {
    const defaultValue: Partial<ITravelPlannerForm> = {
      reason: data?.payload.reason,
      branchOffice: data?.payload.branchOffice,
      "date-range":
        data?.payload?.dates &&
        data.payload.dates.checkIn &&
        data.payload.dates.checkOut
          ? {
              from: DateHelper.dateFromISOString(data.payload.dates?.checkIn),
              to: DateHelper.dateFromISOString(data.payload.dates?.checkOut),
            }
          : undefined,
      customer: data?.payload?.employees,
      hotel: data?.payload?.destination
        ? {
            id: data.payload.destination?.id!,
            value: data.payload.destination?.value!,
            destinationBreadCrumb:
              data.payload.destination?.destinationBreadCrumb!,
            type: data.payload.destination?.type!,
            typeName: data.payload.destination?.typeName!,
          }
        : undefined,
      room: data?.payload?.rooms
        ? data.payload.rooms?.map((item) => ({
            employee: {
              count: item.employee?.count ?? 0,
            },
            guest: { count: item.guest?.count ?? 0 },
            child: {
              count: item.child?.count ?? 0,
              ages: item.child?.ages ?? [],
            },
          }))
        : [isB2C ? DefaultPBTpPassenger : DefaultRBTpPassenger],
    };

    return defaultValue;
  }, []);

  const { isCheck } = usePermGuard({
    permission: requestAddGuestPassengersAllowPerm,
  });

  const tpHotelForm = useForm<ITravelPlannerForm>({
    defaultValues,
  });

  const renderHotelTpForm = () => (
    <Form
      methods={tpHotelForm}
      onSubmit={(values: ITravelPlannerForm) => {
        localeStorageHelper.set("travel-planner", {
          ...values,
        });
        if (isB2E) setSelectedEmployees(values.room, values.customer);
        if (values.hotel.type === 0) {
          openLoader({
            key: ILoaderKeys.HotelDetailLoaderKey,
            open: true,
          });
        }
        navigateToHotelList(
          {
            employees: values.customer,
            rooms: values.room.map((item) => ({
              child: item.child,
              employee: item.employee,
              guest: item.guest,
            })),
            destination: values.hotel,
            dates: {
              checkIn: DateHelper.dateToISOString(values["date-range"].from!),
              checkOut: DateHelper.dateToISOString(values["date-range"].to!),
            },
            reason: values.reason,
            branchOffice: values.branchOffice,
          },
          values.hotel.type === 0 ? "_self" : "_blank",
        );
      }}
    >
      <div
        className={cn("grid grid-cols-1 gap-3", {
          "lg:grid-cols-3": isB2E,
          "lg:grid-cols-4": isB2C,
        })}
      >
        <OverlayWrapper>
          <TpHotelSearchWrappedFormItem
            name="hotel"
            rules={{ required: requiredRule }}
          />
        </OverlayWrapper>
        <OverlayWrapper>
          <TpDateRangePickerWrappedFormItem
            name="date-range"
            rules={{
              validate: () =>
                validateDateRange(
                  tpHotelForm.watch("date-range")?.from,
                  tpHotelForm.watch("date-range")?.to,
                ),
            }}
          />
        </OverlayWrapper>
        <OverlayWrapper>
          <TpRoomSelectWrappedFormItem
            checkInDate={tpHotelForm.watch("date-range")?.from}
            name="room"
          />
        </OverlayWrapper>
        {isB2E && (
          <div className="col-span-1 flex flex-col gap-3 lg:col-span-2 lg:flex-row">
            <div className="flex-1 shrink-0">
              <CustomerSelectWrappedFormItem
                name="customer"
                rules={
                  !isCheck && {
                    validate: () =>
                      validatePassengersAndCustumer(
                        tpHotelForm.watch("room"),
                        tpHotelForm.watch("customer"),
                      ),
                  }
                }
              />
            </div>
            <TpBranchOfficeDropdownWrappedFormItem
              name="branchOffice"
              rules={{ required: requiredRule }}
            />
            <div className="flex-1 shrink-0">
              <TpReasonDropdownWrappedFormItem
                name="reason"
                rules={{ required: requiredRule }}
              />
            </div>
          </div>
        )}
        <div>
          <Button
            size="lg"
            variant="primary"
            className="size-full min-h-[54px]"
          >
            {`${t("Hotel")} ${t("Search")}`}
          </Button>
        </div>
      </div>
    </Form>
  );

  if (props.isCollapsible && !isDesktop) {
    return (
      <Card className={cn(!props.hideShadow && "shadow-2xl", "py-3")}>
        <Collapsible>
          <CollapsibleTrigger className="w-full">
            <div className="flex w-full items-center justify-between">
              <div className="flex w-full flex-col justify-start text-left">
                <div className="text-base font-medium">
                  {defaultValues.hotel?.value}
                </div>
                <div className="-mt-0.5 text-sm font-medium text-gray-500">
                  {`${dayjs(defaultValues["date-range"]?.from).format(
                    "DD MMM",
                  )} - ${dayjs(defaultValues["date-range"]?.to).format(
                    "DD MMM",
                  )}, ${t("{{roomCount}} room, {{travelCount}} travellers", {
                    roomCount: defaultValues.room?.length,
                    travelCount: defaultValues.room?.reduce(
                      (acc, item) =>
                        acc +
                        (item.employee?.count ?? 0) +
                        (item.guest?.count ?? 0) +
                        (item.child?.count ?? 0),
                      0,
                    ),
                  })}`}
                </div>
              </div>
              <div>
                <LIcon name="Search" className="text-gray-500" />
              </div>
            </div>
          </CollapsibleTrigger>
          <CollapsibleContent className="pt-4">
            {renderHotelTpForm()}
          </CollapsibleContent>
        </Collapsible>
      </Card>
    );
  }

  return (
    <Card
      className={cn({
        "shadow-2xl": !props.hideShadow,
        "z-[51]": isDesktop,
      })}
    >
      {renderHotelTpForm()}
    </Card>
  );
};

export default HotelTravelPlannerView;
