/* eslint-disable react-hooks/exhaustive-deps */
/* eslint-disable radix */
import dayjs from "dayjs";
import { useEffect, useState } from "react";

import { usePageInfo } from "@/hooks/use-page-info";
import useRoom from "@/hooks/use-room/useRoom";
import { DateHelper } from "@/libs/helpers/date-time-helper";
import { storageManager } from "@/libs/storage-manager";
import { useLocalizationStore } from "@/store/localization-store/use-localization-store";
import { useTravelPlannerStore } from "@/store/travel-planner-store/use-travel-planner-store";

import type { IHotelTravelPlannerPayload } from "../types/hotel-travel-planner.types";

type SearchPayload = Partial<
  Pick<
    IHotelTravelPlannerPayload,
    "rooms" | "employees" | "dates" | "destination" | "reason" | "branchOffice"
  >
>;

export const useHotelTravelPlanner = () => {
  const { setModuleData, travelPlannerState, setData } =
    useTravelPlannerStore();
  const { setQueryParams, info, isDone } = usePageInfo();
  const [state, setState] = useState({
    isReady: false,
  });
  const {
    tp: { roomsFromString, roomsToString },
  } = useRoom();
  const { localizationState } = useLocalizationStore();

  function setSearchData(payload: SearchPayload) {
    storageManager.set("core-travel-planner", {
      moduleName: "Hotel",
      payload,
    });
    setModuleData({
      moduleName: "Hotel",
      payload,
    });
  }

  function navigateToHotelList(
    payload: SearchPayload,
    target?: "_blank" | "_self" | "_parent" | "_top",
  ) {
    setSearchData(payload);
    const pushUrl = ["otel", "Otel"].includes(
      payload.destination?.typeName ?? "",
    )
      ? "/hotel-detail"
      : "/hotel-list";

    setQueryParams(
      {
        destination: payload.destination?.value,
        destinationId: payload.destination?.id.toString(),
        destinationType: payload.destination?.type.toString(),
        destinationTypeName: payload.destination?.typeName,
        checkinDate: DateHelper.isoStringToFullYearString(
          payload.dates!.checkIn!,
          "-",
        ),
        checkoutDate: DateHelper.isoStringToFullYearString(
          payload.dates!.checkOut!,
          "-",
        ),
        rooms: roomsToString(payload.rooms!),
        lng: localizationState.data.language,
        rgn: localizationState.data.region,
        cur: localizationState.data.currency,
      },
      pushUrl,
      pushUrl === "/hotel-detail" ? (target ?? "_blank") : "_self",
    );
  }

  function getLinkGenerator(payload: SearchPayload) {
    const tpData = travelPlannerState.data as {
      moduleName: "Hotel";
      payload: IHotelTravelPlannerPayload;
    };

    const pushUrl = ["otel", "Otel"].includes(
      payload.destination?.typeName ?? "",
    )
      ? "/hotel-detail"
      : "/hotel-list";

    const searchData = {
      destination: payload.destination?.value,
      destinationId: payload.destination?.id.toString(),
      destinationType: payload.destination?.type.toString(),
      destinationTypeName: payload.destination?.typeName,
      checkinDate: DateHelper.isoStringToFullYearString(
        tpData.payload.dates!.checkIn!,
        "-",
      ),
      checkoutDate: DateHelper.isoStringToFullYearString(
        tpData.payload.dates!.checkOut!,
        "-",
      ),
      rooms: roomsToString(tpData.payload.rooms!),
      lng: localizationState.data.language,
      rgn: localizationState.data.region,
      cur: localizationState.data.currency,
    };

    const params = new URLSearchParams();
    Object.entries(searchData).forEach(([key, value]) => {
      if (value) {
        params.set(key, value);
      }
    });

    const link = `${pushUrl}?${params.toString()}`;
    return link;
  }

  function getSearchPayloadFromQueryParams() {
    const { queryParams } = info!;
    const storagedTpData = storageManager.get("core-travel-planner");
    const payload: SearchPayload = {
      ...storagedTpData?.payload,
      destination: {
        id: parseInt(queryParams.destinationId ?? ""),
        value: queryParams.destination ?? "",
        type: parseInt(queryParams.destinationType ?? ""),
        typeName: queryParams.destinationTypeName ?? "",
        destinationBreadCrumb: queryParams.destinationBreadCrumb ?? "",
      },
      dates: {
        checkIn: DateHelper.dateToISOString(
          dayjs(queryParams.checkinDate).toDate(),
        ),
        checkOut: DateHelper.dateToISOString(
          dayjs(queryParams.checkoutDate).toDate(),
        ),
      },
      rooms: roomsFromString(queryParams.rooms ?? ""),
    };

    if (payload) {
      setSearchData(payload);
    }

    return payload;
  }

  useEffect(() => {
    if (isDone && travelPlannerState?.data?.moduleName !== "Hotel") {
      setData(undefined);
    }
    if (isDone && travelPlannerState.status === "done") {
      if (
        info?.path === "/hotel-list" ||
        info?.path === "/search" // This parameter added according to old structure
      ) {
        getSearchPayloadFromQueryParams();
      } else if (
        info?.path === "/hotel-detail" ||
        info?.path === "/detail" // This parameter added according to old structure
      ) {
        getSearchPayloadFromQueryParams();
      }
      setState(() => ({
        isReady: true,
      }));
    }
  }, [travelPlannerState.status, isDone]);

  return {
    setSearchData,
    navigateToHotelList,
    data: travelPlannerState.data as {
      moduleName: "Hotel";
      payload: IHotelTravelPlannerPayload;
    },
    isReady: state.isReady,
    getLinkGenerator,
  };
};
