/* eslint-disable react-hooks/exhaustive-deps */

"use client";

import dayjs from "dayjs";
import React, { useCallback, useEffect, useState } from "react";

import type { AppliedFilterData } from "@/components/widget/applied-filter/applied-filter";
import { useAppliedFilter } from "@/components/widget/applied-filter/applied-filter";
import type {
  EventPayload,
  OnSubmitPayload,
} from "@/components/widget/core-list/core-list";
import CoreList from "@/components/widget/core-list/core-list";
import type { FilterItemType } from "@/components/widget/core-list/core-list-filter/core-list-filter";
import { FilterItemTypeEnum } from "@/components/widget/core-list/core-list-filter/core-list-filter";
import type { CoreListSortValueType } from "@/components/widget/core-list/core-list-sort";
import RentACarCard from "@/components/widget/rent-a-car-card/rent-a-car-card";
import { useLocale } from "@/hooks/useLocale";
import { ILoaderKeys } from "@/models/loader.model";
import type { Car } from "@/services/eva-display/rent-a-car/rent-a-car-detail/rent-a-car-detail.types";
import type {
  IRentACarRequest,
  IRentACarRequestFilter,
  IRentACarSearchResponse,
  PassengerList,
  RentACarItem,
} from "@/services/eva-display/rent-a-car/rent-a-car-search/rent-a-car-search.types";
import { useRentACarSearch } from "@/services/eva-display/rent-a-car/rent-a-car-search/rent-a-car-search-service";
import type { SearchFilterListItem } from "@/services/eva-display/tour/tour-search/tour-search.types";
import { useLoaderStore } from "@/store/loader-store/use-loader-store";

import { useRentACarSummary } from "../../summary/hook/use-rent-a-car-summary";
import { useRentACarTravelPlanner } from "../../travel-planner/hooks/use-rent-a-car-travel-planner";

const initialRequestValue = {
  CarRentalSearchRequest: {
    SearchId: null, // {{SearchId}},
    PickupDateTime: "2024-02-15T12:00:00.000", // Alış tarihi
    ReturnDateTime: "2024-02-20T12:00:00.000", // Teslimat tarihi
    DriverAge: 30, // sürücü yaşı - seçili kullanıcının yaşı gönderilecek veya sabit
    SearchLeg: {
      PickupStationId: 56769, // Alış yeri
      PickupStationType: 6, // Destination type
      ReturnStationId: 56769, // Teslimat teri
      ReturnStationType: 6, // Destination type
    },
    Filter: {
      Page: 1,
      PageSize: 10,
    },
  },
};

const RentACarList = () => {
  const { t } = useLocale();
  const { mutateAsync, isLoading } = useRentACarSearch();
  const { isReady, data: searchData } = useRentACarTravelPlanner();
  const { setSearchAndDetailData, goSummary } = useRentACarSummary();
  const { removeLoader } = useLoaderStore();
  const {
    data: appliedFilters,
    setData: setAppliedFilters,
    updateFilterByKey,
    removeFilter,
    clearFilter,
  } = useAppliedFilter();
  const [sortingValue, setSortingValue] =
    useState<CoreListSortValueType>("Average");
  const [{ mergedData: data, lastResponse }, setState] = useState<{
    lastResponse?: IRentACarSearchResponse;
    responseData: Array<{
      key: string;
      data: Array<RentACarItem>;
    }>;
    mergedData: Array<RentACarItem>;
    isLoading: boolean;
    isReady: boolean;
    passengerList: PassengerList[];
  }>({
    responseData: [],
    mergedData: [],
    passengerList: [],
    isLoading: false,
    isReady: false,
  });
  const filterMetaData: Partial<
    Record<keyof IRentACarRequestFilter, SearchFilterListItem[]>
  > = {
    PassengerCounts: lastResponse?.filter?.passengerCountInformations || [],
    DriverAgeLimits: lastResponse?.filter?.ageLimitInformations || [],
    CarNames: lastResponse?.filter?.carNameInformations || [],
    CarTypes: lastResponse?.filter?.carTypeInformations || [],
  };

  const [requestData, setRequestData] =
    useState<IRentACarRequest>(initialRequestValue);

  const setAppliedFilter = (
    searchFilter: typeof initialRequestValue.CarRentalSearchRequest.Filter,
  ) => {
    const exceptedKeys = ["Page", "PageSize"];
    const appliedFilterNewData = [];
    for (const [key, val] of Object.entries<Array<string | number>>(
      searchFilter as unknown as Record<string, Array<string | number>>,
    )) {
      if (typeof val !== undefined && !exceptedKeys.includes(key)) {
        if (key === "AutomaticTransmission") {
          const newFilterValues: AppliedFilterData[] = [];
          if (val !== null) {
            newFilterValues.push({
              filterKey: "gearType",
              filterValue: val ? "automatic" : "manual",
              filterLabel: val ? "Otomatik" : "Manuel",
            });
            appliedFilterNewData.push(...newFilterValues);
          }
        } else if (val?.length > 0 && val !== null) {
          const newFilterValues: AppliedFilterData[] = [];
          newFilterValues.push(
            ...(val?.map((item: any) => {
              return {
                filterKey: key,
                filterValue: item,
                filterLabel:
                  (
                    filterMetaData?.[key as keyof IRentACarRequestFilter] || []
                  )?.find((filter: any) => filter.key === item)?.value || "",
              };
            }) || []),
          );
          appliedFilterNewData.push(...newFilterValues);
        }
      }
    }
    setAppliedFilters([...appliedFilterNewData]);
  };

  function applySort(eventPayload: EventPayload) {
    let orderBy = null as any;

    if (eventPayload.payload.value === "PriceAscending") orderBy = false;
    if (eventPayload.payload.value === "PriceDescending") orderBy = true;

    setState((prevState) => ({
      ...prevState,
      isLoading: true,
      mergedData: [],
      responseData: [],
    }));
    setRequestData((prevState) => ({
      CarRentalSearchRequest: {
        ...prevState.CarRentalSearchRequest,
        Filter: {
          ...prevState.CarRentalSearchRequest.Filter,
          IsPriceSortedByDesc: orderBy,
          Page: 1,
        },
        SearchId: null,
      },
    }));
  }

  function applyLoadMore() {
    setRequestData((prevState) => ({
      CarRentalSearchRequest: {
        ...prevState.CarRentalSearchRequest,
        SearchId: lastResponse?.searchId,
        Filter: {
          ...prevState.CarRentalSearchRequest.Filter,
          Page: (lastResponse?.filter?.page || 0) + 1,
        },
      },
    }));
  }

  const handleChangeListAction = (eventPayload: EventPayload) => {
    if (eventPayload.type === "loadMore") applyLoadMore();

    if (eventPayload.type === "sort") applySort(eventPayload);
  };

  const handleSubmitListFilter = (eventPayload: OnSubmitPayload["payload"]) => {
    setState((prevState) => ({
      ...prevState,
      isLoading: true,
      mergedData: [],
      responseRecords: [],
    }));
    setRequestData((prevState) => {
      let automaticTransmission = null;
      const hasAutomaticTransmission =
        eventPayload?.gearType?.includes("automatic");
      const hasManualTransmission = eventPayload?.gearType?.includes("manual");
      if (hasAutomaticTransmission) {
        automaticTransmission = true;
      }
      if (hasManualTransmission) {
        automaticTransmission = false;
      }
      const newFilter = {
        CarTypes: eventPayload?.CarTypes || [],
        CarNames: eventPayload?.CarNames || [],
        DriverAgeLimits: eventPayload?.DriverAgeLimits || [],
        PassengerCounts: eventPayload?.PassengerCounts || [],
        AutomaticTransmission: automaticTransmission,
        Page: 1,
        PageSize: 10,
      };

      return {
        ...prevState,
        CarRentalSearchRequest: {
          ...prevState.CarRentalSearchRequest,
          Filter: newFilter,
          SearchId: null,
        },
      };
    });
  };

  const handleClearFilter = () => {
    clearFilter();
    setState((prevState) => ({
      ...prevState,
      isLoading: true,
      mergedData: [],
      responseData: [],
    }));
    setRequestData((prevState) => ({
      CarRentalSearchRequest: {
        ...prevState.CarRentalSearchRequest,
        Filter: {
          Page: 1,
          PageSize: 10,
        },
        SearchId: null,
      },
    }));
  };

  const handleRemoveFilter = (item: AppliedFilterData) => {
    if (item.filterKey === "gearType") {
      setRequestData((prevState: any) => {
        const newFilter = {
          ...prevState.CarRentalSearchRequest.Filter,
          AutomaticTransmission: null,
          Page: 1,
        };
        return {
          CarRentalSearchRequest: {
            ...prevState.CarRentalSearchRequest,
            Filter: newFilter,
            SearchId: null,
          },
        };
      });
      removeFilter(item);
      return;
    }
    const appliedFiltersByKey = appliedFilters.filter(
      (filter) =>
        filter.filterKey === item.filterKey &&
        filter.filterValue !== item.filterValue,
    );
    updateFilterByKey(item.filterKey, appliedFiltersByKey);
    const newFilter = {
      ...requestData.CarRentalSearchRequest.Filter,
      [item.filterKey]: appliedFiltersByKey.map(
        (appliedItem) => appliedItem.filterValue,
      ),
      Page: 1,
    };
    setRequestData((prevState) => ({
      CarRentalSearchRequest: {
        ...prevState.CarRentalSearchRequest,
        Filter: newFilter,
        SearchId: null,
      },
    }));
  };
  function prepareFilterMeta(
    name: string,
    label: string,
    items: SearchFilterListItem[] | null | undefined,
    value: string[] | null | undefined,
    openCollapse = true,
  ) {
    return {
      name,
      type: FilterItemTypeEnum.CHECKBOX_LIST as FilterItemType,
      value,
      properties: {
        label,
        openCollapse,
        items:
          items?.map((item) => ({
            key: item.key,
            value: item.key,
            label: `${item.value} (${item.count})`,
          })) || [],
      },
    };
  }

  const getData = useCallback(() => {
    if (!searchData?.payload) return;

    const payload = { ...requestData };

    payload.CarRentalSearchRequest.SearchLeg.PickupStationId =
      searchData?.payload.origin.id;
    payload.CarRentalSearchRequest.SearchLeg.PickupStationType =
      searchData?.payload.origin.type;
    payload.CarRentalSearchRequest.SearchLeg.ReturnStationId =
      searchData?.payload.destination.id;
    payload.CarRentalSearchRequest.SearchLeg.ReturnStationType =
      searchData?.payload.destination.type;
    payload.CarRentalSearchRequest.PickupDateTime = dayjs(
      searchData.payload.departureDate.date,
    ).format("YYYY-MM-DDTHH:mm:ss");
    payload.CarRentalSearchRequest.ReturnDateTime = dayjs(
      searchData.payload.returnDate.date,
    ).format("YYYY-MM-DDTHH:mm:ss");

    mutateAsync(payload).then((e) => {
      setState((prevState) => {
        const newResponseData = prevState.responseData.filter(
          (item) =>
            item.key !== e?.filter?.page?.toString() &&
            Number(item.key) < e?.filter?.page,
        );
        const newMergeData = newResponseData
          ? newResponseData.map((item) => item.data).flat()
          : [];
        return {
          ...prevState,
          responseData: [
            ...newResponseData,
            {
              key: e?.filter?.page?.toString(),
              data: e.items,
            },
          ],
          mergedData: [...newMergeData, ...(e?.items || [])],
          isLoading: false,
          lastResponse: e,
          passengerList: e.passengerList,
        };
      });
    });
  }, [
    mutateAsync,
    requestData,
    searchData?.payload?.departureDate?.date,
    searchData?.payload?.destination?.id,
    searchData?.payload?.destination?.type,
    searchData?.payload?.origin?.id,
    searchData?.payload?.origin?.type,
    searchData?.payload?.returnDate?.date,
  ]);

  useEffect(() => {
    if (isReady) {
      getData();
      setAppliedFilter(
        requestData.CarRentalSearchRequest
          .Filter as typeof initialRequestValue.CarRentalSearchRequest.Filter,
      );
    }

    let newSortingValue: CoreListSortValueType = "Average";
    const isPriceSortedByDesc =
      requestData.CarRentalSearchRequest.Filter?.IsPriceSortedByDesc;

    if (isPriceSortedByDesc) {
      newSortingValue = "PriceDescending";
    } else if (isPriceSortedByDesc === null) {
      newSortingValue = "Average";
    } else {
      newSortingValue = "PriceAscending";
    }
    setSortingValue(newSortingValue);

    setTimeout(() => {
      removeLoader(ILoaderKeys.ListingLoaderKey);
    }, 250);
  }, [getData, isReady, requestData]);

  if (!searchData || !searchData?.payload) {
    return null;
  }

  return (
    <CoreList
      data={data || []}
      isLoading={isLoading}
      isEmpty={!data?.length}
      noDataText={t(
        "We couldn't find vehicles for the selected filters. Please try again with different filters",
      )}
      filterOptions={[
        {
          name: "card-container",
          type: FilterItemTypeEnum.CARD_CONTAINER,
          value: [],
          properties: {},
          filterOptions: [
            prepareFilterMeta(
              "CarTypes",
              "Araç Tipi",
              filterMetaData.CarTypes,
              requestData?.CarRentalSearchRequest.Filter?.CarTypes?.map(
                (item) => item?.toString(),
              ) || [],
            ),
            {
              name: "div1",
              type: FilterItemTypeEnum.DIVIDER,
              value: [],
              properties: {},
            },
            prepareFilterMeta(
              "gearType",
              "Vites Tipi",
              [
                {
                  key: "manual",
                  value: "Manuel",
                  count: lastResponse?.filter?.manuelTransmissionCount || 0,
                  selected: true,
                },
                {
                  key: "automatic",
                  value: "Otomatik",
                  count: lastResponse?.filter?.automaticTransmissionCount || 0,
                  selected: true,
                },
              ],
              [],
            ),
            {
              name: "div1",
              type: FilterItemTypeEnum.DIVIDER,
              value: [],
              properties: {},
            },
            prepareFilterMeta(
              "CarNames",
              "Araç Markası",
              lastResponse?.filter?.carNameInformations || [],
              requestData?.CarRentalSearchRequest.Filter?.CarNames?.map(
                (item) => item?.toString(),
              ) || [],
            ),
            {
              name: "div1",
              type: FilterItemTypeEnum.DIVIDER,
              value: [],
              properties: {},
            },
            prepareFilterMeta(
              "DriverAgeLimits",
              "Sürücü Yaş Limiti",
              lastResponse?.filter?.ageLimitInformations || [],
              requestData?.CarRentalSearchRequest.Filter?.DriverAgeLimits?.map(
                (item) => item?.toString(),
              ) || [],
            ),
            {
              name: "div1",
              type: FilterItemTypeEnum.DIVIDER,
              value: [],
              properties: {},
            },
            prepareFilterMeta(
              "PassengerCounts",
              "Yolcu Sayısı",
              lastResponse?.filter?.passengerCountInformations || [],
              requestData.CarRentalSearchRequest.Filter?.PassengerCounts?.map(
                (item) => item?.toString(),
              ) || [],
            ),
          ],
        },
      ]}
      filterOptionValues={{
        CarTypes: lastResponse?.filter?.carTypeInformations
          .filter((item) => item.selected)
          .map((item) => item.key),
        // eslint-disable-next-line no-nested-ternary
        gearType: requestData.CarRentalSearchRequest.Filter
          ?.AutomaticTransmission
          ? ["automatic"]
          : requestData.CarRentalSearchRequest.Filter?.AutomaticTransmission ===
              false
            ? ["manual"]
            : [],
        CarNames: lastResponse?.filter?.carNameInformations
          .filter((item) => item.selected)
          .map((item) => item.key),
        DriverAgeLimits: lastResponse?.filter?.ageLimitInformations
          .filter((item) => item.selected)
          .map((item) => item.key),
        PassengerCounts: lastResponse?.filter?.passengerCountInformations
          .filter((item) => item.selected)
          .map((item) => item.key),
      }}
      appliedFilters={appliedFilters}
      totalCount={lastResponse?.totalCount || 0}
      onSubmit={handleSubmitListFilter}
      onChange={handleChangeListAction}
      onRemoveFilter={handleRemoveFilter}
      onClearFilter={handleClearFilter}
      renderItem={(item: RentACarItem) => (
        <RentACarCard
          item={item as Car}
          id={item?.key}
          onClickPrimaryButton={(id) => {
            setSearchAndDetailData({
              searchId: lastResponse?.searchId || "",
              searchItemKey: id,
              detailKey: id,
              passengersKeys: lastResponse?.passengerList ?? [],
            });
            goSummary();
          }}
        />
      )}
      hasMore={lastResponse?.hasMorePage || false}
      sortValue={sortingValue}
      renderHeader={() => {
        return (
          <div>
            <h2 className="inline-block text-2xl font-semibold">
              {searchData.payload.origin?.value || ""}
            </h2>
            <div className="ml-1 mt-1 inline-block text-sm text-gray-500">
              {lastResponse?.totalCount || 0} sonuç bulundu.
            </div>
          </div>
        );
      }}
    />
  );
};

export default RentACarList;
