import { useCallback, useEffect, useState } from "react";
import { useEffectOnce } from "usehooks-ts";

import { Button } from "@/components/ui/button/button";
import type { IUserItem } from "@/components/widget/travel-planner/components/user-select/user-select";
import { CustomerSelectWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-customer-select-form";
import { getManagerHelper } from "@/libs/storage-manager/get-manager-helper";
import { useUserInformationsMutation } from "@/services/eva-display/user/user-information.service";
import { useUserDepartmentQuery } from "@/services/eva-display/user-department/user-department.service";
import type { IUserDepartment } from "@/services/eva-display/user-department/user-department.types";

import {
  rentACarDriverPrefix,
  useRentACarSummaryCtx,
} from "../rent-a-car-summary.provider";

const InfoField = ({ label, value }: { label: string; value: string }) => {
  return (
    <div className="text-sm">
      <span className="text-gray-500"> {label} </span>
      <p className="overflow-hidden text-ellipsis text-gray-800 "> {value} </p>
    </div>
  );
};

export const RentACarWorkerDriverForm = () => {
  const {
    form: { methods },
  } = useRentACarSummaryCtx();
  const { get } = getManagerHelper("core-travel-planner");
  const { data: userList, isSuccess } = useUserDepartmentQuery();
  const { mutateAsync: userInformationMutation } =
    useUserInformationsMutation();

  const [customerInfo, setCustomerInfo] = useState<IUserItem[]>(
    methods?.getValues("CustomerSelect") || [],
  );
  const { data } = useUserDepartmentQuery();

  const registerDriverInfo = useCallback(
    async (customer: IUserDepartment) => {
      await userInformationMutation({
        userIds: [Number(customer?.value)],
      }).then((e) => {
        if (e?.[0]?.userGuidId) {
          methods?.setValue(
            `${rentACarDriverPrefix}-identity`,
            e?.[0]?.passaportDetail?.citizenNo || "",
          );
          methods?.setValue(
            `${rentACarDriverPrefix}-name`,
            e?.[0]?.userDetail?.firstName || "",
          );
          methods?.setValue(
            `${rentACarDriverPrefix}-surname`,
            e?.[0]?.userDetail?.lastName || "",
          );
          methods?.setValue(
            `${rentACarDriverPrefix}-gender`,
            `${e?.[0]?.userDetail?.gender || "0"}` as "0" | "1",
          );
          methods?.setValue(
            `${rentACarDriverPrefix}-country`,
            e?.[0]?.passaportDetail?.nationality,
          );
        }
      });
      methods?.setValue(
        `${rentACarDriverPrefix}-id`,
        Number(customer?.value) || 0,
      );
      methods?.setValue(
        `${rentACarDriverPrefix}-email`,
        customer?.properties.email || "",
      );
      methods?.setValue(`${rentACarDriverPrefix}-phone`, {
        dialCode: customer?.properties.phoneNumber.slice(0, 2) || "90",
        areaCode: customer?.properties.phoneNumber.slice(3, 6) || "0",
        phoneNumber: customer?.properties.phoneNumber.slice(
          6,
          customer?.properties.phoneNumber.length,
        ),
      });
      methods?.setValue(
        `${rentACarDriverPrefix}-birthDate`,
        customer?.properties?.birthDate,
      );
    },
    [methods, userInformationMutation],
  );

  useEffectOnce(() => {
    if (isSuccess && !methods?.getValues("CustomerSelect")) {
      const tpData = get("core-travel-planner").payload;
      const selectedCustomer = userList?.filter(
        (item) => item?.value === tpData.employees?.[0]?.value,
      )[0];
      const value = [
        {
          name: selectedCustomer?.name || "",
          value: selectedCustomer?.value || "",
        },
      ];
      methods?.register("CustomerSelect", {
        value,
      });
      setCustomerInfo(value);
      if (selectedCustomer) registerDriverInfo(selectedCustomer);
    }
  });

  useEffect(() => {
    const subscription = methods?.watch((value, { name }) => {
      if (name === "CustomerSelect") {
        setCustomerInfo((value?.CustomerSelect || []) as IUserItem[]);
        if ((value?.CustomerSelect?.length || 0) > 0) {
          const customer = value.CustomerSelect?.[0];
          const selectedCustomer = data?.find(
            (item) => item.value === customer?.value,
          );
          if (selectedCustomer) registerDriverInfo(selectedCustomer);
        }
      }
    });
    return () => {
      subscription?.unsubscribe();
      methods?.unregister("CustomerSelect");
    };
  }, [data, methods, registerDriverInfo]);

  return (
    <div>
      {customerInfo?.length === 0 ? (
        <CustomerSelectWrappedFormItem
          name="CustomerSelect"
          placeholder="Kurum Çalışanı Seçimi"
          rules={{
            required: "Kurum çalışanı seçimi zorunludur",
          }}
        />
      ) : (
        <>
          <div className="flex items-center justify-between">
            <p className="text-sm font-semibold text-gray-800">
              Kurum Çalışanı Seçimi
            </p>
            <Button
              variant="primary-text"
              className="!p-0"
              onClick={() => {
                methods?.setValue("CustomerSelect", []);
              }}
            >
              Seçimi Değiştir
            </Button>
          </div>
          <div className="grid grid-cols-3 rounded-primary border border-gray-200 bg-gray-50 p-4">
            <InfoField
              label="Ad Soyad"
              value={
                data?.find((item) => item.value === customerInfo[0]?.value)
                  ?.name || ""
              }
            />
            <InfoField
              label="Telefon Numarası"
              value={
                data?.find((item) => item.value === customerInfo[0]?.value)
                  ?.properties.phoneNumber || ""
              }
            />
            <InfoField
              label="E-posta"
              value={
                data?.find((item) => item.value === customerInfo[0]?.value)
                  ?.properties.email || ""
              }
            />
          </div>
        </>
      )}
    </div>
  );
};
