/* eslint-disable react-hooks/exhaustive-deps */
import { useEffect, useMemo, useState } from "react";
import { useForm } from "react-hook-form";

import Form from "@/components/form/form";
import OverlayWrapper from "@/components/shared/overlay/overlay-wrapper";
import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import { Checkbox } from "@/components/ui/checkbox/checkbox";
import { TpBranchOfficeDropdownWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-branch-office-dropdown-form";
import { CustomerSelectWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-customer-select-form";
import TpDateTimePickerWrappedFormItem from "@/features/travel-planner/components/travel-planner/components/tp-date-time-picker-form";
import { TpPassengerSelectWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-passenger-select";
import { TpReasonDropdownWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-reason-dropdown-form";
import { TpRentACarSearchWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-rentcar-search";
import { useApp } from "@/hooks/useApp";
import { useBreakpoints } from "@/hooks/useBreakpoints";
import { useLocale } from "@/hooks/useLocale";
import { useLocalStateCache } from "@/libs/helpers/local-state-cache-helper";
import { cn } from "@/libs/utils";
import { requiredRule } from "@/validations/form-rules/rules/required-rule";

import { useRentACarTravelPlanner } from "../hooks/use-rent-a-car-travel-planner";
import type { IRentACarTravelPlannerPayload } from "../types/rent-a-car-travel-planner.types";

const RentACarTravelPlannerView = () => {
  const { t } = useLocale();
  const { navigateToTransferList, data, isReady } = useRentACarTravelPlanner();
  const { isB2C, isB2E } = useApp();
  const { isDesktop } = useBreakpoints();
  const [deliveryPoint, setDeliveryPoint] = useState(false);
  const { setSelectedEmployees } = useLocalStateCache();
  const defaultValues = useMemo(() => {
    const defaultValue: Partial<IRentACarTravelPlannerPayload> = {
      ...data.payload,
      passengers: data.payload?.passengers ?? {
        employee: {
          count: 1,
        },
        guest: {
          count: 0,
        },
      },
    };

    return defaultValue;
  }, []);

  const tpTransferForm = useForm({
    defaultValues,
  });

  // Set return date from departure date plus 1 day and also if there is a return date, check if it is greater than departure date
  const checkReturnMinDate = () => {
    const departureDate = tpTransferForm.watch("departureDate")?.date;
    if (departureDate) {
      const newDate = new Date(departureDate);
      newDate.setDate(newDate.getDate() + 1);
      return newDate;
    }
    return new Date();
  };

  const handleOnChangeDepartureDate = (value: {
    date?: Date;
    time?: string;
  }) => {
    if (value.date) {
      if (value.time === undefined) {
        tpTransferForm.setValue("departureDate", {
          date: value.date,
          time: "00:00",
        });
      }
      const newDate = new Date(value.date);
      const returnDate = tpTransferForm.getValues("returnDate")?.date;
      const returnTime = tpTransferForm.getValues("returnDate")?.time;
      newDate.setDate(newDate.getDate() + 1);

      // If return date is greater than new date, set return date to new date
      if ((returnDate || newDate) > newDate) return;

      tpTransferForm.setValue("returnDate", {
        date: newDate,
        time: returnTime ?? "00:00",
      });
    }
  };

  useEffect(() => {
    if (
      isReady &&
      data?.payload?.destination?.id &&
      data?.payload?.origin?.id
    ) {
      setDeliveryPoint(data.payload.destination.id !== data.payload.origin.id);
    }
  }, [isReady]);

  return (
    <Card
      className={cn({
        "z-[51]": isDesktop,
      })}
    >
      <Form
        methods={tpTransferForm}
        onSubmit={(values) => {
          if (isB2E && values?.employees)
            setSelectedEmployees(
              [
                {
                  employee: {
                    count: 1,
                  },
                },
              ],
              values.employees,
            );
          navigateToTransferList({
            origin: values.origin,
            departureDate: values.departureDate,
            destination: deliveryPoint ? values.destination : values.origin,
            employees: values.employees,
            passengers: values.passengers ?? {
              employee: {
                count: 1,
              },
              guest: {
                count: 0,
              },
              child: {
                count: 0,
                ages: [],
              },
            },
            reason: values.reason,
            branchOffice: values.branchOffice,
            returnDate: values.returnDate,
          });
        }}
      >
        <div className="grid gap-3">
          <div
            className={cn("grid grid-cols-1 gap-3", {
              "lg:grid-cols-4": isB2E,
              "lg:grid-cols-9": isB2C,
            })}
          >
            <OverlayWrapper
              className={cn({
                "lg:col-span-2": isB2C || (isB2E && !deliveryPoint),
                "lg:col-span-4": isB2C && !deliveryPoint,
              })}
            >
              <TpRentACarSearchWrappedFormItem
                placeHolder={
                  deliveryPoint
                    ? t("Pickup Location")
                    : t("Pickup and Dropoff Location")
                }
                name="origin"
                rules={{ required: requiredRule }}
              />
            </OverlayWrapper>
            {deliveryPoint && (
              <OverlayWrapper className={cn({ "lg:col-span-2": isB2C })}>
                <TpRentACarSearchWrappedFormItem
                  placeHolder={t("Dropoff Location")}
                  name="destination"
                  rules={{ required: requiredRule }}
                />
              </OverlayWrapper>
            )}

            <OverlayWrapper className={cn({ "lg:col-span-2": isB2C })}>
              <TpDateTimePickerWrappedFormItem
                name="departureDate"
                label={t("Pickup Date")}
                rules={{ required: requiredRule }}
                onChange={handleOnChangeDepartureDate}
              />
            </OverlayWrapper>
            <OverlayWrapper className={cn({ "lg:col-span-2": isB2C })}>
              <TpDateTimePickerWrappedFormItem
                name="returnDate"
                label={t("Dropoff Date")}
                rules={{ required: requiredRule }}
                fromDate={checkReturnMinDate()}
              />
            </OverlayWrapper>
            {isB2C && (
              <div className={cn({ "lg:col-span-1": isB2C })}>
                <Button
                  size="lg"
                  variant="primary"
                  className="size-full min-h-[54px]"
                >
                  {t("Search")}
                </Button>
              </div>
            )}
          </div>

          <div className="grid grid-cols-1 grid-rows-1 gap-3 lg:grid-cols-4">
            {isB2E && (
              <>
                <div className="col-span-1 flex flex-col gap-3 lg:col-span-3 lg:flex-row">
                  <div className="flex-1 shrink-0">
                    <TpPassengerSelectWrappedFormItem name="passengers" />
                  </div>
                  <div className="flex-1 shrink-0">
                    <CustomerSelectWrappedFormItem
                      name="employees"
                      placeholder="Sürücü Seçimi"
                      rules={{
                        required: requiredRule,
                        validate: (value: any) => {
                          if (value.length < 2) {
                            return true;
                          }
                          return "Sadece 1 sürücü seçebilirsiniz.";
                        },
                      }}
                    />
                  </div>
                  <TpBranchOfficeDropdownWrappedFormItem
                    name="branchOffice"
                    rules={{ required: requiredRule }}
                  />
                  <div className="flex-1 shrink-0">
                    <TpReasonDropdownWrappedFormItem
                      name="reason"
                      rules={{ required: requiredRule }}
                    />
                  </div>
                </div>
                <div>
                  <Button
                    size="lg"
                    variant="primary"
                    className="size-full min-h-[54px]"
                  >
                    {t("Search")}
                  </Button>
                </div>
              </>
            )}
          </div>
          <div>
            <div className="!text-gray-500">
              <Checkbox
                label={t("Different Dropoff Location")}
                variant="primary"
                id="rent-a-car-tp"
                checked={deliveryPoint}
                onCheckedChange={() => setDeliveryPoint(!deliveryPoint)}
              />
            </div>
          </div>
        </div>
      </Form>
    </Card>
  );
};

export default RentACarTravelPlannerView;
