/* eslint-disable react-hooks/exhaustive-deps */

"use client";

import dayjs from "dayjs";
import { useEffect, useState } from "react";

import { usePageInfo } from "@/hooks/use-page-info";
import { storageManager } from "@/libs/storage-manager";
import { useLocalizationStore } from "@/store/localization-store/use-localization-store";
import { useTravelPlannerStore } from "@/store/travel-planner-store/use-travel-planner-store";

import type { IRentACarTravelPlannerPayload } from "../types/rent-a-car-travel-planner.types";

type SearchPayload = Partial<IRentACarTravelPlannerPayload>;

export const useRentACarTravelPlanner = () => {
  const { setModuleData, travelPlannerState } = useTravelPlannerStore();
  const { setQueryParams, info, isDone } = usePageInfo();
  const [state, setState] = useState({
    isReady: false,
  });
  const { localizationState } = useLocalizationStore();

  function setSearchData(payload: SearchPayload) {
    storageManager.set("core-travel-planner", {
      moduleName: "RentACar",
      payload,
    });
    setModuleData({
      moduleName: "RentACar",
      payload,
    });
  }

  function navigateToTransferList(payload: SearchPayload) {
    setSearchData(payload);
    const pushUrl = "/rentcar/list";

    setQueryParams(
      {
        destination: payload.destination?.value,
        destinationId: payload.destination?.id.toString(),
        destinationType: payload.destination?.type.toString(),
        destinationTypeName: payload.destination?.typeName,
        origin: payload.origin?.value,
        originId: payload.origin?.id.toString(),
        originType: payload.origin?.type.toString(),
        originTypeName: payload.origin?.typeName,
        checkindate: `${dayjs(payload.departureDate?.date).format(
          "YYYY-MM-DD",
        )}${payload.departureDate?.time}`,
        checkoutdate: `${dayjs(payload.returnDate?.date).format(
          "YYYY-MM-DD",
        )}T${payload.returnDate?.time}`,
        passenger: `1-0`,
        lng: localizationState.data.language,
        rgn: localizationState.data.region,
        cur: localizationState.data.currency,
      },
      pushUrl,
      "_self",
    );
  }

  function getSearchPayloadFromQueryParams() {
    const { queryParams } = info!;
    const storagedTpData = storageManager.get("core-travel-planner");
    const payload: SearchPayload = {
      ...storagedTpData?.payload,
      destination: {
        id: Number(queryParams.destinationId),
        value: queryParams.destination ?? "",
        type: Number(queryParams.destinationType),
        typeName: queryParams.destinationTypeName ?? "",
      },
      origin: {
        id: Number(queryParams.originId),
        value: queryParams.origin ?? "",
        type: Number(queryParams.originType),
        typeName: queryParams.originTypeName ?? "",
      },
      departureDate: {
        date: dayjs(queryParams.checkindate).toDate(),
        time: dayjs(queryParams.checkindate).format("HH:mm"),
      },
      returnDate: {
        date: dayjs(queryParams.checkoutdate).toDate(),
        time: dayjs(queryParams.checkoutdate).format("HH:mm"),
      },
    };

    if (payload) {
      setSearchData(payload);
    }

    return payload;
  }

  useEffect(() => {
    if (isDone && travelPlannerState.status === "done" && !state.isReady) {
      if (info?.path === "/rentcar/list") {
        getSearchPayloadFromQueryParams();
      }
      if (travelPlannerState.data?.moduleName !== "RentACar") {
        setSearchData({});
      }

      setState({
        isReady: true,
      });
    }
  }, [travelPlannerState.status, isDone]);

  return {
    isReady: state.isReady,
    navigateToTransferList,
    getSearchPayloadFromQueryParams,
    setSearchData,
    data: travelPlannerState.data as {
      moduleName: "RentACar";
      payload: IRentACarTravelPlannerPayload;
    },
  };
};
