/* eslint-disable default-case */
/* eslint-disable react-hooks/exhaustive-deps */
import { useSearchParams } from "next/navigation";
import { useMemo, useState } from "react";
import { useForm } from "react-hook-form";

import Form from "@/components/form/form";
import OverlayWrapper from "@/components/shared/overlay/overlay-wrapper";
import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import {
  Tabs,
  TabsContent,
  TabsList,
  TabsTrigger,
} from "@/components/ui/tabs/tabs";
import type { RoomAndCustomerCounterValue } from "@/components/widget/travel-planner/components/customer-group-select/components/room-and-customer-counter";
import { DefaultPBTpPassenger } from "@/data/default/tp-passenger";
import TpPeriodDateWrappedFormItem from "@/features/travel-planner/components/travel-planner/components/tp-period-date-picker-form";
import type { ITpTourSearchValue } from "@/features/travel-planner/components/travel-planner/components/tp-tour-search";
import { TpTourSearchWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-tour-search";
import { useBreakpoints } from "@/hooks/useBreakpoints";
import { useLocale } from "@/hooks/useLocale";
import { localeStorageHelper } from "@/libs/helpers/locale-storage-helper";
import { cn } from "@/libs/utils";

import { useTourTravelPlanner } from "../hooks/use-tour-travel-planner";
import { calculateDate } from "../libs/tour-travel-planner-calculate-date";
import { selectPeriodType } from "../libs/tour-travel-planner-get-period-type";

interface ITpDateRangePickerValue {
  anySelected: boolean;
  date: {
    from: Date;
    to: Date;
  };
  period: {
    from: Date;
    to: Date;
  };
}

export interface ITravelPlannerForm {
  tour: ITpTourSearchValue;
  dateRange: ITpDateRangePickerValue;
  room: RoomAndCustomerCounterValue[];
  isDomestic: boolean;
}

const TourTravelPlannerView = () => {
  const searchParams = useSearchParams();
  const { isDesktop } = useBreakpoints();
  const { t } = useLocale();
  const { data, navigateToTourList } = useTourTravelPlanner();

  const [activeTab, setActiveTab] = useState(
    searchParams.get("isDomestic") === null ||
      searchParams.get("isDomestic") === "true"
      ? "1"
      : "2",
  );
  const [isDomestic, setIsDomestic] = useState(
    searchParams.get("isDomestic") === null ||
      searchParams.get("isDomestic") === "true",
  );

  const isDomesticBool = searchParams.get("isDomestic");
  const checkinDate = searchParams.get("checkinDate");
  const checkoutDate = searchParams.get("checkoutDate");
  const isPeriodType = searchParams.get("periodType");

  const defaultValues = useMemo(() => {
    let destinationTypeId = 3;
    switch (data?.payload.destination?.typeName) {
      case "Bölge":
        destinationTypeId = 1;
        break;
      case "Kategori":
        destinationTypeId = 3;
        break;
      case "Hareket Noktası":
      case "Çıkış Noktası":
        destinationTypeId = 4;
        break;
      case "Destinations":
        destinationTypeId = 6;
        break;
    }
    const defaultValue: Partial<ITravelPlannerForm> = {
      tour: data?.payload?.destination
        ? {
            id: data.payload.destination?.id!,
            value: data.payload.destination?.value!,
            destinationBreadCrumb:
              data.payload.destination?.destinationBreadCrumb!,
            type: destinationTypeId,
            typeName: data.payload.destination?.typeName!,
          }
        : undefined,
      room: data?.payload?.rooms
        ? data.payload.rooms?.map((item) => ({
            guest: { count: item.guest?.count ?? 2 },
            child: {
              count: item.child?.count ?? 0,
              ages: item.child?.ages ?? [],
            },
          }))
        : [DefaultPBTpPassenger],
      isDomestic,
    };
    return defaultValue;
  }, []);

  const tpTourForm = useForm<ITravelPlannerForm>({
    defaultValues,
  });

  return (
    <Card className={cn(isDesktop && "z-[51]")}>
      <Form
        methods={tpTourForm}
        onSubmit={(values: ITravelPlannerForm) => {
          localeStorageHelper.set("core-travel-planner", {
            ...values,
          });
          navigateToTourList({
            rooms: values.room!.map((item) => ({
              child: item.child,
              guest: item.guest,
            })),
            destination: values.tour,
            dates: {
              checkIn: calculateDate(values.dateRange, true, checkinDate!),
              checkOut: calculateDate(values.dateRange, false, checkoutDate!),
            },
            isDomestic,
            periodType:
              Number(selectPeriodType(values.dateRange, isPeriodType)) || 3,
          });
        }}
      >
        <Tabs
          defaultValue={
            isDomesticBool === null || isDomesticBool === "true" ? "1" : "2"
          }
        >
          <TabsList>
            <TabsTrigger
              className={`mr-2 rounded-primary border border-transparent ${
                activeTab === "1"
                  ? "!border !border-primary-500 !text-primary-500"
                  : ""
              }`}
              value="1"
              onClick={() => {
                setActiveTab("1");
                setIsDomestic(true);
              }}
            >
              {t("Domestic Tour")}
            </TabsTrigger>

            <TabsTrigger
              className={`rounded-primary border border-transparent ${
                activeTab === "2"
                  ? "!border !border-primary-500 !text-primary-500"
                  : ""
              }`}
              value="2"
              onClick={() => {
                setActiveTab("2");
                setIsDomestic(false);
              }}
            >
              {t("Abroad Tour")}
            </TabsTrigger>
          </TabsList>
          <TabsContent value="1">
            <div className="grid gap-3">
              <div className="grid grid-cols-1 gap-3 lg:grid-cols-3">
                <OverlayWrapper>
                  <TpTourSearchWrappedFormItem
                    name="tour"
                    title={t("Search Tour")}
                    label={t("Search Tour")}
                    isDomestic
                  />
                </OverlayWrapper>

                <OverlayWrapper>
                  <TpPeriodDateWrappedFormItem name="dateRange" />
                </OverlayWrapper>
                <div>
                  <Button
                    size="lg"
                    variant="primary"
                    className="size-full min-h-[54px]"
                  >
                    {t("Search")}
                  </Button>
                </div>
              </div>
            </div>
          </TabsContent>
          <TabsContent value="2">
            <div className="grid gap-3">
              <div className="grid grid-cols-1 gap-3 lg:grid-cols-3">
                <OverlayWrapper>
                  <TpTourSearchWrappedFormItem
                    name="tour"
                    title={t("Search Tour")}
                    label={t("Search Tour")}
                    isDomestic={false}
                  />
                </OverlayWrapper>

                <OverlayWrapper>
                  <TpPeriodDateWrappedFormItem name="dateRange" />
                </OverlayWrapper>
                <div>
                  <Button
                    size="lg"
                    variant="primary"
                    className="size-full min-h-[54px]"
                  >
                    {t("Search")}
                  </Button>
                </div>
              </div>
            </div>
          </TabsContent>
        </Tabs>
      </Form>
    </Card>
  );
};

export default TourTravelPlannerView;
