/* eslint-disable react-hooks/exhaustive-deps */
import dayjs from "dayjs";
import { useMemo, useState } from "react";
import { useForm } from "react-hook-form";
import { useMediaQuery } from "usehooks-ts";

import Form from "@/components/form/form";
import OverlayWrapper from "@/components/shared/overlay/overlay-wrapper";
import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import {
  Tabs,
  TabsContent,
  TabsList,
  TabsTrigger,
} from "@/components/ui/tabs/tabs";
import { TpBranchOfficeDropdownWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-branch-office-dropdown-form";
import { CustomerSelectWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-customer-select-form";
import TpDateTimePickerWrappedFormItem from "@/features/travel-planner/components/travel-planner/components/tp-date-time-picker-form";
import { TpPassengerSelectWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-passenger-select";
import { TpReasonDropdownWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-reason-dropdown-form";
import { TpTransferSearchWrappedFormItem } from "@/features/travel-planner/components/travel-planner/components/tp-transfer-search";
import { useApp } from "@/hooks/useApp";
import { useLocale } from "@/hooks/useLocale";
import { useLocalStateCache } from "@/libs/helpers/local-state-cache-helper";
import { cn } from "@/libs/utils";
import { requiredRule } from "@/validations/form-rules/rules/required-rule";

import { useTransferTravelPlanner } from "../hooks/use-transfer-travel-planner";
import { validatePassengersAndCustomer } from "../libs/transfer-travel-planner-validations";
import type { ITransferTravelPlannerPayload } from "../types/transfer-travel-planner.types";

const TransferTravelPlannerView = () => {
  const { isB2C, isB2E } = useApp();
  const { t } = useLocale();
  const { navigateToTransferList, data } = useTransferTravelPlanner();
  const { setSelectedEmployees } = useLocalStateCache();
  const [activeTab, setActiveTab] = useState(
    dayjs(data?.payload.returnDate?.date).isValid() ? "2" : "1",
  );
  const isDesktop = useMediaQuery("(min-width: 768px)");

  const defaultValues = useMemo(() => {
    if (!data?.payload) {
      return {
        passengers: {
          employee: {
            count: 0,
          },
          guest: {
            count: 1,
          },
          child: {
            count: 0,
            ages: [],
          },
        },
      };
    }
    const defaultValue: Partial<ITransferTravelPlannerPayload> = {
      ...data.payload,
      passengers:
        data.payload?.passengers ??
        (isB2E
          ? {
              employee: {
                count: 1,
              },
              guest: {
                count: 0,
              },
              child: {
                count: 0,
                ages: [],
              },
            }
          : {
              employee: {
                count: 0,
              },
              guest: {
                count: 1,
              },
              child: {
                count: 0,
                ages: [],
              },
            }),
    };

    return defaultValue;
  }, []);

  const tpTransferForm = useForm({
    defaultValues,
  });

  return (
    <Card
      className={cn({
        "z-[51]": isDesktop,
      })}
    >
      <Form
        methods={tpTransferForm}
        onSubmit={(values) => {
          if (isB2E)
            setSelectedEmployees(
              [
                {
                  employee: {
                    count: values?.employees?.length || 0,
                  },
                },
              ],
              values?.employees || [],
            );

          navigateToTransferList({
            departure: values.departure,
            departureDate: values.departureDate,
            destination: values.destination,
            employees: values.employees,
            passengers:
              values.passengers ??
              (isB2E
                ? {
                    employee: {
                      count: 1,
                    },
                    guest: {
                      count: 0,
                    },
                    child: {
                      count: 0,
                      ages: [],
                    },
                  }
                : {
                    employee: {
                      count: 0,
                    },
                    guest: {
                      count: 1,
                    },
                    child: {
                      count: 0,
                      ages: [],
                    },
                  }),
            reason: values.reason,
            branchOffice: values.branchOffice,
            returnDate: activeTab === "1" ? undefined : values.returnDate,
          });
        }}
      >
        <Tabs defaultValue={activeTab}>
          <TabsList>
            <TabsTrigger
              className="mr-2 rounded-primary border border-transparent data-[state=active]:!border data-[state=active]:!border-primary-500 data-[state=active]:text-primary-500"
              value="2"
              onClick={() => setActiveTab("2")}
            >
              {t("Round Trip")}
            </TabsTrigger>
            <TabsTrigger
              className="rounded-primary border border-transparent data-[state=active]:!border data-[state=active]:!border-primary-500 data-[state=active]:text-primary-500"
              value="1"
              onClick={() => setActiveTab("1")}
            >
              {t("One Way")}
            </TabsTrigger>
          </TabsList>
          <TabsContent value="1">
            <div className="grid gap-3">
              <div className="grid grid-cols-1 gap-3 lg:grid-cols-3">
                <OverlayWrapper className="col-start-1 col-end-2">
                  <TpTransferSearchWrappedFormItem
                    placeHolder={t("From")}
                    name="departure"
                    rules={{ required: requiredRule }}
                  />
                </OverlayWrapper>
                <OverlayWrapper
                  className={cn({
                    "col-span-2": isB2C && isDesktop,
                  })}
                >
                  <TpTransferSearchWrappedFormItem
                    placeHolder={t("To")}
                    name="destination"
                    rules={{ required: requiredRule }}
                  />
                </OverlayWrapper>
                <OverlayWrapper>
                  <TpDateTimePickerWrappedFormItem
                    rules={{
                      validate(value: { date: Date; time: string }) {
                        if (!value?.date || !value?.time) {
                          return requiredRule.message;
                        }

                        return true;
                      },
                    }}
                    name="departureDate"
                    label={t("Departure Date")}
                  />
                </OverlayWrapper>

                <OverlayWrapper>
                  <TpPassengerSelectWrappedFormItem name="passengers" />
                </OverlayWrapper>
                {isB2E && (
                  <div className="col-span-1 flex flex-col gap-3 lg:col-span-2 lg:flex-row">
                    <div className="flex-1 shrink-0">
                      <CustomerSelectWrappedFormItem
                        name="employees"
                        rules={{
                          validate: (value: any) =>
                            validatePassengersAndCustomer(
                              tpTransferForm.watch("passengers") as any,
                              value,
                            ),
                        }}
                      />
                    </div>
                    <TpBranchOfficeDropdownWrappedFormItem
                      name="branchOffice"
                      rules={{ required: requiredRule }}
                    />
                    <div className="flex-1 shrink-0">
                      <TpReasonDropdownWrappedFormItem
                        name="reason"
                        rules={{ required: requiredRule }}
                      />
                    </div>
                  </div>
                )}
                <div>
                  <Button
                    size="lg"
                    variant="primary"
                    className="size-full min-h-[54px]"
                  >
                    {t("Search")}
                  </Button>
                </div>
              </div>
            </div>
          </TabsContent>
          <TabsContent value="2">
            <div className="grid gap-3">
              <div className="grid grid-cols-1 gap-3 lg:grid-cols-4">
                <OverlayWrapper>
                  <TpTransferSearchWrappedFormItem
                    placeHolder={t("From")}
                    name="departure"
                    rules={{ required: requiredRule }}
                  />
                </OverlayWrapper>
                <OverlayWrapper>
                  <TpTransferSearchWrappedFormItem
                    placeHolder={t("To")}
                    name="destination"
                    rules={{ required: requiredRule }}
                  />
                </OverlayWrapper>
                <OverlayWrapper>
                  <TpDateTimePickerWrappedFormItem
                    rules={{
                      validate(value: { date: Date; time: string }) {
                        if (!value?.date || !value?.time) {
                          return requiredRule.message;
                        }

                        return true;
                      },
                    }}
                    name="departureDate"
                    label={t("Departure Date")}
                  />
                </OverlayWrapper>
                <OverlayWrapper>
                  <TpDateTimePickerWrappedFormItem
                    rules={{
                      validate(value: { date: Date; time: string }) {
                        const hasDateValue = dayjs(value?.date).isValid();
                        const hasTimeValue =
                          value?.time && value?.time !== "Invalid Date";
                        if (!hasDateValue || !hasTimeValue) {
                          return requiredRule.message;
                        }

                        return true;
                      },
                    }}
                    name="returnDate"
                    label={t("Return Date")}
                  />
                </OverlayWrapper>
              </div>

              <div className="grid grid-rows-1 gap-3 lg:grid-cols-4">
                <OverlayWrapper>
                  <TpPassengerSelectWrappedFormItem name="passengers" />
                </OverlayWrapper>
                {isB2E && (
                  <div className="col-span-1 flex flex-col gap-3 lg:col-span-2 lg:flex-row">
                    <div className="flex-1 shrink-0">
                      <CustomerSelectWrappedFormItem
                        name="employees"
                        rules={{
                          validate: (value: any) =>
                            validatePassengersAndCustomer(
                              tpTransferForm.watch("passengers") as any,
                              value,
                            ),
                        }}
                      />
                    </div>
                    <TpBranchOfficeDropdownWrappedFormItem
                      name="branchOffice"
                      rules={{ required: requiredRule }}
                    />
                    <div className="flex-1 shrink-0">
                      <TpReasonDropdownWrappedFormItem
                        name="reason"
                        rules={{ required: requiredRule }}
                      />
                    </div>
                  </div>
                )}
                <div>
                  <Button
                    size="lg"
                    variant="primary"
                    className="size-full min-h-[54px]"
                  >
                    {t("Search")}
                  </Button>
                </div>
              </div>
            </div>
          </TabsContent>
        </Tabs>
      </Form>
    </Card>
  );
};

export default TransferTravelPlannerView;
