/* eslint-disable react-hooks/exhaustive-deps */

"use client";

import dayjs from "dayjs";
import { useEffect, useState } from "react";

import { usePageInfo } from "@/hooks/use-page-info";
import { storageManager } from "@/libs/storage-manager";
import { useLocalizationStore } from "@/store/localization-store/use-localization-store";
import { useTravelPlannerStore } from "@/store/travel-planner-store/use-travel-planner-store";

import type { ITransferTravelPlannerPayload } from "../types/transfer-travel-planner.types";

type SearchPayload = Partial<ITransferTravelPlannerPayload>;

export const useTransferTravelPlanner = () => {
  const { setModuleData, setData, travelPlannerState } =
    useTravelPlannerStore();
  const { setQueryParams, info, isDone } = usePageInfo();
  const [state, setState] = useState({
    isReady: false,
  });
  const { localizationState } = useLocalizationStore();

  function setSearchData(payload: SearchPayload) {
    storageManager.set("core-travel-planner", {
      moduleName: "Transfer",
      payload,
    });
    setModuleData({
      moduleName: "Transfer",
      payload,
    });
  }

  function passengersToString(passengers: SearchPayload["passengers"]) {
    let passengerString = "";
    if (passengers?.employee) {
      passengerString += passengers.employee.count;
    }
    if (passengers?.guest) {
      passengerString += `${passengerString === "" ? "" : "_"}${
        passengers.guest.count
      }`;
    }

    return passengerString;
  }

  function navigateToTransferList(payload: SearchPayload) {
    setSearchData(payload);
    const pushUrl = "/transfer/list";
    setQueryParams(
      {
        destination: payload.destination?.value,
        destinationId: payload.destination?.id.toString(),
        destinationType: payload.destination?.type.toString(),
        destinationTypeName: payload.destination?.typeName,
        departure: payload.departure?.value,
        departureId: payload.departure?.id.toString(),
        departureType: payload.departure?.type.toString(),
        departureTypeName: payload.departure?.typeName,
        checkindate: `${dayjs(payload.departureDate?.date).format(
          "YYYY-MM-DD",
        )}${payload.departureDate?.time}`,
        checkoutdate: payload.returnDate
          ? `${dayjs(payload.returnDate?.date).format("YYYY-MM-DD")}T${
              payload.returnDate?.time
            }`
          : undefined,
        passenger: passengersToString(payload.passengers!),
        lng: localizationState.data.language,
        rgn: localizationState.data.region,
        cur: localizationState.data.currency,
      },
      pushUrl,
      "_self",
    );
  }

  function getSearchPayloadFromQueryParams() {
    const { queryParams } = info!;
    const storagedTpData = storageManager.get("core-travel-planner");
    const payload: SearchPayload = {
      ...storagedTpData?.payload,
      destination: {
        id: Number(queryParams.destinationId),
        value: queryParams.destination ?? "",
        type: Number(queryParams.destinationType),
        typeName: queryParams.destinationTypeName ?? "",
      },
      departure: {
        id: Number(queryParams.departureId),
        value: queryParams.departure ?? "",
        type: Number(queryParams.departureType),
        typeName: queryParams.departureTypeName ?? "",
      },
      departureDate: {
        date: dayjs(queryParams.checkindate).toDate(),
        time: dayjs(queryParams.checkindate).format("HH:mm"),
      },
      returnDate: queryParams.checkoutdate
        ? {
            date: dayjs(queryParams.checkoutdate).toDate(),
            time: dayjs(queryParams.checkoutdate).format("HH:mm"),
          }
        : undefined,
    };

    if (payload) {
      setSearchData(payload);
    }

    return payload;
  }

  useEffect(() => {
    if (isDone && travelPlannerState?.data?.moduleName !== "Transfer") {
      setData(undefined);
    }
    if (isDone && travelPlannerState.status === "done" && !state.isReady) {
      if (info?.path.includes("/transfer/list")) {
        getSearchPayloadFromQueryParams();
      }
      if (travelPlannerState.data?.moduleName !== "Transfer") {
        setSearchData({});
      }

      setState({
        isReady: true,
      });
    }
  }, [travelPlannerState.status, isDone]);

  return {
    isReady: state.isReady,
    navigateToTransferList,
    getSearchPayloadFromQueryParams,
    setSearchData,
    data: travelPlannerState.data as {
      moduleName: "Transfer";
      payload: ITransferTravelPlannerPayload;
    },
  };
};
