/* eslint-disable react/no-danger */

"use client";

import "./roomInfo.css";

import { useCallback, useState } from "react";
import { useEffectOnce } from "usehooks-ts";

import ContactModal from "@/components/shared/contact-modal/contact-modal";
import { Button } from "@/components/ui/button/button";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import { Modal } from "@/components/ui/modal/modal";
import { useModal } from "@/components/ui/modal/modal.hook";
import { Tooltip } from "@/components/ui/tooltip/tooltip";
import RoomAllOptionModal from "@/components/widget/room/room-all-option-modal/room-all-option-modal";
import RoomAvailableDatesModal from "@/components/widget/room/room-available-dates-modal/room-available-dates-modal";
import type { RoomFacilities } from "@/components/widget/room/room-card/room-card";
import {
  RoomCardAttributes,
  RoomCardImage,
  RoomCardProvider,
  RoomCardTitle,
  RoomsCampaignInformation,
  RoomsCancellationPolicy,
  RoomsHeading,
  RoomsProperty,
  RoomsPropertyFacilities,
  RoomsPropertyTotalPrice,
} from "@/components/widget/room/room-card/room-card";
import RoomPriceInformationModal from "@/components/widget/room/room-price-information-modal/room-price-information-modal";
import { useHotelWidgetTravelPlanner } from "@/features/modules/hotel/hotel-widget/hooks/use-hotel-widget-travel-planner";
import { useApp } from "@/hooks/useApp";
import { useBreakpoints } from "@/hooks/useBreakpoints";
import { useLocale } from "@/hooks/useLocale";
import { storageManager } from "@/libs/storage-manager";
import { cn } from "@/libs/utils";
import { ILoaderKeys } from "@/models/loader.model";
import type {
  Facility,
  Room,
  RoomCollectionResult,
} from "@/services/eva-display/hotel/hotel-detail/hotel-detail-services.types";
import { useLoaderStore } from "@/store/loader-store/use-loader-store";

import useRoomSelect from "../../hooks/use-room-select";

export interface RoomInfoProps {
  roomInfo: RoomCollectionResult;
  defaultFirstSelectRoom?: boolean;
}

interface RoomInfoModalProps {
  modalState: boolean;
  data: Room | null;
}

const RoomInfo = ({
  roomInfo,
  defaultFirstSelectRoom = false,
}: RoomInfoProps) => {
  const { navigateToRoomList } = useHotelWidgetTravelPlanner();
  const { openLoader } = useLoaderStore();
  const tpData = storageManager.get("core-travel-planner")?.payload;
  const hasTpDates = !!tpData?.dates?.checkIn && !!tpData?.dates?.checkOut;

  const { state, setRoomInfo } = useRoomSelect();
  const { isDesktop } = useBreakpoints();

  const { isB2C, isOtel } = useApp();
  const { t } = useLocale();

  const { closeModal, openModal, modalOpen } = useModal({ open: false });
  const [priceModal, setPriceModal] = useState<RoomInfoModalProps>({
    modalState: false,
    data: null,
  });
  const [showAllItems, setShowAllItems] = useState(2);

  function getRooms() {
    const temp: Room[] = [];
    const restrictionTemp: Room[] = [];
    roomInfo.rooms.forEach((item) => {
      const isOnDemand = isB2C || isOtel ? false : item.onDemand;
      if (!isOnDemand && item.roomIndex === state.activeTab) {
        if (item.restrictionList && item.restrictionList.length > 0) {
          restrictionTemp.push(item);
        } else {
          temp.push(item);
        }
      }
    });
    if (restrictionTemp.length > 0) {
      restrictionTemp.forEach((item) => {
        temp.push(item);
      });
    }

    return temp;
  }

  const isSelectedRoom = useCallback(
    (roomKey: string) => {
      if (
        state.selectedRooms[state.activeTab - 1]?.rooms &&
        state.selectedRooms[state.activeTab - 1]?.rooms[0]?.key === roomKey
      ) {
        return true;
      }
      return false;
    },
    [state.activeTab, state.selectedRooms],
  );

  const rooms = getRooms();

  const handleClickSelectRoom = useCallback(
    (roomProperty: Room) => {
      const filteredData = { ...roomInfo };
      if (filteredData.rooms) {
        filteredData.rooms = filteredData.rooms.filter(
          (item) =>
            item.roomIndex === state.activeTab && item.key === roomProperty.key,
        );
      }
      if (isSelectedRoom(roomProperty.key)) {
        setRoomInfo(filteredData, state.activeTab, false);
      } else {
        setRoomInfo(filteredData, state.activeTab, true);
      }
    },
    [isSelectedRoom, roomInfo, setRoomInfo, state.activeTab],
  );

  const handleClickApply = ({
    checkIn,
    checkOut,
  }: {
    checkIn: Date;
    checkOut?: Date;
  }) => {
    openLoader({ open: true, key: ILoaderKeys.HotelWidgetTPLoaderKey });
    navigateToRoomList({
      ...tpData,
      dates: {
        checkIn,
        checkOut,
      },
    });
  };

  useEffectOnce(() => {
    // @todo : this approach should be move to inside of context
    if (
      defaultFirstSelectRoom &&
      rooms?.[0] &&
      hasTpDates &&
      !rooms?.[0].onDemand
    ) {
      handleClickSelectRoom(rooms[0]);
    }
  });

  if (!tpData) return null;

  if (rooms.length === 0) return null;

  return (
    <div className="flex flex-col items-center justify-center gap-4">
      <div className="w-full md:min-w-[750px]">
        <RoomCardProvider>
          <div className="grid grid-cols-12 gap-2">
            {!isDesktop && (
              <div className="col-span-12 block md:hidden">
                <RoomCardTitle>{roomInfo.title}</RoomCardTitle>
              </div>
            )}
            <div className="col-span-3">
              {roomInfo?.images ? (
                roomInfo?.images?.length > 0 && (
                  <RoomCardImage
                    imgUrl={roomInfo.images[0]?.url}
                    onClick={openModal}
                  >
                    {roomInfo.images.length > 1 && (
                      <span>+{roomInfo.images.length - 1}</span>
                    )}
                  </RoomCardImage>
                )
              ) : (
                <RoomCardImage imgUrl={undefined} />
              )}
            </div>
            <div className="col-span-9">
              {isDesktop && <RoomCardTitle>{roomInfo.title}</RoomCardTitle>}
              <RoomCardAttributes
                viewType={roomInfo.viewType}
                attributes={[
                  ...((roomInfo?.attributes || []) as Facility[]),
                  ...(roomInfo?.facilities || []),
                ]}
              >
                {roomInfo.facilities && roomInfo.facilities.length > 4 && (
                  <Button
                    className="!m-0 !justify-start !p-0"
                    variant="primary-text"
                    onClick={openModal}
                  >
                    {t("See all features")}( {roomInfo.facilities.length - 4})
                    <LIcon name="ChevronRight" size="20" />
                  </Button>
                )}
              </RoomCardAttributes>
            </div>
          </div>
          <div
            className={cn(
              hasTpDates ? "flex flex-col gap-2" : "w-full flex justify-end",
            )}
          >
            {!hasTpDates && (
              <RoomAvailableDatesModal
                availableDates={roomInfo.hotelAvailabilityItems}
                onClickApply={handleClickApply}
              />
            )}
            {isDesktop && hasTpDates && (
              <RoomsHeading headingItem={roomInfo.rooms[0]?.roomHeadings} />
            )}
            {hasTpDates &&
              rooms.slice(0, showAllItems).map((roomProperty) => (
                <RoomsProperty
                  active={isSelectedRoom(roomProperty.key)}
                  key={roomProperty.roomIndex}
                >
                  <div
                    data-disabled={roomProperty.onDemand}
                    className="flex w-full flex-1 basis-10/12 justify-between data-[disabled='true']:opacity-50 md:pr-4"
                  >
                    <div className="mr-4 basis-5/12">
                      <RoomsPropertyFacilities
                        lowestPriceDescription={
                          roomProperty.lowestPriceDescription as string
                        }
                        discountInformations={
                          roomProperty?.discountInformations
                        }
                        mealType={
                          roomProperty?.mealType as unknown as RoomFacilities
                        }
                        freeChildStatus={
                          roomProperty.freeChildStatus as unknown as RoomFacilities
                        }
                        cancellationPolicyText={
                          isDesktop
                            ? null
                            : (roomProperty.cancellationPolicy as unknown as RoomFacilities)
                        }
                        campaignInformations={
                          isDesktop
                            ? null
                            : roomProperty?.campaignInformations?.[0]
                        }
                        addons={roomProperty?.addons}
                      />
                    </div>
                    {isDesktop && (
                      <div className="flex basis-4/12 flex-col gap-2">
                        {roomProperty?.cancellationPolicy?.description && (
                          <Tooltip
                            position="top"
                            label={
                              <div
                                className="flex max-w-[250px] flex-col"
                                dangerouslySetInnerHTML={{
                                  __html:
                                    roomProperty.cancellationPolicy.tooltip,
                                }}
                              />
                            }
                          >
                            <div className="cursor-pointer whitespace-nowrap">
                              <RoomsCancellationPolicy
                                cancellationPolicyText={
                                  roomProperty.cancellationPolicy.description
                                }
                              />
                            </div>
                          </Tooltip>
                        )}

                        {roomProperty?.campaignInformations?.length > 0 &&
                          roomProperty?.campaignInformations?.[0]?.tooltip && (
                            <Tooltip
                              position="top"
                              label={
                                <div
                                  className="flex max-w-[250px] flex-col"
                                  dangerouslySetInnerHTML={{
                                    __html:
                                      roomProperty?.campaignInformations?.[0]
                                        ?.tooltip,
                                  }}
                                />
                              }
                            >
                              <div className="cursor-pointer whitespace-nowrap">
                                <RoomsCampaignInformation
                                  description={
                                    roomProperty?.campaignInformations?.[0]
                                      ?.description
                                  }
                                />
                              </div>
                            </Tooltip>
                          )}
                      </div>
                    )}

                    <div className="basis-3/12 whitespace-nowrap">
                      <RoomsPropertyTotalPrice
                        discountRateDescription={
                          roomProperty.discountRateDescription
                        }
                        nightlyRateDescription={
                          roomProperty.nightlyRateDescription
                        }
                        basePrice={roomProperty.totalPriceWithDiscount}
                        totalPrice={roomProperty.totalPrice}
                      >
                        <Button
                          className="!m-0 !justify-start !p-0"
                          variant="primary-text"
                          onClick={() =>
                            setPriceModal({
                              modalState: true,
                              data: roomProperty,
                            })
                          }
                        >
                          {t("Price Information")}
                          <LIcon name="ChevronRight" size="20" />
                        </Button>
                      </RoomsPropertyTotalPrice>
                    </div>
                  </div>

                  <div className="flex w-full flex-col items-end gap-2 md:w-auto">
                    {roomProperty?.restrictionList && (
                      <span className="text-xs font-semibold text-error-500">
                        {roomProperty.restrictionList[0]?.tooltip}
                      </span>
                    )}
                    {(isB2C || isOtel) && roomProperty.onDemand ? (
                      <ContactModal>
                        <Button className="min-w-28" variant="outline-primary">
                          {t("Request")}
                        </Button>
                      </ContactModal>
                    ) : (
                      <Button
                        onClick={() => handleClickSelectRoom(roomProperty)}
                        disabled={roomProperty?.restrictionList?.length > 0}
                        className={cn(
                          "min-w-28",
                          isSelectedRoom(roomProperty.key) && "!bg-primary-50",
                        )}
                        variant="outline-primary"
                      >
                        {isSelectedRoom(roomProperty.key) ? (
                          <>
                            <LIcon name="Check" size="16" />
                            &nbsp;{t("Selected")}
                          </>
                        ) : (
                          t("Select")
                        )}
                      </Button>
                    )}
                  </div>
                </RoomsProperty>
              ))}
            {hasTpDates && rooms.length > 2 && (
              <div className="flex justify-center">
                <Button
                  variant="link"
                  onClick={() => {
                    if (showAllItems === 2) {
                      setShowAllItems(rooms.length);
                    } else {
                      setShowAllItems(2);
                    }
                  }}
                  rightIcon={{
                    name: showAllItems === 2 ? "ChevronDown" : "ChevronUp",
                    size: 20,
                  }}
                >
                  {showAllItems === 2
                    ? t("See other rates for this room ({{count}})", {
                        count: rooms.length - 2,
                      })
                    : t("Close")}
                </Button>
              </div>
            )}
          </div>
        </RoomCardProvider>
      </div>
      <Modal
        size={isOtel && isDesktop ? "xl" : "full"}
        open={modalOpen}
        onClose={closeModal}
        title={t("Room features")}
        className="z-[101] !overflow-hidden !px-0 md:!px-6"
        footer
      >
        <div
          className={cn(
            "mt-5 flex max-h-[calc(100vh-100px)] justify-center overflow-auto",
            {
              "max-h-[calc(100vh-100px)]": isDesktop,
              "max-h-[calc(100%-25px)]": !isDesktop,
            },
          )}
        >
          <RoomAllOptionModal
            roomTitle={roomInfo.title}
            images={roomInfo.images?.map((item) => item.url)}
            informationText={roomInfo.description}
            roomAttributes={roomInfo.attributes}
            roomFacilities={roomInfo.facilities}
          />
        </div>
      </Modal>
      <Modal
        size="xl"
        open={priceModal.modalState}
        onClose={() => setPriceModal({ modalState: false, data: null })}
        title={t("Price Information")}
        footer
      >
        {priceModal.data && (
          <RoomPriceInformationModal
            nightlyRateDescription={priceModal.data.nightlyRateDescription}
            totalPrice={priceModal.data.basePrice}
            nightlyRateResults={priceModal.data.nightlyRateResults}
          />
        )}
      </Modal>
    </div>
  );
};

export default RoomInfo;
