/* eslint-disable react-hooks/exhaustive-deps */

"use client";

import dayjs from "dayjs";
import startCase from "lodash-es/startCase";
import { useEffect, useState } from "react";

import {
  Accordion,
  AccordionContent,
  AccordionItem,
  AccordionTrigger,
} from "@/components/ui/accordion/accordion";
import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import { Divider } from "@/components/ui/divider/divider";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import Spinner from "@/components/ui/spinner";
import { Tooltip } from "@/components/ui/tooltip/tooltip";
import { getFormatedFlightTime } from "@/libs/helpers/date-time-helper";
import { DownloadVoucherPdfHelper } from "@/libs/helpers/download-voucher-pdf";
import {
  getAirportCode,
  getCityName,
  sortCombinationGroups,
  sortFlightSegments,
} from "@/libs/helpers/flight-helper";
import { PriceManager } from "@/libs/price-manager/price-manager";
import type { IFlightDetailResponse } from "@/services/eva-display/flight/flight-detail/flight-detail.service";
import type { IRetrieveDiscountCoupons } from "@/services/eva-display/shared/eva-display-shared-types";
import { useVoucherService } from "@/services/eva-display/voucher/voucher.services";

import BookingSuccessHeadingInfos from "./booking-success-heading-infos";
import BookingSuccessContainer from "./shareds/booking-success-container";
import { BookingSuccessDiscounts } from "./shareds/booking-success-discounts";
import BookingSuccessHeading from "./shareds/booking-success-heading";

interface FlightBookingSuccessProps {
  data: {
    resultString: string;
    bookingCode: string;
    saleCode: string;
    descriptionList: string[];
    needHelp: string;
    relevantInformations: string;
    voucherButtonString: string;
    flightInformation: null;
    priceSummary: {
      caption: string;
      discountString: string;
      totalPrice: string;
      basePrice: string;
      taxPrice: string;
      totalPriceString: string;
      totalPriceTitle: string;
    };
    bookingInformation: {
      productKey: string;
      caption: string;
      roundString: string;
      roundDepartureTime: string;
      roundArrivelTime: string;
      tripString: string;
      tripDepartureTime: string;
      tripArrivelTime: string;
      accommodationCountString: string;
      accommodationCount: string;
      isOneWayCombinable: boolean;
      cabinType: string;
      note: string;
    };
    optionDate: string;
    passengerDescription: string;
    passengerList: string[];
    paymentType: string;
    isRecurringPayment: boolean;
    discountCoupons?: IRetrieveDiscountCoupons | null;
  };
  cartId: string;
}

function FlightBookingSuccess({ data, cartId }: FlightBookingSuccessProps) {
  const [detailData, setDetailData] = useState<IFlightDetailResponse | null>(
    null,
  );
  const voucherMutationServices = useVoucherService({
    onSuccess: (responseData) => {
      DownloadVoucherPdfHelper(responseData, data?.bookingCode);
    },
  });

  useEffect(() => {
    const detailDataStr = sessionStorage.getItem(cartId);
    if (detailDataStr) {
      setDetailData(JSON.parse(detailDataStr));
    }
  }, []);

  if (!detailData) {
    return (
      <div>
        <Spinner />
      </div>
    );
  }

  const sortingData = sortCombinationGroups(
    detailData.pricingCombinations.length > 1
      ? detailData.pricingCombinations.flatMap((item) => item.combinationGroups)
      : detailData.pricingCombinations[0]!.combinationGroups,
  );

  return (
    <BookingSuccessContainer>
      <Card>
        <BookingSuccessHeading heading={data.resultString} />
        <BookingSuccessHeadingInfos
          className="mt-4"
          infoItems={data.descriptionList.map((desc) => ({
            text: desc,
          }))}
        />
        <Card className="mt-4 flex flex-col gap-4 border border-gray-200 bg-gray-50 py-2 lg:flex-row">
          <div className="flex items-center gap-2">
            <span>
              <LIcon name="AlertCircle" className="size-5 text-gray-500" />
            </span>
            <span className="text-xs text-gray-500">
              Bilet ve uçuş detaylarını indirebilirsiniz
            </span>
          </div>
        </Card>
        <div className="mt-4 flex w-full justify-center lg:justify-end">
          <Button
            className="text-nowrap text-sm"
            variant="primary"
            loading={voucherMutationServices.isLoading}
            leftIcon={{ name: "Download", className: "w-4 h-4" }}
            size="sm"
            onClick={() => voucherMutationServices.mutate(data.bookingCode)}
          >
            E-bileti indir
          </Button>
        </div>
      </Card>

      <Card>
        <h3 className="mb-4 text-xl font-semibold">Uçuş Bilgileri</h3>
        <Divider />
        <div className="space-y-4">
          <div className="grid grid-cols-1 gap-5 text-sm lg:grid-cols-2">
            <div className="flex flex-col gap-1">
              <span>Uçuş Tipi</span>
              <div className="flex items-center">
                <LIcon name="Plane" size="16" />
                <strong>
                  {data.bookingInformation.isOneWayCombinable
                    ? "Gidiş Dönüş"
                    : "Tek Yön"}
                  , {data.bookingInformation.cabinType}
                </strong>
              </div>
            </div>
            <div className="flex flex-col gap-1">
              <span>Kişi Sayısı</span>
              <div className="flex items-center">
                <LIcon name="Users" size="16" />
                <strong>{data.passengerDescription}</strong>
              </div>
            </div>
            <div className="flex flex-col gap-1">
              <span>Kişi Bilgileri</span>
              <strong>{data.passengerList.map(startCase).join(", ")}</strong>
            </div>
            {data.bookingInformation.note && (
              <div className="flex flex-col gap-1">
                <span>Özel İstek</span>
                <div className="flex items-center">
                  <strong>{data.bookingInformation.note}</strong>
                  <Tooltip
                    position="bottom"
                    label="Tüm özel istekleriniz, tesisin uygunluğuna göre işleme alınacaktır."
                    isAlwaysVisible
                  >
                    <LIcon name="Info" size="16" className="ml-[5px]" />
                  </Tooltip>
                </div>
              </div>
            )}
          </div>

          <Divider />

          <div className="flex flex-col gap-4">
            {sortingData.map((item) => {
              const flightSegments = sortFlightSegments(item.flightSegments);

              const flightSegment = flightSegments[0]!;
              const arrivalSegment = flightSegments[flightSegments.length - 1]!;
              const directs = flightSegments.map(
                (_flightSegmentItem) => _flightSegmentItem.marketingAirline,
              );

              const itemData = {
                key: item.key,
                destination: getCityName(arrivalSegment!.destination),
                origin: getCityName(flightSegment!.origin),
                date: dayjs(flightSegment!.departureTime).format(
                  "D MMMM YYYY ddd",
                ),
                logoSrc: `https://concorecdn.jollytur.com/concore/media/site/assets/img/theme/${
                  flightSegment!.operatingAirlineCode
                }-42x42.gif?v1040`,
                brandName: flightSegment!.operatingAirline,
                flightCode: flightSegment!.flightNumber,
                totalFlightHours: getFormatedFlightTime(item.travelTime),
                flightStartTime: dayjs(flightSegment!.departureTime).format(
                  "HH:mm",
                ),
                endStartTime: dayjs(arrivalSegment!.arrivalTime).format(
                  "HH:mm",
                ),
                transferInformation: `${
                  directs.length === 1
                    ? "Aktarmasız"
                    : `${directs.length - 1} Aktarmalı`
                }`,
                startIATACode: getAirportCode(flightSegment!.origin)!,
                endIATACode: getAirportCode(arrivalSegment!.destination)!,
                direction: item.groupId === 0 ? "Gidiş" : "Dönüş",
                cabinInfo: item.cabinClassTitle,
              };

              return (
                <div
                  key={itemData.key}
                  className="space-y-1 rounded-primary border p-4 text-sm"
                >
                  <div className="flex justify-between">
                    <div className="font-semibold">{itemData.direction}</div>
                    <div>{itemData.date}</div>
                  </div>

                  <div className="flex flex-col items-center  justify-between gap-4  pb-4 lg:flex-row lg:gap-0">
                    <div className="min-w-[150px]">
                      <div className="font-semibold">{itemData.brandName}</div>
                      <div className="text-xs">{itemData.flightCode}</div>
                    </div>
                    <div className="space-y-1 text-center">
                      <div className="text-xs text-gray-500">
                        {itemData.totalFlightHours}
                      </div>
                      <div className="flex gap-2">
                        <div className="flex gap-1">
                          <span className="font-semibold">
                            {itemData.flightStartTime}
                          </span>
                          <span className="text-gray-500">
                            {itemData.startIATACode}
                          </span>
                        </div>
                        <div className="flex items-center gap-1 text-gray-500">
                          <span>----</span>
                          <LIcon className="rotate-45" size="20" name="Plane" />
                          <div className="flex items-center gap-0">
                            ----{">"}
                          </div>
                        </div>
                        <div className="flex gap-1">
                          <span className="font-semibold">
                            {itemData.endStartTime}
                          </span>
                          <span className="text-gray-500">
                            {itemData.endIATACode}
                          </span>
                        </div>
                      </div>
                      <div className="font-semibold">
                        {itemData.transferInformation}
                      </div>
                      <div className="text-xs text-gray-500">
                        {itemData.cabinInfo}
                      </div>
                    </div>
                    <div className="min-w-[150px]" />
                  </div>
                </div>
              );
            })}
          </div>
        </div>
      </Card>
      <Card>
        <h3 className="mb-4 text-xl font-semibold">Fiyat Özeti</h3>
        <Divider />
        <div className="space-y-4">
          <div className="flex justify-between">
            <span>Fiyat</span>
            <span>
              {PriceManager({
                currency: data.priceSummary.basePrice.slice(0, 3) || "TRY",
                value: +data.priceSummary.basePrice.slice(3),
              })}
            </span>
          </div>
          <div className="flex justify-between">
            <span>Vergi Bedeli</span>
            <span>
              {PriceManager({
                currency: data.priceSummary.taxPrice.slice(0, 3) || "TRY",
                value: +data.priceSummary.taxPrice.slice(3),
              })}
            </span>
          </div>
          <BookingSuccessDiscounts
            discountsCoupon={data.discountCoupons?.discountCoupons}
            title="İndirimler"
          />

          <div className="flex items-center justify-between rounded-primary border p-4">
            <div>Toplam Tutar</div>
            <div className="text-lg font-bold">
              {PriceManager({
                currency: data.priceSummary.totalPrice.slice(0, 3) || "TRY",
                value: +data.priceSummary.totalPrice.slice(3),
              })}
            </div>
          </div>
        </div>
      </Card>

      <Card>
        {data.relevantInformations && (
          <Accordion
            type="single"
            collapsible
            defaultValue="relevantInformations"
            className="mb-[16px]"
          >
            <AccordionItem
              value="relevantInformations"
              className="rounded-primary border !bg-[#F9FAFB] p-3"
            >
              <AccordionTrigger className="py-0 hover:no-underline">
                <div className="mb-[8px] flex items-center text-gray-800">
                  <LIcon className="mr-[8px]" size="16" name="Info" /> Önemli
                  Bilgiler
                </div>
              </AccordionTrigger>
              <AccordionContent>
                <div
                  className="flex-1 [&>ul>li]:mb-1 [&>ul>li]:list-disc [&>ul]:ml-4"
                  // eslint-disable-next-line react/no-danger
                  dangerouslySetInnerHTML={{
                    __html: data.relevantInformations,
                  }}
                />
              </AccordionContent>
            </AccordionItem>
          </Accordion>
        )}

        {data.needHelp && (
          <div className="flex rounded-primary border border-gray-200 p-3">
            <LIcon className="mr-[8px]" size="20" name="Phone" />
            <div
              className="flex-1"
              // eslint-disable-next-line react/no-danger
              dangerouslySetInnerHTML={{ __html: data.needHelp }}
            />
          </div>
        )}
      </Card>
    </BookingSuccessContainer>
  );
}

export default FlightBookingSuccess;
