"use client";

import type { UseFormReturn } from "react-hook-form";

import OverlayWrapper from "@/components/shared/overlay/overlay-wrapper";
import { Button } from "@/components/ui/button/button";
import { useApp } from "@/hooks/useApp";
import { useLocale } from "@/hooks/useLocale";
import { cn } from "@/libs/utils";
import { requiredRule } from "@/validations/form-rules/rules/required-rule";

import type { ITravelPlannerForm } from "../travel-planner-flight-form";
import { TpBranchOfficeDropdownWrappedFormItem } from "./tp-branch-office-dropdown-form";
import { TpCabinSelectWrappedFormItem } from "./tp-cabin-select-form";
import { CustomerSelectWrappedFormItem } from "./tp-customer-select-form";
import { TpDatePickerWrappedFormItem } from "./tp-date-picker-form";
import { TpFlightSearchWrappedFormItem } from "./tp-flight-search";
import { TpReasonDropdownWrappedFormItem } from "./tp-reason-dropdown-form";

interface TpFlightOneWayFormProps {
  tpFlightForm: UseFormReturn<ITravelPlannerForm, any>;
}

const TpFlightOneWayForm = ({ tpFlightForm }: TpFlightOneWayFormProps) => {
  const { isB2C, isB2E } = useApp();
  const { t } = useLocale();
  const validateReturnDate = () => {
    const entryDate = tpFlightForm.watch("departure-date")?.date;
    const returnDate = tpFlightForm.watch("return-date")?.date;
    if (!returnDate) {
      return "Lütfen dönüş tarihi seçiniz.";
    }

    if (entryDate && returnDate && returnDate.valueOf() < entryDate.valueOf()) {
      return "Dönüş tarihi, gidiş tarihinden sonra olmalıdır.";
    }

    return true;
  };

  const validatePassengersAndCustumer = () => {
    const passengers = tpFlightForm.watch("passengers");
    const customer = tpFlightForm.watch("customer");
    const employeeCount = passengers?.employee?.count as unknown as number;
    const customerCount = customer?.length;

    if (employeeCount !== customerCount) {
      return "Çalışan sayısına göre, kişi seçimi eşit olmalıdır.";
    }

    return true;
  };

  const validateDepartureDate = () => {
    const departureDate = tpFlightForm.watch("departure-date")?.date;
    if (!departureDate) {
      return "Lüften gidiş tarihi seçiniz.";
    }

    return true;
  };

  return (
    <div
      className={cn("grid grid-cols-1 gap-4", {
        "lg:grid-cols-4": isB2E,
        "lg:grid-cols-3": isB2C,
      })}
    >
      <OverlayWrapper>
        <TpFlightSearchWrappedFormItem
          placeHolder={t("From")}
          name="origin"
          rules={{ required: requiredRule }}
        />
      </OverlayWrapper>
      <OverlayWrapper>
        <TpFlightSearchWrappedFormItem
          placeHolder={t("To")}
          name="departure"
          rules={{ required: requiredRule }}
        />
      </OverlayWrapper>
      <OverlayWrapper>
        <TpDatePickerWrappedFormItem
          placeholder={t("Departure Date")}
          name="departure-date"
          rules={{ validate: validateDepartureDate }}
        />
      </OverlayWrapper>
      <OverlayWrapper>
        <TpDatePickerWrappedFormItem
          placeholder={t("Return Date")}
          name="return-date"
          rules={{ validate: validateReturnDate }}
        />
      </OverlayWrapper>
      <OverlayWrapper>
        <TpCabinSelectWrappedFormItem name="passengers" />
      </OverlayWrapper>
      {isB2E && (
        <div className="col-span-1 flex flex-col gap-3 lg:col-span-2 lg:flex-row">
          <div className="flex-1 shrink-0">
            <CustomerSelectWrappedFormItem
              name="customer"
              rules={{ validate: validatePassengersAndCustumer }}
            />
          </div>
          <TpBranchOfficeDropdownWrappedFormItem
            name="branchOffice"
            rules={{ required: requiredRule }}
          />
          <div className="flex-1 shrink-0">
            <TpReasonDropdownWrappedFormItem
              name="reason"
              rules={{ required: requiredRule }}
            />
          </div>
        </div>
      )}
      <div>
        <Button size="lg" variant="primary" className="size-full min-h-[54px]">
          {t("Search")}
        </Button>
      </div>
    </div>
  );
};

export default TpFlightOneWayForm;
