import { Plane } from "lucide-react";
import { useDebouncedCallback } from "use-debounce";
import { useEffectOnce } from "usehooks-ts";

import FormItem from "@/components/form/form-item";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import TpSearch from "@/components/widget/travel-planner/components/tp-search/tp-search";
import TpValidationMessage from "@/components/widget/travel-planner/components/tp-validation-message/tp-validation-message";
import { useTpSearchMutation } from "@/services/jolly-api/jolly.services";

export interface ITpFlightSearchValue {
  id: number;
  value: string;
  type: number;
}

interface TpFlightSearchProps {
  value?: ITpFlightSearchValue;
  onChange: (value: ITpFlightSearchValue | null) => void;
  placeHolder: string;
  errorMessage?: string;
}

export const TpFlightSearch = (props: TpFlightSearchProps) => {
  const flightSearchMutation = useTpSearchMutation();
  const debounceMutate = useDebouncedCallback((query) => {
    flightSearchMutation.mutate({
      type: "FlightPlanner",
      query,
    });
  }, 800);
  const data = flightSearchMutation.data?.data
    ? (flightSearchMutation.data?.data
        ?.map((item) => ({
          ...item,
          key: item.id.toString(),
        }))
        .filter((item) => item.value !== "") ?? [])
    : [];

  const isLoading =
    flightSearchMutation.isLoading || flightSearchMutation.isIdle;

  useEffectOnce(() => {
    if (props.value) {
      if (flightSearchMutation.status === "idle") {
        debounceMutate(props.value.value);
      }
    }
  });

  return (
    <div className="relative">
      <TpSearch
        defaultQuery={props.value?.value}
        data={data}
        onChangeQuery={(query) => {
          if (!query) {
            props.onChange?.(null);
          }
          flightSearchMutation.reset();
          if (query.length > 2) {
            debounceMutate(query);
          }
        }}
        placeholder={props.placeHolder}
        leftIcon={<LIcon name="Plane" />}
        onSelect={(item) => {
          props.onChange({
            id: item.id,
            value: item.value,
            type: item.destinationType,
          });
        }}
        isLoading={isLoading}
        renderItem={(item) => (
          <div className="flex items-center gap-2 p-3 px-4">
            <Plane className="w-5 shrink-0" />
            <span className="text-sm">{item.value}</span>
          </div>
        )}
      />
      {props.errorMessage && (
        <TpValidationMessage message={props.errorMessage} />
      )}
    </div>
  );
};

export const TpFlightSearchWrappedFormItem = (props: {
  name: string;
  placeHolder: string;
  rules?: any;
}) => {
  return (
    <FormItem name={props.name} rules={props.rules}>
      {({ field, setValue, fieldState }) => (
        <TpFlightSearch
          placeHolder={props.placeHolder}
          value={field.value}
          onChange={(value) => {
            setValue(value);
          }}
          errorMessage={fieldState.error?.message}
        />
      )}
    </FormItem>
  );
};
