import { Home, LayoutGrid, MapPin } from "lucide-react";
import { useDebouncedCallback } from "use-debounce";
import { useEffectOnce } from "usehooks-ts";

import FormItem from "@/components/form/form-item";
import ListTile from "@/components/ui/list-tile/list-tile";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import TpSearch from "@/components/widget/travel-planner/components/tp-search/tp-search";
import TpValidationMessage from "@/components/widget/travel-planner/components/tp-validation-message/tp-validation-message";
import { useApp } from "@/hooks/useApp";
import { useLocale } from "@/hooks/useLocale";
import { useTpSearchMutation } from "@/services/jolly-api/jolly.services";

export interface ITpHotelSearchValue {
  id: number;
  value: string;
  destinationBreadCrumb: string;
  typeName: string;
  type: number;
}

interface TpHotelSearchProps {
  value?: ITpHotelSearchValue;
  onChange?: (value: ITpHotelSearchValue | null) => void;
  errorMessage?: string;
}

const ListTileIcons = {
  Bölge: <MapPin />,
  Otel: <Home />,
  Kategori: <LayoutGrid />,
};

export const TpHotelSearch = (props: TpHotelSearchProps) => {
  const { isB2C } = useApp();
  const { t } = useLocale();
  const hotelSearchMutation = useTpSearchMutation();
  const debounceMutate = useDebouncedCallback((query) => {
    hotelSearchMutation.mutate({
      type: "HotelPlanner",
      query,
    });
  }, 800);
  const data = hotelSearchMutation.data?.data
    ? (hotelSearchMutation.data?.data
        ?.map((item) => ({
          ...item,
          key: item.id.toString(),
        }))
        .filter((item) => item.value !== "") ?? [])
    : [];

  const isLoading = hotelSearchMutation.isLoading || hotelSearchMutation.isIdle;

  useEffectOnce(() => {
    if (props.value) {
      if (hotelSearchMutation.status === "idle") {
        hotelSearchMutation.mutate({
          type: "HotelPlanner",
          query: props.value.value,
        });
      }
    }
  });

  function getTrailingText(key: string) {
    switch (key) {
      case "Bölge":
        return t("Region");
      case "Otel":
        return t("Hotel");
      case "Kategori":
        return t("Category");
      default:
        return t("Hotel");
    }
  }

  return (
    <div className="relative">
      <TpSearch
        placeholder={isB2C ? t("Where are you going?") : "Seyahat Yeri"}
        defaultQuery={props.value?.value}
        data={data}
        onChangeQuery={(query) => {
          if (!query) {
            props.onChange?.(null);
          }
          hotelSearchMutation.reset();
          if (query.length > 2) {
            debounceMutate(query);
          }
        }}
        leftIcon={<LIcon name="Search" />}
        onSelect={(item) => {
          props.onChange?.({
            id: item.id,
            value: item.value,
            destinationBreadCrumb: item.destinationBreadCrumb,
            typeName: item.typeName,
            type: item.destinationType,
          });
        }}
        isLoading={isLoading}
        renderItem={(item) => (
          <ListTile
            className="w-full"
            title={item.value}
            leading={
              ListTileIcons[item.typeName as keyof typeof ListTileIcons] ??
              ListTileIcons.Otel
            }
            subTitle={item.destinationBreadCrumb}
            trailing={getTrailingText(item.typeName)}
          />
        )}
      />
      {props.errorMessage && (
        <TpValidationMessage message={props.errorMessage} />
      )}
    </div>
  );
};

export const TpHotelSearchWrappedFormItem = (props: {
  name: string;
  rules?: any;
}) => {
  return (
    <FormItem name={props.name} rules={props.rules}>
      {({ field, setValue, fieldState }) => (
        <TpHotelSearch
          value={field.value}
          onChange={(value) => {
            setValue(value);
          }}
          errorMessage={fieldState.error?.message}
        />
      )}
    </FormItem>
  );
};
