import { MapPin, Plane, Truck } from "lucide-react";
import React from "react";
import { useDebouncedCallback } from "use-debounce";
import { useEffectOnce } from "usehooks-ts";

import FormItem from "@/components/form/form-item";
import ListTile from "@/components/ui/list-tile/list-tile";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import TpSearch from "@/components/widget/travel-planner/components/tp-search/tp-search";
import TpValidationMessage from "@/components/widget/travel-planner/components/tp-validation-message/tp-validation-message";
import { useTpSearchMutation } from "@/services/jolly-api/jolly.services";

export interface ITpFlightSearchValue {
  id: number;
  value: string;
  type: number;
  typeName?: string;
}

interface TpFlightSearchProps {
  value?: ITpFlightSearchValue;
  onChange: (value: ITpFlightSearchValue | null) => void;
  placeHolder: string;
  errorMessage?: string;
}

const ListTileIcons = {
  Bölge: <MapPin />,
  Otogar: <Truck />,
  Havaalanı: <Plane />,
};

export const TpRentACarSearch = (props: TpFlightSearchProps) => {
  const transferSearchMutation = useTpSearchMutation();
  const debounceMutate = useDebouncedCallback((query) => {
    transferSearchMutation.mutate({
      type: "CarRentalPlanner",
      query,
    });
  }, 800);
  const data = transferSearchMutation.data?.data
    ? (transferSearchMutation.data?.data
        ?.map((item) => ({
          ...item,
          key: item.id.toString(),
        }))
        .filter((item) => item.value !== "") ?? [])
    : [];

  const isLoading =
    transferSearchMutation.isLoading || transferSearchMutation.isIdle;

  useEffectOnce(() => {
    if (props.value) {
      if (transferSearchMutation.status === "idle") {
        debounceMutate(props.value.value);
      }
    }
  });

  return (
    <div className="relative">
      <TpSearch
        defaultQuery={props.value?.value}
        data={data}
        onChangeQuery={(query) => {
          if (!query) {
            props.onChange?.(null);
          }
          transferSearchMutation.reset();
          if (query.length > 2) {
            debounceMutate(query);
          }
        }}
        placeholder={props.placeHolder}
        leftIcon={<LIcon name="MapPin" />}
        onSelect={(item) => {
          props.onChange({
            id: item.id,
            value: item.value,
            type: item.destinationType,
            typeName: item.typeName,
          });
        }}
        isLoading={isLoading}
        renderItem={(item) => (
          <ListTile
            className="w-full"
            title={item.value}
            leading={
              ListTileIcons[item.typeName as keyof typeof ListTileIcons] ??
              ListTileIcons.Otogar
            }
            subTitle={item.destinationBreadCrumb}
            trailing={item.typeName}
          />
        )}
      />
      {props.errorMessage && (
        <TpValidationMessage message={props.errorMessage} />
      )}
    </div>
  );
};
export const TpRentACarSearchWrappedFormItem = (props: {
  name: string;
  placeHolder: string;
  rules?: any;
}) => {
  return (
    <FormItem name={props.name} rules={props.rules}>
      {({ field, setValue, fieldState }) => (
        <TpRentACarSearch
          placeHolder={props.placeHolder}
          value={field.value}
          onChange={(value) => {
            setValue(value);
          }}
          errorMessage={fieldState.error?.message}
        />
      )}
    </FormItem>
  );
};
