/* eslint-disable no-nested-ternary */
import React from "react";
import { useBoolean } from "usehooks-ts";

import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import {
  Popover,
  PopoverContent,
  PopoverTrigger,
} from "@/components/ui/popover/popover";
import TriggerButton from "@/components/ui/trigger-button/trigger-button";
import GroupsCustomersCounter from "@/components/widget/travel-planner/components/customer-group-select/components/rooms-and-customers-counter";
import TpBottomBox from "@/components/widget/travel-planner/components/tp-bottom-box/tp-bottom-box";
import { useLabels } from "@/hooks/useLabels";
import { useLocale } from "@/hooks/useLocale";

interface TpTourPassengerSelectProps {
  value: any;
  tourQuota: number;
  onChange: (value: any) => void;
  onAcceptClick?: VoidFunction;
  renderTriggerElement?: (value: any) => React.ReactNode;
}

interface Group {
  guest?: {
    count: number;
  };
  child?: {
    count: number;
    ages?: number[];
  };
  employee?: {
    count: number;
  };
}

export const TpTourPassengerSelect = ({
  value,
  tourQuota,
  onChange,
  onAcceptClick,
  renderTriggerElement,
}: TpTourPassengerSelectProps) => {
  const { value: isOpen, setValue: setIsOpen } = useBoolean();
  const { guest } = useLabels();
  const { t } = useLocale();

  const totalPeople = value?.reduce(
    (total: number, { guest: guestParam, child, employee }: Group) =>
      total +
      (guestParam?.count || 0) +
      (child?.count || 0) +
      (employee?.count || 0),
    0,
  );

  const label =
    value?.guest?.count || value?.child?.count
      ? [
          value?.guest?.count && `${value?.guest.count} ${guest?.label}`,
          value?.child?.count && `${value?.child.count} ${t("Child")}`,
        ]
          .filter(Boolean)
          .join(", ")
      : "Kişi Seçin";

  const handleNewGroup = () =>
    onChange([
      ...value,
      { guest: { count: 2 }, child: { count: 0, ages: [] } },
    ]);

  const handleRemoveGroup = (index: number) => {
    const newValue = [...value];
    newValue?.splice(index, 1);
    onChange(newValue);
  };

  const roomWarnings = value?.map((group: Group) => {
    const isGuestCountOverLimit = (group?.guest?.count || 0) > 3;
    const isChildCountOverLimit = (group?.child?.count || 0) > 2;

    return isGuestCountOverLimit || isChildCountOverLimit;
  });

  const hasRoomWarning = roomWarnings?.some((warning: boolean) => warning);

  const warningMessage =
    totalPeople > tourQuota
      ? `Bu tarih için ${tourQuota} kişilik yer kaldı. Talep oluşturabilir veya kontenjan olan diğer tarih seçeneklerini inceleyebilirsiniz.`
      : hasRoomWarning
        ? "Sitemizden 1 oda için en fazla “3 yetişkin, 2 çocuk” seçimi yapabilirsiniz. Daha fazla kişi için oda ekleyerek ilerleyebilirsiniz."
        : "";

  const warningType = totalPeople > tourQuota ? "error" : "warning";

  return (
    <Popover open={isOpen} onOpenChange={setIsOpen}>
      <PopoverTrigger className="group w-full">
        {renderTriggerElement?.(value) ?? (
          <TriggerButton
            isFocus={isOpen}
            size="lg"
            fullWidth
            className="w-full group-data-[state=open]:border-primary-200 group-data-[state=open]:ring-2 group-data-[state=open]:ring-primary-50"
            label="Kişi Bilgisi"
            leftIcon={<LIcon name="User" />}
            iconHiddenOnMobile
            value={label}
          />
        )}
      </PopoverTrigger>
      <PopoverContent className="w-full p-0">
        <GroupsCustomersCounter
          value={value}
          onChange={onChange}
          onNewGroup={handleNewGroup}
          onRemoveGroup={handleRemoveGroup}
          items={() => [
            {
              label: guest.label,
              name: "guest",
              min: 1,
              max: 9,
            },
            {
              label: t("Child"),
              name: "child",
              min: 0,
              max: 9,
              hasAge: true,
              agesLabel: (ageIndex: number) =>
                `${ageIndex + 1}${t(". child age")}`,
            },
          ]}
          warning={totalPeople > tourQuota || hasRoomWarning}
          warningMessage={warningMessage}
          warningType={warningType}
          maxGroupCount={2}
        />
        <TpBottomBox
          buttonDisabled={totalPeople > tourQuota || hasRoomWarning}
          onApplyClick={() => {
            setIsOpen(false);
            onAcceptClick?.();
          }}
          label={t("{{roomCount}} room, {{travelCount}} travellers", {
            roomCount: value?.length,
            travelCount: totalPeople,
          })}
        />
      </PopoverContent>
    </Popover>
  );
};
