import { Home, LayoutGrid, MapPin } from "lucide-react";
import { useDebouncedCallback } from "use-debounce";
import { useEffectOnce } from "usehooks-ts";

import FormItem from "@/components/form/form-item";
import ListTile from "@/components/ui/list-tile/list-tile";
import LIcon from "@/components/ui/lucid-icon/lucid-icon";
import TpSearch from "@/components/widget/travel-planner/components/tp-search/tp-search";
import TpValidationMessage from "@/components/widget/travel-planner/components/tp-validation-message/tp-validation-message";
import { useTpSearchMutation } from "@/services/jolly-api/jolly.services";

export interface ITpTourSearchValue {
  id: number;
  value: string;
  destinationBreadCrumb: string;
  typeName: string;
  type: number;
}

interface TpTourSearchProps {
  value?: ITpTourSearchValue;
  onChange?: (value: ITpTourSearchValue | undefined) => void;
  errorMessage?: string;
  title?: string;
  label?: string;
  isDomestic?: boolean;
}

const ListTileIcons = {
  Bölge: <MapPin />,
  Otel: <Home />,
  Kategori: <LayoutGrid />,
};

export const TpTourSearch = (props: TpTourSearchProps) => {
  const tourSearchMutation = useTpSearchMutation();
  const debounceMutate = useDebouncedCallback((query) => {
    const searchType = props.isDomestic
      ? "DomesticTourPlanner"
      : "AbroadTourPlanner";
    tourSearchMutation.mutate({
      type: searchType,
      query,
    });
  }, 800);
  const data = tourSearchMutation?.data?.data
    ? (tourSearchMutation.data?.data
        ?.map?.((item) => ({
          ...item,
          key: item.id.toString(),
        }))
        .filter((item) => item.value !== "") ?? [])
    : [];

  const isLoading = tourSearchMutation.isLoading || tourSearchMutation.isIdle;

  useEffectOnce(() => {
    if (props.value) {
      if (tourSearchMutation.status === "idle") {
        const searchType = props.isDomestic
          ? "DomesticTourPlanner"
          : "AbroadTourPlanner";
        tourSearchMutation.mutate({
          type: searchType,
          query: props.value.value,
        });
      }
    }
  });

  return (
    <div className="relative">
      <TpSearch
        title={props.title}
        placeholder={props.label}
        defaultQuery={props.value?.value}
        data={data}
        onChangeQuery={(query) => {
          tourSearchMutation.reset();
          if (query.length > 2) {
            debounceMutate(query);
          } else if (!query) {
            props.onChange?.(undefined);
          }
        }}
        leftIcon={<LIcon name="Search" />}
        onSelect={(item) => {
          props.onChange?.({
            id: item.id,
            value: item.value,
            destinationBreadCrumb: item.destinationBreadCrumb,
            typeName: item.typeName,
            type: item.destinationType,
          });
        }}
        isLoading={isLoading}
        renderItem={(item) => (
          <ListTile
            className="w-full"
            title={item.value}
            leading={
              ListTileIcons[item.typeName as keyof typeof ListTileIcons] ??
              ListTileIcons.Otel
            }
            subTitle={item.destinationBreadCrumb}
            trailing={item.typeName}
          />
        )}
      />
      {props.errorMessage && (
        <TpValidationMessage message={props.errorMessage} />
      )}
    </div>
  );
};

export const TpTourSearchWrappedFormItem = (props: {
  name: string;
  title?: string;
  label?: string;
  rules?: any;
  isDomestic: boolean;
}) => {
  return (
    <FormItem name={props.name} rules={props.rules}>
      {({ field, setValue, fieldState }) => (
        <TpTourSearch
          title={props.title}
          label={props.label}
          value={field.value}
          isDomestic={props.isDomestic}
          onChange={(value) => {
            setValue(value);
          }}
          errorMessage={fieldState.error?.message}
        />
      )}
    </FormItem>
  );
};
