/* eslint-disable react/jsx-no-useless-fragment */
/* eslint-disable no-prototype-builtins */

"use client";

import dayjs from "dayjs";
import { Search } from "lucide-react";
import { usePathname } from "next/navigation";
import { useState } from "react";
import { useForm } from "react-hook-form";
import { useBoolean, useMediaQuery } from "usehooks-ts";

import Form from "@/components/form/form";
import Card from "@/components/ui/card/card";
import { Dialog, DialogContent } from "@/components/ui/dialog";
import {
  Tabs,
  TabsContent,
  TabsList,
  TabsTrigger,
} from "@/components/ui/tabs/tabs";
import type { RoomAndCustomerCounterValue } from "@/components/widget/travel-planner/components/customer-group-select/components/room-and-customer-counter";
import type { FlightCustomerCabinTypeValue } from "@/components/widget/travel-planner/components/flight-customer-group-select/flight-customer-group-select";
import type { ITpDatePickerValue } from "@/components/widget/travel-planner/components/tp-date-picker/tp-date-picker.types";
import { useFlightSearch } from "@/features/flight/hooks/use-flight-search";
import { useApp } from "@/hooks/useApp";
import { useBreakpoints } from "@/hooks/useBreakpoints";
import { useLocale } from "@/hooks/useLocale";
import { useLocalStateCache } from "@/libs/helpers/local-state-cache-helper";
import { localeStorageHelper } from "@/libs/helpers/locale-storage-helper";
import { cn } from "@/libs/utils";

import TpFlightOneWayForm from "./components/tp-flight-one-way-form";
import TpFlightRoundTripForm from "./components/tp-flight-round-trip-form";
import type { ITpFlightSearchValue } from "./components/tp-flight-search";

export interface ITravelPlannerForm {
  origin: ITpFlightSearchValue;
  departure: ITpFlightSearchValue;
  "departure-date": ITpDatePickerValue;
  "return-date": ITpDatePickerValue;
  passengers: {
    employee: RoomAndCustomerCounterValue;
    guest: RoomAndCustomerCounterValue;
    child: RoomAndCustomerCounterValue;
    cabinType: FlightCustomerCabinTypeValue;
  };
  customer: { name: string; value: string }[];
  flightTripKind: string;
}

interface TravelPlannerFormProps {
  hideShadow?: boolean;
  defaultValues: ITravelPlannerForm;
}

function TpFlightMobileForm({
  children,
  tpPlaceholderText,
}: {
  children: React.ReactNode;
  tpPlaceholderText: string;
}) {
  const modalBoolean = useBoolean();

  return (
    <>
      <div className="flex items-center justify-between rounded-lg border bg-white p-3">
        <div>
          <button
            className="text-sm font-medium text-gray-500"
            type="submit"
            onClick={modalBoolean.setTrue}
          >
            {tpPlaceholderText}
          </button>
        </div>

        <Search className="text-gray-500" />
      </div>

      <Dialog
        modal
        open={modalBoolean.value}
        onOpenChange={modalBoolean.setValue}
      >
        <DialogContent className="size-full">{children}</DialogContent>
      </Dialog>
    </>
  );
}

function TpFlightForm({ hideShadow, defaultValues }: TravelPlannerFormProps) {
  const { isDesktop } = useBreakpoints();
  const { t } = useLocale();
  const tpFlightForm = useForm<ITravelPlannerForm>({
    defaultValues: {
      ...defaultValues,
    },
  });
  const { isB2E } = useApp();
  const { setSelectedEmployees } = useLocalStateCache();
  const [activeTab, setActiveTab] = useState(defaultValues?.flightTripKind);
  const { setVariables } = useFlightSearch();

  return (
    <Card
      className={cn({
        "shadow-2xl": !hideShadow,
        "border-0": !isDesktop,
        "z-[51]": isDesktop,
      })}
    >
      <Form
        methods={tpFlightForm}
        onSubmit={(values: ITravelPlannerForm) => {
          localeStorageHelper.set("travel-planner", {
            ...values,
            flightTripKind: values.flightTripKind,
          });
          if (isB2E)
            setSelectedEmployees(
              [
                {
                  employee: {
                    count: values.passengers.employee
                      .count as unknown as number,
                  },
                },
              ],
              values.customer,
            );

          setVariables({
            searchLegs: [
              {
                originId: values.origin.id,
                originName: values.origin.value,
                originType: Number(values.origin.type),
                departureId: Number(values.departure.id),
                departureName: values.departure.value,
                departureType: Number(values.departure.type),
                departureTime: values["departure-date"].date!,
                cabinClass: Number(values.passengers.cabinType.value),
                cabinClassTitle: values.passengers.cabinType
                  .title as unknown as string,
                returnTime: values["return-date"]?.date
                  ? values["return-date"].date
                  : values["departure-date"].date,
              },
            ],
            flightTripKind: values.flightTripKind,
            passenger: Object.values(values.passengers)
              .filter((item) => item?.hasOwnProperty("count"))
              .map((item) => item.count)
              .join(","),
          });
        }}
      >
        <Tabs
          defaultValue={activeTab}
          onValueChange={(value) => {
            tpFlightForm.setValue("flightTripKind", value);
            setActiveTab(value);
          }}
        >
          <TabsList>
            <TabsTrigger
              className="mr-2 rounded-primary border border-transparent data-[state=active]:!border data-[state=active]:!border-primary-500 data-[state=active]:text-primary-500"
              value="2"
            >
              {t("Round Trip")}
            </TabsTrigger>
            <TabsTrigger
              className="rounded-primary border border-transparent data-[state=active]:!border data-[state=active]:!border-primary-500 data-[state=active]:text-primary-500"
              value="1"
            >
              {t("One Way")}
            </TabsTrigger>
          </TabsList>
          <TabsContent value="1">
            <TpFlightOneWayForm tpFlightForm={tpFlightForm} />
          </TabsContent>
          <TabsContent value="2">
            <TpFlightRoundTripForm tpFlightForm={tpFlightForm} />
          </TabsContent>
        </Tabs>
      </Form>
    </Card>
  );
}

const TravelPlannerFlightForm = ({
  hideShadow,
  defaultValues,
}: TravelPlannerFormProps) => {
  const { t } = useLocale();
  const pathName = usePathname();
  const isDesktop = useMediaQuery("(min-width: 768px)");
  const employeCount = defaultValues?.passengers?.employee?.count ?? 0;
  const customerCount = defaultValues?.passengers?.guest?.count ?? 0;
  const childCount = defaultValues?.passengers?.child?.count ?? 0;
  const totalPassengers =
    (employeCount as number) +
    (customerCount as number) +
    (childCount as number);
  const cabinType = defaultValues?.passengers?.cabinType?.title;
  const departureDate = dayjs(defaultValues?.["departure-date"]?.date).format(
    "DD MMM ddd",
  );
  const flightTripKind = defaultValues?.flightTripKind;
  const returnDate = dayjs(defaultValues?.["return-date"]?.date).format(
    "DD MMM ddd",
  );
  const tpPlaceholderText = `${departureDate} ${flightTripKind === "2" ? `- ${returnDate}` : ""}, ${totalPassengers} ${t("Passenger")}, ${t(cabinType as unknown as string)}`;

  return (
    <>
      {isDesktop || pathName !== "/flight-list" ? (
        <TpFlightForm defaultValues={defaultValues} hideShadow={hideShadow} />
      ) : (
        <TpFlightMobileForm tpPlaceholderText={tpPlaceholderText}>
          <TpFlightForm defaultValues={defaultValues} hideShadow={hideShadow} />
        </TpFlightMobileForm>
      )}
    </>
  );
};

export default TravelPlannerFlightForm;
