"use client";

import { useForm } from "react-hook-form";

import Form from "@/components/form/form";
import { Button } from "@/components/ui/button/button";
import Card from "@/components/ui/card/card";
import type { RoomAndCustomerCounterValue } from "@/components/widget/travel-planner/components/customer-group-select/components/room-and-customer-counter";
import type { ITpDateRangePickerValue } from "@/components/widget/travel-planner/components/tp-date-range-picker/tp-date-range-picker.types";
import { useHotelTravelPlanner } from "@/features/modules/hotel/travel-planner/hooks/use-hotel-travel-planner";
import { useLocale } from "@/hooks/useLocale";
import { DateHelper } from "@/libs/helpers/date-time-helper";
import { localeStorageHelper } from "@/libs/helpers/locale-storage-helper";
import { cn } from "@/libs/utils";
import { requiredRule } from "@/validations/form-rules/rules/required-rule";

import type { IBranchOfficeItem } from "./components/tp-branch-office-dropdown-form";
import { CustomerSelectWrappedFormItem } from "./components/tp-customer-select-form";
import { TpDateRangePickerWrappedFormItem } from "./components/tp-date-range-picker-form";
import type { ITpHotelSearchValue } from "./components/tp-hotel-search";
import { TpHotelSearchWrappedFormItem } from "./components/tp-hotel-search";
import type { IReasonItem } from "./components/tp-reason-dropdown-form";
import { TpReasonDropdownWrappedFormItem } from "./components/tp-reason-dropdown-form";
import { TpRoomSelectWrappedFormItem } from "./components/tp-room-select-form";

export interface ITravelPlannerForm {
  hotel: ITpHotelSearchValue;
  "date-range": ITpDateRangePickerValue;
  room: RoomAndCustomerCounterValue[];
  customer: { name: string; value: string }[];
  reason: IReasonItem;
  branchOffice?: IBranchOfficeItem;
}

interface TravelPlannerFormProps {
  hideShadow?: boolean;
  defaultValues: ITravelPlannerForm;
}

const TravelPlannerForm = ({
  hideShadow,
  defaultValues,
}: TravelPlannerFormProps) => {
  const tpHotelForm = useForm<ITravelPlannerForm>({
    defaultValues,
  });
  const { navigateToHotelList } = useHotelTravelPlanner();
  const { t } = useLocale();
  const validatePassengersAndCustumer = () => {
    const passengers = tpHotelForm.watch("room");
    const customer = tpHotelForm.watch("customer");
    const employeeCount = passengers
      .map((item) => item.employee?.count)
      .reduce((a, b) => {
        if (a === undefined || b === undefined) {
          return 0;
        }
        return a + b;
      }, 0);
    const customerCount = customer?.length;

    if (employeeCount !== customerCount) {
      return "Çalışan sayısına göre, kişi seçimi eşit olmalıdır.";
    }

    return true;
  };

  const validateDateRange = () => {
    const dateRangeFrom = tpHotelForm.watch("date-range").from;
    const dateRangeTo = tpHotelForm.watch("date-range").to;
    if (!dateRangeFrom || !dateRangeTo) {
      return t("This field is required");
    }

    return true;
  };

  return (
    <Card className={cn(!hideShadow && "shadow-2xl")}>
      <Form
        methods={tpHotelForm}
        onSubmit={(values: ITravelPlannerForm) => {
          localeStorageHelper.set("travel-planner", {
            ...values,
          });
          navigateToHotelList({
            employees: values.customer,
            rooms: values.room.map((item) => ({
              child: item.child,
              employee: item.employee,
              guest: item.guest,
            })),
            destination: values.hotel,
            dates: {
              checkIn: DateHelper.dateToISOString(values["date-range"].from!),
              checkOut: DateHelper.dateToISOString(values["date-range"].to!),
            },
            reason: values.reason,
          });
        }}
      >
        <div className="grid grid-cols-1 gap-3 lg:grid-cols-3">
          <div>
            <TpHotelSearchWrappedFormItem
              name="hotel"
              rules={{ required: requiredRule }}
            />
          </div>
          <div>
            <TpDateRangePickerWrappedFormItem
              name="date-range"
              rules={{ validate: validateDateRange }}
            />
          </div>
          <div className="grid grid-cols-2 gap-4 md:contents">
            <TpRoomSelectWrappedFormItem
              checkInDate={tpHotelForm.watch("date-range")?.from}
              name="room"
            />

            <CustomerSelectWrappedFormItem
              name="customer"
              rules={{ validate: validatePassengersAndCustumer }}
            />
          </div>
          <div>
            <TpReasonDropdownWrappedFormItem
              name="reason"
              rules={{ required: requiredRule }}
            />
          </div>
          <div>
            <Button
              size="lg"
              variant="primary"
              className="size-full min-h-[54px]"
            >
              {`${t("Hotel")} ${t("Search")}`}
            </Button>
          </div>
        </div>
      </Form>
    </Card>
  );
};

export default TravelPlannerForm;
