import { useState } from "react";
import { useQuery } from "react-query";
import { useUpdateEffect } from "usehooks-ts";

import type { MutationOptions } from "@/@types/types/query.types";
import type { RangeValue } from "@/components/ui/range-slider/range-slider";
import { DUMMY_HOTEL_LIST_REQUEST } from "@/data/dummy/hotel-list-request";
import { useHotelSearch } from "@/features/hotel/hooks/use-hotel-search";
import { usePageInfo } from "@/hooks/use-page-info/use-page-info";
import { useHttp } from "@/hooks/useHttp";
import { useLocale } from "@/hooks/useLocale";
import { setParametersToStrategyRunner } from "@/libs/helpers/strategy-service.helper";
import { useServerDataCtx } from "@/providers/server-data-provider/server-data-provider";

import type { IHotelSearchResultSerializer } from "./hotel-search-services.serializer";
import { hotelSearchResultSerializer } from "./hotel-search-services.serializer";
import type {
  IHotelListItem,
  IHotelSearchRequest,
  IHotelSearchRequestFilter,
} from "./hotel-search-services.types";

interface ListedDataItem {
  filter: Pick<IHotelSearchRequest, "pageNumber">;
  listData: IHotelListItem[];
}

interface HotelListSearcQueryProps {
  variables?: IHotelSearchRequest;
  queryOptions?: MutationOptions<IHotelSearchResultSerializer>;
}

export const useHotelListSearchQuery = ({
  variables,
  queryOptions,
}: HotelListSearcQueryProps = {}) => {
  const { strategyKeys } = useServerDataCtx();
  const http = useHttp("wl-display-api");
  const [listedData, setListedData] = useState<ListedDataItem[]>([]);
  const { info } = usePageInfo();
  const { getVariables, getRoom } = useHotelSearch();
  const { rooms } = getVariables()!;
  const [requestVariables, setRequestVariables] = useState(
    variables || DUMMY_HOTEL_LIST_REQUEST,
  );
  const [memRequestVariables, setMemRequestVariables] = useState(
    variables || DUMMY_HOTEL_LIST_REQUEST,
  );
  const { setQueryParams } = usePageInfo();
  const { getCultureCode } = useLocale();
  const [isFilterApplicable, setIsFilterApplicable] = useState(true);

  const service = async (hotelSearchRequest: IHotelSearchRequest) =>
    http
      .post(
        `/api/v1/strategy/run?culture=${getCultureCode()}`,
        setParametersToStrategyRunner(
          {
            hotelSearchRequest,
          },
          strategyKeys.hotelList,
        ),
      )
      .then((res) => hotelSearchResultSerializer(res.data));

  const query = useQuery(
    ["hotel-search", requestVariables],
    () => service({ ...requestVariables }),
    {
      ...queryOptions,
      refetchOnMount: false,
      refetchOnWindowFocus: false,
    },
  );

  const setFilterValue = (
    key: keyof IHotelSearchRequest,
    value: unknown,
    requestParams: Pick<IHotelSearchRequest, "searchId" | "requestKey">,
    setUrl: boolean = true,
  ) => {
    setRequestVariables({
      ...requestVariables,
      searchId: query.data?.metadata.searchId || requestParams.searchId,
      requestKey: query.data?.metadata.requestKey || requestParams.requestKey,
      [key]: value,
    });
    if (setUrl) setQueryParams({ [key]: value as string });
  };

  const setQueryData = (args: IHotelSearchRequest) => setRequestVariables(args);

  /**
   * @description View more event
   */
  const getMoreData = (
    requestParams: Pick<IHotelSearchRequest, "searchId" | "requestKey">,
  ) => {
    setFilterValue(
      "pageNumber",
      requestVariables.pageNumber + 1,
      requestParams,
      false,
    );
  };

  /**
   *
   * @description Apply someone filter to datalist
   */
  const applyFilter = <K extends keyof IHotelSearchRequestFilter>(
    key: keyof IHotelSearchRequestFilter,
    value: IHotelSearchRequestFilter[K],
  ) => {
    const newFilter: Partial<
      Record<keyof IHotelSearchRequestFilter, IHotelSearchRequestFilter[K]>
    > = {
      [key]: value,
    };

    setRequestVariables((prev) => {
      const values = {
        ...memRequestVariables,
        searchId: query.data?.metadata.searchId || prev.searchId,
        requestKey: null,
        pageNumber: 1,
        filter: {
          ...memRequestVariables.filter,
          ...newFilter,
        },
      };

      setMemRequestVariables(values);

      if (isFilterApplicable) {
        return values;
      }

      return prev;
    });
    setListedData([]);
    window.scrollTo(0, 0);
  };

  const handleIsApplicable = (
    filterParameters?:
      | {
          filterKey: keyof IHotelSearchRequestFilter;
          filterValue: string;
        }
      | boolean,
  ) => {
    if (filterParameters !== false) {
      setIsFilterApplicable(true);
      setTimeout(() => {
        if (filterParameters && typeof filterParameters !== "boolean") {
          applyFilter(filterParameters.filterKey, filterParameters.filterValue);
        } else if (filterParameters) {
          setRequestVariables(memRequestVariables);
          setListedData([]);
        }
      }, 500);
      return;
    }
    setIsFilterApplicable(false);
  };

  const clearFilter = (withoutPrice: boolean = true) => {
    setRequestVariables((prev) => {
      const value: IHotelSearchRequest = {
        ...requestVariables,
        searchId: query.data?.metadata.searchId,
        pageNumber: 1,
        filter: {
          hotelName: "",
          facilities: [],
          locations: [],
          mealTypes: [],
          themes: [],
        },
      };
      if (!withoutPrice) {
        value.filter.minPrice = prev.filter.minPrice;
        value.filter.maxPrice = prev.filter.maxPrice;
      }
      setMemRequestVariables(value);
      return value;
    });
    setListedData([]);
  };

  /**
   * @description Apply someone order by to datalist
   */
  const applyOrderBy = (orderBy: string) => {
    setRequestVariables((prev) => ({
      ...requestVariables,
      searchId: query.data?.metadata.searchId || prev.searchId,
      requestKey: null,
      pageNumber: 1,
      orderBy,
    }));
    setListedData([]);
  };

  /**
   * @description Apply someone price filter to datalist
   */
  const applyPriceFilter = (price: RangeValue) => {
    setRequestVariables((prev) => {
      const values = {
        ...requestVariables,
        searchId: query.data?.metadata.searchId || prev.searchId,
        requestKey: null,
        pageNumber: 1,
        filter: {
          ...requestVariables.filter,
          minPrice: price.start,
          maxPrice: price.end,
        },
      };

      setMemRequestVariables(values);

      if (isFilterApplicable) {
        return values;
      }

      return prev;
    });
    setListedData([]);
  };

  const mergeListData = (): Array<IHotelListItem | { key: string }> => {
    let currentDataList: IHotelListItem[] = [];
    if (
      query.data &&
      !listedData.some(
        (item) => item.filter.pageNumber === requestVariables.pageNumber,
      )
    ) {
      currentDataList = query.data?.hotelList || [];
      setListedData([
        ...listedData,
        {
          filter: { pageNumber: requestVariables.pageNumber },
          listData: currentDataList,
        },
      ]);
    }
    const beforeDataList = listedData.reduce((prev: IHotelListItem[], curr) => {
      return [...prev, ...(curr.listData || [])];
    }, []);

    return [...beforeDataList, ...currentDataList].map((item, index) => ({
      ...item,
      key: index.toString(),
    }));
  };

  useUpdateEffect(() => {
    if (info?.queryParams.destinationId) {
      setRequestVariables((prev) => ({
        ...prev,
        destinationId: Number(info?.queryParams.destinationId),
        destinationName: info?.queryParams.destination,
        checkInDate: info?.queryParams.checkinDate || prev.checkInDate,
        checkOutDate: info?.queryParams.checkoutDate || prev.checkOutDate,
        rooms: getRoom(rooms),
      }));
      setListedData([]);
    }
  }, [
    info?.queryParams.destinationId,
    info?.queryParams.checkinDate,
    info?.queryParams.checkoutDate,
    info?.queryParams.rooms,
  ]);

  return {
    ...query,
    requestVariables,
    setQueryData,
    setFilterValue,
    getMoreData,
    applyFilter,
    applyOrderBy,
    clearFilter,
    applyPriceFilter,
    /**
     * @description this is final data for main hotel list
     */
    finalData: mergeListData(),
    /**
     * @description this function applies to filter applicable when user on mobile
     */
    onIsApplicable: handleIsApplicable,
  };
};

export const usePureHotelListSearchQuery = (
  hotelSearchRequest: IHotelSearchRequest,
  queryOptions?: MutationOptions<IHotelSearchResultSerializer>,
) => {
  const { strategyKeys } = useServerDataCtx();
  const http = useHttp("wl-display-api");
  const { getCultureCode } = useLocale();
  const service = () =>
    http
      .post(
        `/api/v1/strategy/run?culture=${getCultureCode()}`,
        setParametersToStrategyRunner(
          {
            hotelSearchRequest,
          },
          strategyKeys.hotelList,
        ),
      )
      .then((res) => hotelSearchResultSerializer(res.data));

  return useQuery({
    queryFn: service,
    queryKey: ["hotel-search"],
    ...queryOptions,
  });
};

export const useHotelListService = () => {
  const { strategyKeys } = useServerDataCtx();
  const http = useHttp("wl-display-api");
  const { getCultureCode } = useLocale();

  function getHotelListService(hotelSearchRequest: IHotelSearchRequest) {
    return http
      .post(
        `/api/v1/strategy/run?culture=${getCultureCode()}`,
        setParametersToStrategyRunner(
          {
            hotelSearchRequest,
          },
          strategyKeys.hotelList,
        ),
      )
      .then((res) => hotelSearchResultSerializer(res.data));
  }

  return { getHotelListService };
};
